/***************************************************************************
                 base.h  -  Base defines for brute terrain
                             -------------------
    begin                : Mon Sep 8 2003
    copyright            : (C) 2003 by Cody Russell
    email                : cody `at' jhu.edu
 ***************************************************************************
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoBrute, base.h

 The Initial Developer of the Original Code is Cody Russell.
 Portions created by Cody Russell are Copyright (C) 2003
 Cody Russell. All Rights Reserved.
 
 ***************************************************************************/
 
#ifndef __NEOBRUTE_BASE_H
#define __NEOBRUTE_BASE_H
 
#if defined(HAVE_CONFIG_H) && defined(NEOBRUTE_INTERNALS)
#  include "buildconfig.h"
#endif

#include <neoengine/base.h>
#include <neoengine/terrain.h>

#define NEOBRUTEVERSION_MAJOR             NEOENGINEVERSION_MAJOR
#define NEOBRUTEVERSION_MINOR             NEOENGINEVERSION_MINOR
#define NEOBRUTEVERSION_REVISION          NEOENGINEVERSION_REVISION

#define NEOBRUTEVERSIONSTRING             NEOENGINEVERSIONSTRING

                                                                                
#ifdef WIN32
                                                                                
#  ifdef _MSC_VER
#    pragma warning( disable : 4231 )
#  endif

#  ifndef BUILD_STATIC

#    ifdef NEOBRUTE_EXPORTS
                                                                                
#      define NEOBRUTE_API __declspec(dllexport)
#      define NEOBRUTEEXPIMP_TEMPLATE
                                                                                
#    else
                                                                                
#      define NEOBRUTE_API __declspec(dllimport)
#      define NEOBRUTEEXPIMP_TEMPLATE extern
                                                                                
#    endif
                                                                                
#    ifdef __cplusplus
        // Instantiate class std::vector<classname> and std::list<classname>
        // This does not create an object. It only forces the generation of all
        // of the members of the class.
        // It exports them from the DLL and imports them into the .exe file.
#      if ( _MSC_VER >= 1300 )
#        define BRUTEUDTVectorEXPIMP(classname) NEOBRUTEEXPIMP_TEMPLATE template class NEOBRUTE_API std::allocator< classname >; NEOBRUTEEXPIMP_TEMPLATE template class NEOBRUTE_API std::vector< classname >
#      elif !defined( __MINGW32__ )
#        define BRUTEUDTVectorEXPIMP(classname) NEOBRUTEEXPIMP_TEMPLATE template class NEOBRUTE_API std::vector< classname >
#      else
#        define BRUTEUDTVectorEXPIMP(classname)
#      endif
#    endif
#  else

#    define NEOBRUTE_API
#    define NEOBRUTEEXPIMP_TEMPLATE
#    define BRUTEUDTVectorEXPIMP

#  endif

#elif defined(POSIX) || defined(__APPLE__)
                                                                                
#  define NEOBRUTE_API
#  define NEOBRUTE_TEMPLATE
                                                                                
#  define BRUTEUDTVectorEXPIMP(classname)
                                                                                
#else
#  error "Platform uninplemented"
#endif

namespace NeoBrute
{
#ifndef BUILD_STATIC

#ifdef __cplusplus
extern "C"
{
#endif

	/**
	* Called by the engine when the plugin is loaded
	* \return                                                  Error code
	*/
	NEOBRUTE_API int                                           Initialize();

	/**
	* Create terrain object
	* \return                                                  New terrain manager
	*/
	NEOBRUTE_API class NeoEngine::TerrainManager              *CreateTerrainManager();

	/**
	* Called by the engine when plugin is unloaded
	* \return                                                  Error code
	*/
	NEOBRUTE_API int                                           Shutdown();

#ifdef __cplusplus
}
#endif

#endif // BUILD_STATIC

};

#endif // __NEOBRUTE_BASE_H

