/***************************************************************************
         dll.cpp  -  Dynamic library entry points and init/shutdown
                             -------------------
    begin                : Mon Sep 8 2003
    copyright            : (C) 2003 by Cody Russell
    email                : cody `at' jhu.edu
 ***************************************************************************
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoBrute, dll.cpp

 The Initial Developer of the Original Code is Cody Russell.
 Portions created by Cody Russell are Copyright (C) 2003
 Cody Russell. All Rights Reserved.
 
 ***************************************************************************/

#include "base.h"
#include "link.h"
#include "terrain.h"

using namespace NeoEngine;
using namespace NeoBrute;

#if defined( WIN32 ) && !defined( BUILD_STATIC )
 
#ifndef WIN32_LEAN_AND_MEAN
#  define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
 
// Win32-specific init for dll loading
BOOL NEOBRUTE_API APIENTRY DllMain( HANDLE hModule, DWORD ulReason, LPVOID lpReserved 
)
{
        switch( ulReason )
        {
                case DLL_PROCESS_ATTACH:
                case DLL_THREAD_ATTACH:
                        break;
                                                                                
                case DLL_THREAD_DETACH:
                case DLL_PROCESS_DETACH:
                        break;
        }
                                                                                
        return TRUE;
}
                                                                                
                                                                                
#endif

namespace NeoBrute
{

#ifdef BUILD_STATIC

ModuleStatic *g_pkStaticModule = 0;

#else

extern "C"
{

#endif

NEOBRUTE_API int Initialize()
{
        return 0;
}
                                                                                
                                                                                
NEOBRUTE_API NeoEngine::TerrainManager *CreateTerrainManager()
{
        return new NeoBrute::BruteTerrainManager;
}

NEOBRUTE_API void GetVersion( std::string *pstrName, int *piMajor, int *piMinor, int *piRevision )
{
        *pstrName   = "Brute-Force Terrain Method";
        *piMajor    = NEOBRUTEVERSION_MAJOR;
        *piMinor    = NEOBRUTEVERSION_MINOR;
        *piRevision = NEOBRUTEVERSION_REVISION;
}

                                                                                
NEOBRUTE_API int Shutdown()
{
#ifdef BUILD_STATIC
	delete g_pkStaticModule;
#endif

        return 0;
}

#ifdef BUILD_STATIC

#else

}

#endif

#ifdef BUILD_STATIC

Link::Link()
{
#if defined( WIN32 ) && defined( _DEBUG )
	g_pkStaticModule = new ModuleStatic( "neobrute_debug" );
#else
	g_pkStaticModule = new ModuleStatic( "neobrute" );
#endif

	g_pkStaticModule->m_Symbols.Insert( "CreateTerrainManager", (void**)CreateTerrainManager );
	g_pkStaticModule->m_Symbols.Insert( "Initialize",           (void**)Initialize );
	g_pkStaticModule->m_Symbols.Insert( "Shutdown",             (void**)Shutdown );
	g_pkStaticModule->m_Symbols.Insert( "GetVersion",           (void**)GetVersion );

	NeoEngine::Core::GetModuleManager()->InsertModule( g_pkStaticModule );
};

#endif

};
