/***************************************************************************
                           bone.cpp  -  Bone chunk
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, bone.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "bone.h"
#include "chunktype.h"
#include "stdstring.h"
#include "integer.h"
#include "vector3d.h"
#include "quaternion.h"
#include "matrix.h"

#include <neoengine/bone.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


BoneChunk::~BoneChunk()
{
	delete m_pkBone;
}


int BoneChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	m_pkBone = new Bone;

	Chunk *pkChunk;

	//Get ID
	if( !( pkChunk = FindChunk( "id", ChunkType::INTEGER, -1 ) ) )
		neolog << LogLevel( ERROR ) << "*** Invalid bone chunk: No ID" << endl;
	else
		m_pkBone->m_uiID = dynamic_cast< IntChunk* >( pkChunk )->m_iData;

	//Get optional name
	if( ( pkChunk = FindChunk( "name", ChunkType::STRING, -1 ) ) )
		m_pkBone->SetName( dynamic_cast< StringChunk* >( pkChunk )->m_strData );

	//Get optional position
	if( ( pkChunk = FindChunk( "pos", ChunkType::VECTOR, -1 ) ) )
		m_pkBone->SetTranslation( dynamic_cast< VectorChunk* >( pkChunk )->m_kData );

	//Get optional rotation
	if( ( pkChunk = FindChunk( "rot", ChunkType::QUATERNION, -1 ) ) )
		m_pkBone->SetRotation( dynamic_cast< QuaternionChunk* >( pkChunk )->m_kData );
	else if( ( pkChunk = FindChunk( "rot", ChunkType::MATRIX, -1 ) ) )
		m_pkBone->SetRotation( Quaternion( dynamic_cast< MatrixChunk* >( pkChunk )->m_kData ) );

	return 1;
}


}; // namespace NeoChunkIO



