/***************************************************************************
                        chunk.cpp  -  Chunk base class
                             -------------------
    begin                : Tue Feb 18 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, chunk.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "chunk.h"


using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


Chunk::~Chunk()
{
	vector< Chunk* >::iterator ppkChunk = m_vpkSubChunks.begin();
	vector< Chunk* >::iterator ppkEnd   = m_vpkSubChunks.end();

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
		delete( *ppkChunk );
}



void Chunk::AttachChunk( Chunk *pkChunk, bool bReplace )
{
	if( bReplace )
	{
		vector< Chunk* >::iterator ppkSubChunk = m_vpkSubChunks.begin();
		vector< Chunk* >::iterator ppkSubEnd   = m_vpkSubChunks.end();

		for( ; ppkSubChunk != ppkSubEnd; )
		{
			if( ( (*ppkSubChunk)->m_usType == pkChunk->m_usType ) &&
			    ( !pkChunk->m_strID.length() || ( pkChunk->m_strID == (*ppkSubChunk)->m_strID ) ) )
			{
				vector< Chunk* >::iterator ppkSubSubChunk = (*ppkSubChunk)->m_vpkSubChunks.begin();
				vector< Chunk* >::iterator ppkSubSubEnd   = (*ppkSubChunk)->m_vpkSubChunks.end();
			
				for( ; ppkSubSubChunk != ppkSubSubEnd; ++ppkSubSubChunk )
					pkChunk->AttachChunk( *ppkSubSubChunk );
				
				(*ppkSubChunk)->m_vpkSubChunks.clear();
			
				delete( *ppkSubChunk );

				m_vpkSubChunks.erase( ppkSubChunk );			
			}
			else
				++ppkSubChunk;
		}
	}

	m_vpkSubChunks.push_back( pkChunk );
}


Chunk *Chunk::FindChunk( const HashString &rstrID, unsigned short usType, int iRecurse, bool bIgnoreID )
{
	if( ( bIgnoreID || ( rstrID == m_strID ) ) && ( !usType || ( usType == m_usFindType ) || ( usType == m_usType ) ) )
		return this;

	if( !iRecurse )
		return 0;

	vector< Chunk* >::iterator ppkChunk = m_vpkSubChunks.begin();
	vector< Chunk* >::iterator ppkEnd   = m_vpkSubChunks.end();

	Chunk *pkChunk;

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		if( ( pkChunk = (*ppkChunk)->FindChunk( rstrID, usType, ( iRecurse > 0 ) ? iRecurse : 0 ) ) )
			return pkChunk;
	}

	return 0;
}


int Chunk::FindChunks( const HashString &rstrID, unsigned short usType, std::vector<Chunk*> *pvpkResult, bool bRecurse, bool bIgnoreID )
{
	int iCount = 0;

	//Search our subchunks
	vector< Chunk* >::iterator ppkChunk = m_vpkSubChunks.begin();
	vector< Chunk* >::iterator ppkEnd   = m_vpkSubChunks.end();

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		if( (*ppkChunk)->FindChunk( rstrID, usType, 0, bIgnoreID ) )
		{
			pvpkResult->push_back( *ppkChunk );
			++iCount;
		}
	}
	
	if( !bRecurse )
		return iCount;

	ppkChunk = m_vpkSubChunks.begin();
	
	for( ; ppkChunk != ppkEnd; ++ppkChunk )
		iCount += (*ppkChunk)->FindChunks( rstrID, usType, pvpkResult, true, bIgnoreID );

	return iCount;
}


int Chunk::ParseRecursive( unsigned int uiFlags, FileManager *pkFileManager )
{
	int iRet = 0;

	vector< Chunk* >::iterator ppkChunk = m_vpkSubChunks.begin();
	vector< Chunk* >::iterator ppkEnd   = m_vpkSubChunks.end();

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
		if( (*ppkChunk)->m_bParse && ( ( iRet = (*ppkChunk)->ParseRecursive( uiFlags, pkFileManager ) ) < 0 ) )
			return iRet;

	return( ParseData( uiFlags, pkFileManager ) );
}



};

