/***************************************************************************
                   core.h  -  Chunk I/O core object and services
                             -------------------
    begin                : Tue Jan 20 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, core.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEOCHUNKIO_CORE_H
#define __NEOCHUNKIO_CORE_H

#include "base.h"


namespace NeoChunkIO
{


class NotifyCallback;


/**
  * \brief Core chunk I/O object
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOCHUNKIO_API Core
{
	protected:

		/*! Singleton object */
		static Core                                  *s_pkSingleton;

		/*! Notification callback */
		NotifyCallback                               *m_pkNotifyCallback;


		/**
		* Prevent new objects to be created
		*/
		                                              Core();


	public:

		/**
		*/
		                                             ~Core();


		/**
		* Initialize chunk I/O core
		* \return                                     true if successful, false if not
		*/
		bool                                          Initialize();

		/**
		* Terminate chunk I/O core, deallocated core object
		*/
		void                                          Shutdown();

		/**
		* Set notification callback object
		* \param pkCallback                           Callback object
		* \return                                     Old callback object
		*/
		inline NotifyCallback                        *SetNotifyCallback( NotifyCallback *pkCallback );

		/**
		* \return                                     Callback object
		*/
		inline NotifyCallback                        *GetNotifyCallback();

		/**
		* Get singleton object
		* \return                                     Chunk I/O core object
		*/
		inline static Core                           *Get();
};


#include "core_inl.h"


};


#endif
