/***************************************************************************
                     light.cpp  -  Light source chunk
                             -------------------
    begin                : Wed Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, light.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "light.h"
#include "boundingvolume.h"
#include "stdstring.h"
#include "vector3d.h"
#include "color.h"
#include "bool.h"
#include "float.h"

#include <neoengine/light.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{


int LightChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	SceneEntityChunk::ParseData( uiFlags, pkFileManager );

	Chunk *pkChunk = 0;
	Light  *pkLight  = 0;

	// First we need to find out the type, cast_shadows and direction for the light, as they are parameters to the constructor
	
	string strTypeString = "none";
	Light::LIGHTTYPE eType;
	
	if( ( pkChunk = FindChunk( "type", ChunkType::STRING, -1 ) ) )
		strTypeString = dynamic_cast< StringChunk* >( pkChunk )->m_strData;
	
	if ( strTypeString == "directional" ) eType = Light::DIRECTIONAL;
	else if ( strTypeString == "point" ) eType = Light::POINT;
	else if ( strTypeString == "ambient" ) eType = Light::AMBIENT;
	else
	{
		neolog << LogLevel( ERROR ) << "*** Invalid light chunk: Invalid type" << endl;
		return -1;
	}
	
	bool bCastShadows = false;	
	if( ( pkChunk = FindChunk( "castshadows", ChunkType::BOOL, -1 ) ) )
		bCastShadows = dynamic_cast< BoolChunk* >( pkChunk )->m_bData;
	
	// Construct the light class
	
	m_pkEntity = ( pkLight = new Light( eType, bCastShadows ) );
	
	// Fill the rest of parameters if available
	
	if( ( pkChunk = FindChunk( "ambient", ChunkType::COLOR, -1 ) ) )
		pkLight->m_kAmbient = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;

	if( ( pkChunk = FindChunk( "diffuse", ChunkType::COLOR, -1 ) ) )
		pkLight->m_kDiffuse = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;

	if( ( pkChunk = FindChunk( "specular", ChunkType::COLOR, -1 ) ) )
		pkLight->m_kSpecular = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;
	
	if( ( pkChunk = FindChunk( "constant", ChunkType::FLOAT, -1) ) )
		pkLight->m_fConstantAttenuation = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;
	
	if( ( pkChunk = FindChunk( "linear", ChunkType::FLOAT, -1) ) )
		pkLight->m_fLinearAttenuation = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;
	
	if( ( pkChunk = FindChunk( "quadratic", ChunkType::FLOAT, -1) ) )
		pkLight->m_fQuadraticAttenuation = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;


	if( m_pkEntity )
	{
		//Locate attached bounding volume
		if( ( pkChunk = FindChunk( "", ChunkType::BOUNDINGVOLUME, -1 ) ) )
		{
			BoundingVolumeChunk *pkVolChunk = dynamic_cast< BoundingVolumeChunk* >( pkChunk );

			m_pkEntity->SetBoundingVolume( pkVolChunk->m_pkVolume );

			pkVolChunk->m_pkVolume = 0;
		}
	}

	return 1;
}


}; // namespace NeoChunkIO

