/***************************************************************************
                        material.cpp  -  Material chunk
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, material.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "material.h"
#include "chunktype.h"
#include "texlayer.h"
#include "color.h"
#include "stdstring.h"
#include "float.h"
#include "bool.h"
#include "materiallib.h"

#include <neoengine/render.h>
#include <neoengine/logstream.h>

#include <string>

using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{


int MaterialChunk::ReadData( NeoEngine::File *pkFile, ChunkIO::CHUNKIOMODE eMode, unsigned int uiEnd )
{
	TechniqueChunk::ReadData( pkFile, eMode, uiEnd );

	if( !m_bIsCompatible )
	{
		//Make sure we are parsed anyway
		m_bParse = true;
	}

    vector< Chunk* > vpkTechniqueChunks;

	FindChunks( "", ChunkType::TECHNIQUE, &vpkTechniqueChunks, false, true );

	vector< Chunk* >::iterator ppkChunk = vpkTechniqueChunks.begin();
	vector< Chunk* >::iterator ppkEnd   = vpkTechniqueChunks.end();

	TechniqueChunk *pkTech = 0;

	for( ; ppkChunk < ppkEnd; ++ppkChunk )
	{
		pkTech = dynamic_cast< TechniqueChunk* >( *ppkChunk );

		if( pkTech->m_bIsCompatible )
		{
			//Stop parsing the rest of the technique chunks
			for( ++ppkChunk ; ppkChunk < ppkEnd; ++ppkChunk )
			{
				dynamic_cast< TechniqueChunk*>( *ppkChunk )->m_bIsCompatible = false;
				(*ppkChunk)->m_bParse = false;
			}
		}
	}

	return 0;
}


int MaterialChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	Chunk *pkChunk = 0;
	bool bExternal = false;

	if( ( pkChunk = FindChunk( "file", ChunkType::STRING, -1 ) ) )
	{
		string &rstrName = dynamic_cast< StringChunk* >( pkChunk )->m_strData;

		string::size_type iSlash = 0;

		string strLib  = rstrName.substr( 0, ( ( iSlash = rstrName.find_first_of( '/' ) ) != string::npos ) ? iSlash : 0 );
		string strName = rstrName.substr( ( ( iSlash = rstrName.find_last_of( '/' ) ) != string::npos ) ? ( iSlash + 1 ) : 0, string::npos );

		if( !( m_pkMaterial = Core::Get()->GetMaterialManager()->Find( strName ) ) )
		{
			//Try to load as material library
			MaterialLibrary *pkLib = new MaterialLibrary( Core::Get()->GetMaterialManager(), pkFileManager );
			
			if( strLib.length() ? pkLib->Load( strLib + ".nmat" ) : pkLib->Load( strName + ".nmat" ) )
			{
				m_pkMaterial = Core::Get()->GetMaterialManager()->Find( strName );

				unsigned int uiCurMaterials = m_vpkOtherMaterials.size();

				m_vpkOtherMaterials.resize( uiCurMaterials + pkLib->m_vpkMaterials.size() );

				copy( pkLib->m_vpkMaterials.begin(), pkLib->m_vpkMaterials.end(), m_vpkOtherMaterials.begin() + uiCurMaterials );
			}

			delete pkLib, pkLib = 0;

			if( !m_pkMaterial )
			{
				//Try to load as texture then fallthrough and parse subchunk data
				m_pkMaterial = new Material( strName, 0 );
					
				TexturePtr pkTex = Core::Get()->GetRenderDevice() ? (Texture*)Core::Get()->GetRenderDevice()->LoadTexture( strName ) : (Texture*)0;

				if( pkTex )
					m_pkMaterial->m_pkTexture = pkTex;
				else
				{
					neolog << LogLevel( ERROR ) << "*** Unable to load material chunk: Unable to load material or texture [" << ( strLib.length() ? ( strLib + "/" ) + strName : strName ) << "]" << endl;
					m_pkMaterial = 0;
				}
			}
		}

		return 0;
	}

	//Find compatible technique
    vector< Chunk* > vpkTechniqueChunks;
	bool bCompatible = false;

	FindChunks( "", ChunkType::TECHNIQUE, &vpkTechniqueChunks, false, true );

	vector< Chunk* >::iterator ppkChunk = vpkTechniqueChunks.begin();
	vector< Chunk* >::iterator ppkEnd   = vpkTechniqueChunks.end();

	TechniqueChunk *pkTech = 0;

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		pkTech = dynamic_cast< TechniqueChunk* >( *ppkChunk );

		if( pkTech->m_bIsCompatible )
		{
			m_pkMaterial = pkTech->m_pkMaterial;
			bCompatible = true;
			break;
		}
	}

	if( !bCompatible && !bExternal )
	{
		//Parse default technique
		TechniqueChunk::ParseData( uiFlags, pkFileManager );
	}

	if( ( pkChunk = FindChunk( "name", ChunkType::STRING, -1 ) ) )
		m_pkMaterial->SetName( dynamic_cast< StringChunk* >( pkChunk )->m_strData );

	return 1;
}


};


