/***************************************************************************
                        material.h  -  Material chunk
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, material.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef _NECHUNKIO_MATERIAL_H_
#define _NECHUNKIO_MATERIAL_H_


/**
  * \file neochunkio/material.h
  * Material chunk
  */


#include "base.h"
#include "complex.h"
#include "technique.h"

#include <neoengine/material.h>


namespace NeoChunkIO
{


/**
  * \brief Material chunk
  * \author Mattias Jansson (mattias@realityrift.com)
  **/
class NEOCHUNKIO_API MaterialChunk : public TechniqueChunk
{
	protected:

		/*! Other materials loaded by reference to material lib, keep from getting deallocated */
		std::vector< NeoEngine::MaterialPtr >         m_vpkOtherMaterials;


	public:

		/**
		* Initialize chunk
		* \param usType                               Chunk type
		* \param rstrType                             Chunk type as string
		* \param rstrID                               Chunk ID string
		*/ 
		                                              MaterialChunk( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID = "" ) : TechniqueChunk( usType, rstrType, rstrID ) {}
		
		/**
		* Setup chunk with default values
		* \param rstrID                               Chunk ID string, default empty
		*/
		                                              MaterialChunk( const NeoEngine::HashString &rstrID = "" ) : TechniqueChunk( ChunkType::MATERIAL, "material", rstrID ) {}
		
		/**
		* Deallocate data and subchunks
		*/
		virtual                                      ~MaterialChunk() {}

		/**
		* Check that technique is compatible with device
		* \param pkFile                               File
		* \param eMode                                Chunk mode (ChunkIO::ASCII or ChunkIO::BINARY)
		* \param uiEnd                                Bytes to end of chunk (in binary mode)
		* \return                                     <0 if error, number of bytes read if successful
		*/
		virtual int                                   ReadData( NeoEngine::File *pkFile, ChunkIO::CHUNKIOMODE eMode, unsigned int uiEnd );

		/**
		* Parse chunk data
		* \param uiFlags                              Parse flags
		* \param pkFileManager                        File manager
		* \return                                     <0 if error, >0 if successful (0 reserved)
		*/
		virtual int                                   ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager );

		/**
		* Allocate new chunk
		* \param usType                               Type identifier
		* \param rstrType                             Type identifier as string
		* \param rstrID                               ID string
		* \return                                     New chunk
		*/
		static Chunk                                 *Allocator( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID ) { return new MaterialChunk( usType, rstrType, rstrID ); }
};


};


#endif

