/***************************************************************************
                 materiallib.cpp  -  Material library file loader
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, materiallib.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "materiallib.h"
#include "chunkio.h"
#include "chunktype.h"
#include "material.h"

#include <neoengine/file.h>
#include <neoengine/logstream.h>


using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{


bool MaterialLibrary::LoadNode( unsigned int uiFlags )
{
	if( !m_pkFile || !m_pkFile->IsValid() )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to load material lib: Invalid file" << endl;
		return false;
	}

	char   szStr[5]; szStr[4] = 0;
	string strMode;

	if( !m_pkFile->DetermineByteOrder( 0x54414d4e ) )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to load material lib: Failed to determine byte order, possible corrupt file" << endl;
		return false;
	}

	m_pkFile->Read( szStr, 4 );

	if( string( szStr ) != "NMAT" )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to load material lib: Invalid ID" << endl;
		return false;
	}

	//Read mode string
	m_pkFile->Read( szStr, 4 );

	ChunkIO *pkIO = 0;

	if( string( szStr ) == "!bin" )
	{
		pkIO = new ChunkIO( ChunkIO::BINARY );
		m_pkFile->SetBinary( true );
	}
	else if( string( szStr ) == "!txt" )
	{
		pkIO = new ChunkIO( ChunkIO::ASCII );
		m_pkFile->SetBinary( false );
	}
	else
	{
		neolog << LogLevel( ERROR ) << "*** Unable to load material lib: Unknown chunk file mode [" << strMode << "]" << endl;
		return false;
	}


	*m_pkFile >> pkIO->m_iMajorVersion >> pkIO->m_iMinorVersion;

	if( !( pkIO->m_iMajorVersion == ChunkIO::MAJORVERSIONREQUIRED ) || ( pkIO->m_iMinorVersion < ChunkIO::MINORVERSIONREQUIRED ) )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to load material lib: Invalid chunk format version " << pkIO->m_iMajorVersion << "." << pkIO->m_iMinorVersion << endl;
		return false;
	}


	Chunk *pkChunk = 0;

	vector< MaterialChunk* > vpkMaterialChunks;
	
	while( ( pkChunk = pkIO->ReadChunk( m_pkFile )  ) )
	{
		if( pkChunk->GetType() == ChunkType::MATERIAL )
			vpkMaterialChunks.push_back( dynamic_cast< MaterialChunk* >( pkChunk ) );
		else
		{
			neolog << LogLevel( WARNING ) << "*** Invalid material lib: Unsupported toplevel chunk type [" << pkChunk->GetType() << "] [" << pkChunk->GetTypeAsString() << "]" << endl;
			delete pkChunk;
		}
	}


	unsigned int uiParseFlags = 0;

	vector< MaterialChunk* >::iterator ppkChunk = vpkMaterialChunks.begin();
	vector< MaterialChunk* >::iterator ppkEnd   = vpkMaterialChunks.end();

	//Now parse chunks
	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		if( pkIO->ParseChunk( *ppkChunk, uiParseFlags, m_pkFileManager ) >= 0 )
		{
			if( (*ppkChunk)->m_pkMaterial )
			{
				(*ppkChunk)->m_pkMaterial->SetTable( m_pkMaterialTable );

				m_vpkMaterials.push_back( (*ppkChunk)->m_pkMaterial );
			}
		}

		delete( *ppkChunk );
	}
	
	delete pkIO;

	return true;
}


}; // namespace NeoChunkIO
