/***************************************************************************
                      mesh.cpp  -  Mesh blueprint chunk
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, mesh.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "mesh.h"
#include "chunktype.h"
#include "submesh.h"
#include "skeleton.h"
#include "stdstring.h"

#include <vector>

using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{

int MeshChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	Chunk *pkChunk;
	string   strName;

	if( ( pkChunk = FindChunk( "name", ChunkType::STRING, -1 ) ) )
		strName = dynamic_cast< StringChunk* >( pkChunk )->m_strData;

	m_pkMesh = new Mesh( strName );

	vector<Chunk*> vpkSubMeshChunks;

	FindChunks( "", ChunkType::SUBMESH, &vpkSubMeshChunks, false );

	vector<Chunk*>::iterator ppkChunk = vpkSubMeshChunks.begin();
	vector<Chunk*>::iterator ppkEnd   = vpkSubMeshChunks.end();

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		SubMeshChunk *pkSubMeshChunk = dynamic_cast< SubMeshChunk* >( *ppkChunk );

		m_pkMesh->m_vpkSubMeshes.push_back( pkSubMeshChunk->m_pkSubMesh );

		pkSubMeshChunk->m_pkSubMesh = 0;
	}	

	if( ( pkChunk = FindChunk( "", ChunkType::SKELETON, -1 ) ) )
	{
		SkeletonChunk *pkSkeletonChunk = dynamic_cast< SkeletonChunk* >( pkChunk );

		m_pkMesh->m_pkSkeleton = pkSkeletonChunk->m_pkSkeleton;

		pkSkeletonChunk->m_pkSkeleton = 0;
	}

	return 1;
}


}; // namespace NeoChunkIO


