/***************************************************************************
                     meshentity.cpp  -  Mesh entity chunk
                             -------------------
    begin                : Wed Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, meshentity.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "meshentity.h"
#include "boundingvolume.h"
#include "stdstring.h"

#include <neoengine/core.h>
#include <neoengine/mesh.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


int MeshEntityChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	SceneEntityChunk::ParseData( uiFlags, pkFileManager );

	Chunk      *pkChunk = 0;
	MeshEntity *pkMesh  = 0;

	// "mesh" is new style name, "blueprint" is old-style deprecated name
	if( ( pkChunk = FindChunk( "mesh", ChunkType::STRING, -1 ) ) || ( pkChunk = FindChunk( "blueprint", ChunkType::STRING, -1 ) ) )
	{
		StringChunk *pkStringChunk = dynamic_cast< StringChunk* >( pkChunk );

		string::size_type uiSlash;

		string strScene = pkStringChunk->m_strData.substr( 0, pkStringChunk->m_strData.find_first_of( '/' ) );
		string strMesh  = pkStringChunk->m_strData.substr( ( ( uiSlash = pkStringChunk->m_strData.find_last_of( '/' ) ) != string::npos ) ? ( uiSlash + 1 ) : 0, string::npos );

		//Locate blueprint
		const std::vector< Mesh* > *pvpkMeshes = Core::Get()->GetMeshManager()->Find( strMesh );

		if( pvpkMeshes && pvpkMeshes->size() )
			m_pkEntity = ( pkMesh = new MeshEntity( (*pvpkMeshes)[0] ) );
		else
		{
			//FIXME: Try loading scene strScene and locate blueprint
			neolog << LogLevel( ERROR ) << "*** Invalid meshinstance chunk: Failed to locate mesh [" << strMesh << "]" << endl;
		}
	}

	if( m_pkEntity )
	{
		//Locate attached bounding volume
		if( ( pkChunk = FindChunk( "", ChunkType::BOUNDINGVOLUME, -1 ) ) )
		{
			BoundingVolumeChunk *pkVolChunk = dynamic_cast< BoundingVolumeChunk* >( pkChunk );

			m_pkEntity->SetBoundingVolume( pkVolChunk->m_pkVolume );

			pkVolChunk->m_pkVolume = 0;
		}
		//TEMPORARY solution to force a bounding volume
		else
			pkMesh->GenerateBoundingVolume();
	}

	return 1;
}


}; // namespace NeoChunkIO

