/***************************************************************************
                       sceneentity.h  -  Scene entity base chunk
                             -------------------
    begin                : Wed Apr 9 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, sceneentity.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef _NECHUNKIO_SCENEENTITY_H_
#define _NECHUNKIO_SCENEENTITY_H_


/**
  * \file neochunkio/sceneentity.h
  * Scene entity base chunk
  */


#include "base.h"
#include "complex.h"


// External classes
namespace NeoEngine
{
	class SceneEntity;
};


namespace NeoChunkIO
{


/**
  * \brief Scene entity base chunk for all scene entity types
  * \author Mattias Jansson (mattias@realityrift.com)
  **/
class NEOCHUNKIO_API SceneEntityChunk : public ComplexChunk
{
	public:

		/*! Scene entity object */
		NeoEngine::SceneEntity                       *m_pkEntity;

		/**
		* Initialize chunk
		* \param usType                               Chunk type
		* \param rstrType                             Chunk type as string
		* \param rstrID                               Chunk ID string
		*/ 
		                                              SceneEntityChunk( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID = "" ) : ComplexChunk( usType, rstrType, rstrID ), m_pkEntity( 0 ) {}
		
		/**
		* Deallocate data and subchunks
		*/
		virtual                                      ~SceneEntityChunk();

		/**
		* Parse chunk data
		* \param uiFlags                              Parse flags
		* \param pkFileManager                        File manager
		* \return                                     <0 if error, >0 if successful (0 reserved)
		*/
		virtual int                                   ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager );
};


};


#endif

