/***************************************************************************
              shader.cpp  -  Vertex/fragment shader source chunk
                             -------------------
    begin                : Fri Jan 9 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, shader.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "shader.h"
#include "stdstring.h"

#include <neoengine/core.h>
#include <neoengine/render.h>
#include <neoengine/strutil.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


ShaderChunk::~ShaderChunk()
{
}


int ShaderChunk::ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager )
{
	string &rstrType = dynamic_cast< StringChunk* >( FindChunk( "type", ChunkType::STRING ) )->m_strData;

	Shader::SHADERTYPE eType;

	if( rstrType == "vp" )
		eType = Shader::VERTEXSHADER;
	else //if( rstrType == "fp" )
		eType = Shader::FRAGMENTSHADER;

	StringChunk *pkTargetChunk = dynamic_cast< StringChunk* >( FindChunk( "target", ChunkType::STRING ) );

	Shader::SHADERTARGET eTarget = Shader::NOTSUPPORTED;

	if( pkTargetChunk )
	{
		//Find matching target
		vector< string > vstrTargets;

		Explode( ", ", pkTargetChunk->m_strData, &vstrTargets );

		vector< string >::iterator pstrTarget    = vstrTargets.begin();
		vector< string >::iterator pstrTargetEnd = vstrTargets.end();

		for( ; pstrTarget != pstrTargetEnd; ++pstrTarget, eTarget = Shader::NOTSUPPORTED )
		{
			eTarget = Shader::GetTargetFromString( *pstrTarget );

			if( ( eTarget != Shader::NOTSUPPORTED ) && ( eTarget == Shader::GetTarget( eType ) ) )
				break;
		}
	}
	else
		eTarget = Shader::GetTarget( eType );

	if( eTarget == Shader::NOTSUPPORTED )
		return 0;

	StringChunk *pkSourceChunk = dynamic_cast< StringChunk* >( FindChunk( "source", ChunkType::STRING ) );
	StringChunk *pkFileChunk;

	if( pkSourceChunk )
	{
		m_pkShader = Core::Get()->GetRenderDevice()->CreateShader( eType );
		m_pkShader->Compile( &pkSourceChunk->m_strData );
	}
	else if( ( pkFileChunk = dynamic_cast< StringChunk* >( FindChunk( "sourcefile", ChunkType::STRING ) ) ) )
	{
		if( ( m_pkShader = Core::Get()->GetRenderDevice()->GetShader( eType, File::ExtractBaseFileName( pkFileChunk->m_strData ) ) ) )
			return 1;
		m_pkShader = Core::Get()->GetRenderDevice()->CreateShader( eType );
		m_pkShader->Load( pkFileChunk->m_strData );
	}

	return 1;
}


};


