/***************************************************************************
                      skeleton.cpp  -  Skeleton chunk
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, skeleton.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skeleton.h"
#include "skeletonanimation.h"
#include "chunktype.h"
#include "integer.h"
#include "stdstring.h"
#include "bone.h"

#include <neoengine/skeleton.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


SkeletonChunk::~SkeletonChunk()
{
	delete m_pkSkeleton;
}


int SkeletonChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	m_pkSkeleton = new Skeleton;

	//Get bones
	Chunk *pkChunk;
	vector<Chunk*> vpkBoneChunks;

	FindChunks( "", ChunkType::BONE, &vpkBoneChunks, false );

	if( !vpkBoneChunks.size() )
		return 2;

	m_pkSkeleton->m_uiNumBones = vpkBoneChunks.size();
	m_pkSkeleton->m_ppkBones   = new Bone*[ m_pkSkeleton->m_uiNumBones ];

	memset( m_pkSkeleton->m_ppkBones, 0, sizeof( Bone* ) * m_pkSkeleton->m_uiNumBones );

	vector<Chunk*>::iterator ppkChunk = vpkBoneChunks.begin();
	vector<Chunk*>::iterator ppkEnd   = vpkBoneChunks.end();

	BoneChunk *pkBoneChunk = 0;

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		pkBoneChunk = dynamic_cast< BoneChunk* >( *ppkChunk );

		assert( pkBoneChunk->m_pkBone->m_uiID < vpkBoneChunks.size() );

		m_pkSkeleton->m_ppkBones[ pkBoneChunk->m_pkBone->m_uiID ] = pkBoneChunk->m_pkBone;
	}

	//Setup hierarchy
	for( unsigned int uiBone = 0; uiBone < m_pkSkeleton->m_uiNumBones; ++uiBone )
	{
		ppkChunk = vpkBoneChunks.begin();

		for( ; ppkChunk != ppkEnd; ++ppkChunk )
		{
			pkBoneChunk = dynamic_cast< BoneChunk* >( *ppkChunk );

			if( pkBoneChunk->m_pkBone->m_uiID == uiBone )
				break;
		}

		if( ppkChunk == ppkEnd )
		{
			neolog << LogLevel( ERROR ) << "*** Invalid skeleton chunk: Invalid bone chunk hierarchy, unable to locate bone chunk for ID [" << uiBone << "]" << endl;
			return -1;
		}

		//Find parent ID
		int iParent = -1;

		if( ( pkChunk = pkBoneChunk->FindChunk( "parent", ChunkType::INTEGER, -1 ) ) )
			iParent = dynamic_cast< IntChunk* >( pkChunk )->m_iData;
		else if( ( pkChunk = pkBoneChunk->FindChunk( "parent", ChunkType::STRING, -1 ) ) )
		{
			string strName = dynamic_cast< StringChunk* >( pkChunk )->m_strData;

			for( unsigned int uiSearch = 0; ( uiSearch < m_pkSkeleton->m_uiNumBones ) && ( iParent < 0 ); ++uiSearch )
				if( ( uiSearch != uiBone ) && m_pkSkeleton->m_ppkBones[ uiBone ] && ( m_pkSkeleton->m_ppkBones[ uiBone ]->GetName() == strName ) )
					iParent = (int)uiSearch;
		}

		if( iParent < 0 )
		{
			if( m_pkSkeleton->m_pkRoot )
			{
				neolog << LogLevel( ERROR ) << "*** Invalid skeleton chunk: Invalid bone chunk hierarchy, multiple root bones" << endl;
				return -1;
			}

			m_pkSkeleton->m_pkRoot = m_pkSkeleton->m_ppkBones[ uiBone ];
		}
		else
			m_pkSkeleton->m_ppkBones[ iParent ]->AttachNode( m_pkSkeleton->m_ppkBones[ uiBone ], false );
	}

	ppkChunk = vpkBoneChunks.begin();

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
		dynamic_cast< BoneChunk* >( *ppkChunk )->m_pkBone = 0;


	if( !m_pkSkeleton->m_pkRoot )
	{
		neolog << LogLevel( ERROR ) << "*** Invalid skeleton chunk: Invalid bone chunk hierarchy, no root bone" << endl;
		return -1;
	}

	m_pkSkeleton->m_pkRoot->NotifyUpdate( true );

	//Get animations
	vector<Chunk*> vpkAnimChunks;

	FindChunks( "", ChunkType::SKELETONANIMATION, &vpkAnimChunks, false );

	ppkChunk = vpkAnimChunks.begin();
	ppkEnd   = vpkAnimChunks.end();

	SkeletonAnimationChunk *pkAnimChunk = 0;

	for( ; ppkChunk != ppkEnd; ++ppkChunk )
	{
		if( ( pkAnimChunk = dynamic_cast< SkeletonAnimationChunk* >( *ppkChunk ) ) )
		{
			m_pkSkeleton->AddAnimation( pkAnimChunk->m_pkAnim );
			pkAnimChunk->m_pkAnim = 0;
		}
	}

	return 1;
}


}; // namespace NeoChunkIO

