/***************************************************************************
               skeletonanimation.cpp  -  Skeleton animation chunk
                             -------------------
    begin                : Tue Mar 4 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, skeletonanimation.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skeletonanimation.h"
#include "nodeanimation.h"
#include "float.h"
#include "stdstring.h"
#include "integer.h"

#include <neoengine/skeletonanimator.h>

using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{

SkeletonAnimationChunk::~SkeletonAnimationChunk()
{
	delete m_pkAnim;
}


int SkeletonAnimationChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	m_pkAnim = new SkeletonAnimation;

	Chunk *pkChunk;

	if( ( pkChunk = FindChunk( "length", ChunkType::FLOAT, -1 ) ) )
		m_pkAnim->m_fLength = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;

	if( ( pkChunk = FindChunk( "name", ChunkType::STRING, -1 ) ) )
		m_pkAnim->m_strName = dynamic_cast< StringChunk* >( pkChunk )->m_strData;

	if( ( pkChunk = FindChunk( "id", ChunkType::INTEGER, -1 ) ) )
		m_pkAnim->m_uiID    = (unsigned int)dynamic_cast< IntChunk* >( pkChunk )->m_iData;


	//Try to find animation chunks
	vector<Chunk*> vpkAnimChunks;

	FindChunks( "", ChunkType::NODEANIMATION, &vpkAnimChunks, false );

	m_pkAnim->m_vpkChannels.resize( vpkAnimChunks.size() );

	vector<Chunk*>::iterator ppkChunk = vpkAnimChunks.begin();
	vector<Chunk*>::iterator ppkEnd   = vpkAnimChunks.end();

	vector<NodeAnimation*>::iterator ppkAnim = m_pkAnim->m_vpkChannels.begin();

	for( ; ppkChunk != ppkEnd; ++ppkChunk, ++ppkAnim )
	{
		NodeAnimationChunk *pkAnimChunk = dynamic_cast< NodeAnimationChunk* >( *ppkChunk );

		*ppkAnim = pkAnimChunk->m_pkAnim;

		(*ppkAnim)->m_fLength = m_pkAnim->m_fLength;
		(*ppkAnim)->m_strName = m_pkAnim->m_strName;
		(*ppkAnim)->m_uiID    = m_pkAnim->m_uiID;

		pkAnimChunk->m_pkAnim = 0;
	}

	return 1;
}


}; // namespace NeoChunkIO


