/***************************************************************************
                           skin.cpp  -  Skin chunk
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, skin.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skin.h"
#include "chunktype.h"
#include "vertexbuffer.h"
#include "skinvertexbuffer.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


int SkinChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	m_pkSkin = new Skin;

	//Locate skin vertex buffer and vertex buffer
	Chunk *pkChunk;
	unsigned int uiNumVertices;

	if( !( pkChunk = FindChunk( "", ChunkType::SKINVERTEXBUFFER, -1 ) ) )
	{
		neolog << LogLevel( ERROR ) << "*** Invalid skin chunk: no skinvertexbuffer subchunk" << endl;
		return -1;
	}
	else
	{
		SkinVertexBufferChunk *pkBufferChunk = dynamic_cast< SkinVertexBufferChunk* >( pkChunk );

		m_pkSkin->m_pkSkinVertices = pkBufferChunk->m_pkBuffer;

		uiNumVertices              = pkBufferChunk->m_uiNumVertices;

		pkBufferChunk->m_pkBuffer  = 0;
	}

	if( !( pkChunk = FindChunk( "", ChunkType::VERTEXBUFFER, -1 ) ) )
	{
		neolog << LogLevel( ERROR ) << "*** Invalid skin chunk: no vertexbuffer subchunk" << endl;
		return -1;
	}
	else
	{
		VertexBufferChunk *pkBufferChunk = dynamic_cast< VertexBufferChunk* >( pkChunk );

		m_pkSkin->m_pkVertices = pkBufferChunk->m_pkBuffer;

		if( m_pkSkin->m_pkVertices->GetNumElements() != uiNumVertices )
		{
			neolog << LogLevel( ERROR ) << "*** Invalid skin chunk: vertex count mismatch" << endl;
			return -1;
		}
	}

	m_pkSkin->m_iNumVertices = (int)uiNumVertices;

	return 1;
}


}; // namespace NeoChunkIO


