/***************************************************************************
                skinvertexbuffer.cpp  -  Skin vertex buffer chunk
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, skinvertexbuffer.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skinvertexbuffer.h"
#include "chunktype.h"
#include "integer.h"

#include <neoengine/skin.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


SkinVertexBufferChunk::~SkinVertexBufferChunk()
{
	delete [] m_pkBuffer;
}


int SkinVertexBufferChunk::GetSize( bool bIncludeHeader ) const
{
	int iSize = ComplexChunk::GetSize( bIncludeHeader );

	for( unsigned int uiVert = 0; uiVert < m_uiNumVertices; ++uiVert )
		iSize += 4 + m_pkBuffer[ uiVert ].m_iNumInfluences * 20; //num integer + num * influence data size

	return iSize;
}


int SkinVertexBufferChunk::ReadData( File *pkFile, ChunkIO::CHUNKIOMODE eMode, unsigned int uiBytes )
{
	int iOffset = pkFile->Tellg();

	//Get num subchunk
	Chunk *pkChunk;

	if( !( pkChunk = FindChunk( "num", ChunkType::INTEGER, -1 ) ) )
	{
		neolog << LogLevel( ERROR ) << "*** Invalid skinvertexbuffer chunk: no num subchunk" << endl;
		return -1;
	}

	m_uiNumVertices = (unsigned int)dynamic_cast< IntChunk* >( pkChunk )->m_iData;

	m_pkBuffer      = new SkinVertex[ m_uiNumVertices ];

	for( unsigned int i = 0; i < m_uiNumVertices; ++i )
	{
		//Read number of influences
		*pkFile >> m_pkBuffer[i].m_iNumInfluences;

		m_pkBuffer[i].m_pkInfluences = new BoneInfluence[ m_pkBuffer[i].m_iNumInfluences ];

		//Read influence data
		if( ( pkFile->GetByteOrder() == Core::BYTEORDER_SYSTEM ) && ( eMode == ChunkIO::BINARY ) )
			pkFile->Read( m_pkBuffer[i].m_pkInfluences, sizeof( BoneInfluence ) * m_pkBuffer[i].m_iNumInfluences );
		else for( int j = 0; j < m_pkBuffer[i].m_iNumInfluences; ++j )
			*pkFile >> m_pkBuffer[i].m_pkInfluences[j].m_iBoneID
			        >> m_pkBuffer[i].m_pkInfluences[j].m_fWeight
			        >> m_pkBuffer[i].m_pkInfluences[j].m_kOffset.x
			        >> m_pkBuffer[i].m_pkInfluences[j].m_kOffset.y
			        >> m_pkBuffer[i].m_pkInfluences[j].m_kOffset.z;
	}

	return( !*pkFile ? -1 : ( (int)pkFile->Tellg() - iOffset ) );
}


int SkinVertexBufferChunk::WriteData( File *pkFile, ChunkIO::CHUNKIOMODE eMode, unsigned int uiLevel )
{
	int iOffset = pkFile->Tellp();

	if( eMode == ChunkIO::BINARY )
	{
		for( unsigned int i = 0; i < m_uiNumVertices; ++i )
		{
			//Write number of influences
			*pkFile << m_pkBuffer[i].m_iNumInfluences;

			//Write influence data
			for( int j = 0; j < m_pkBuffer[i].m_iNumInfluences; ++j )
				*pkFile << m_pkBuffer[i].m_pkInfluences[j].m_iBoneID
				        << m_pkBuffer[i].m_pkInfluences[j].m_fWeight
				        << m_pkBuffer[i].m_pkInfluences[j].m_kOffset.x
				        << m_pkBuffer[i].m_pkInfluences[j].m_kOffset.y
				        << m_pkBuffer[i].m_pkInfluences[j].m_kOffset.z;
		}
	}
	else if( eMode == ChunkIO::ASCII )
	{
		string strSpacer( ( uiLevel + 1 ) * 2, ' ' );

		for( unsigned int i = 0; i < m_uiNumVertices; ++i )
		{
			//Write number of influences
			*pkFile << endl << strSpacer << m_pkBuffer[i].m_iNumInfluences;

			//Write influence data
			for( int j = 0; j < m_pkBuffer[i].m_iNumInfluences; ++j )
			{
				*pkFile << endl << strSpacer << "  "
				        << m_pkBuffer[i].m_pkInfluences[j].m_iBoneID   << " "
				        << m_pkBuffer[i].m_pkInfluences[j].m_fWeight   << " "
				        << m_pkBuffer[i].m_pkInfluences[j].m_kOffset.x << " "
				        << m_pkBuffer[i].m_pkInfluences[j].m_kOffset.y << " "
				        << m_pkBuffer[i].m_pkInfluences[j].m_kOffset.z;
			}
		}

		*pkFile << endl << string( ( uiLevel - 3 ) * 2, ' '  );
	}

	return( !*pkFile ? -1 : ( (int)pkFile->Tellp() - iOffset ) );
}


}; // namespace NeoChunkIO

