/***************************************************************************
                        skydome.cpp  -  Skydome chunk
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, skydome.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skydome.h"
#include "chunktype.h"
#include "material.h"
#include "quaternion.h"
#include "float.h"
#include "integer.h"

#include <neoengine/skydome.h>
#include <neoengine/logstream.h>


using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


int SkyDomeChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	SkyDome *pkDome = new SkyDome;

	m_pkEntity = pkDome;

	Chunk *pkChunk;

	Quaternion kOrientation;
	float      fDistance    = 100.0f;
	float      fCurvature   = 1.5f;
	int        iSegments    = 16;
	float      fTiles       = 1.0f;

	if( ( pkChunk = FindChunk( "orientation", ChunkType::QUATERNION, -1 ) ) )
		kOrientation = dynamic_cast< QuaternionChunk* >( pkChunk )->m_kData;

	if( ( pkChunk = FindChunk( "distance", ChunkType::FLOAT, -1 ) ) )
		fDistance  = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;

	if( !( pkChunk = FindChunk( "curvature", ChunkType::FLOAT, -1 ) ) )
		fCurvature = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;


	if( ( pkChunk = FindChunk( "segments", ChunkType::INTEGER, -1 ) ) )
		iSegments = dynamic_cast< IntChunk* >( pkChunk )->m_iData;

	if( ( pkChunk = FindChunk( "tiles", ChunkType::FLOAT, -1 ) ) )
		fTiles = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;

	pkDome->Set( kOrientation, fDistance, fCurvature, iSegments, fTiles );


	vector<Chunk*> vpkMatChunks;

	FindChunks( "", ChunkType::MATERIAL, &vpkMatChunks, false );

	if( vpkMatChunks.size() != SkyDome::NUMPLANES )
		neolog << LogLevel( WARNING ) << "*** Invalid skydome chunk: material subchunk count mismatch" << endl;
	else for( unsigned int i = 0; i < SkyDome::NUMPLANES; ++i )
		pkDome->SetMaterial( (SkyDome::SKYDOMEPLANE)i, dynamic_cast< MaterialChunk* >( vpkMatChunks[i] )->m_pkMaterial );

	//Make us findable by scene entity type
	m_usFindType = ChunkType::SCENEENTITY;

	return 1;
}


}; // namespace NeoChunkIO

