/***************************************************************************
                         submesh.cpp  -  Submesh chunks
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, submesh.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "submesh.h"
#include "chunktype.h"
#include "vertexbuffer.h"
#include "polygonbuffer.h"
#include "skin.h"
#include "submeshanimation.h"
#include "material.h"
#include "integer.h"

#include <neoengine/submesh.h>
#include <neoengine/animatedsubmesh.h>
#include <neoengine/skin.h>
#include <neoengine/patch.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{

SubMeshChunk::~SubMeshChunk()
{
	delete m_pkSubMesh;
}


int SubMeshChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	Chunk *pkChunk;

	if( m_usType == ChunkType::SUBMESH )
	{
		m_pkSubMesh = new SubMesh;

		//Locate vertex buffer chunk
		if( ( pkChunk = FindChunk( "", ChunkType::VERTEXBUFFER, -1 ) ) )
			m_pkSubMesh->SetVertexBuffer( dynamic_cast< VertexBufferChunk* >( pkChunk )->m_pkBuffer );
	}
	else if( m_usType == ChunkType::ANIMATEDSUBMESH )
	{
		AnimatedSubMesh *pkAnimatedSubMesh = 0;

		m_pkSubMesh = ( pkAnimatedSubMesh = new AnimatedSubMesh );

		//Locate animation chunk
		vector<Chunk*> vpkAnimChunks;

		FindChunks( "", ChunkType::SUBMESHANIMATION, &vpkAnimChunks, false );

		vector<Chunk*>::iterator ppkChunk = vpkAnimChunks.begin();
		vector<Chunk*>::iterator ppkEnd   = vpkAnimChunks.end();

		for( ; ppkChunk != ppkEnd; ++ppkChunk )
		{
			SubMeshAnimationChunk *pkAnimChunk = dynamic_cast< SubMeshAnimationChunk* >( *ppkChunk );
	
			if( pkAnimatedSubMesh->AddAnimation( pkAnimChunk->m_pkAnim ) )
				pkAnimChunk->m_pkAnim = 0;
			else
				neolog << LogLevel( WARNING ) << "*** Submesh animation [" << pkAnimChunk->m_pkAnim->m_strName << "] [" << pkAnimChunk->m_pkAnim->m_uiID << "] rejected" << endl;
		}
	}
	else if( m_usType == ChunkType::SKELETALSUBMESH )
	{
		SkeletalSubMesh *pkSkeletalSubMesh = 0;

		m_pkSubMesh = ( pkSkeletalSubMesh = new SkeletalSubMesh );

		//Locate skin chunk
		if( ( pkChunk = FindChunk( "", ChunkType::SKIN, -1 ) ) )
			pkSkeletalSubMesh->m_pkSkin = dynamic_cast< SkinChunk* >( pkChunk )->m_pkSkin;
	}
	else if( m_usType == ChunkType::PATCHSUBMESH )
	{
		PatchSurfaceData kData;

		kData.m_ePatchType      = (PatchSurfaceData::PATCHSURFACETYPE)   dynamic_cast< IntChunk* >( FindChunk( "type",       ChunkType::INTEGER, -1 ) )->m_iData;
		kData.m_eVisibleSide    = (PatchSurfaceData::PATCHSURFACEVISIBLE)dynamic_cast< IntChunk* >( FindChunk( "visibility", ChunkType::INTEGER, -1 ) )->m_iData;

		kData.m_iControlWidth   = dynamic_cast< IntChunk* >( FindChunk( "width",      ChunkType::INTEGER, -1 ) )->m_iData;
		kData.m_iControlHeight  = dynamic_cast< IntChunk* >( FindChunk( "height",     ChunkType::INTEGER, -1 ) )->m_iData;

		kData.m_pkControlPoints = dynamic_cast< VertexBufferChunk* >( FindChunk( "controlpoints", ChunkType::VERTEXBUFFER, -1 ) )->m_pkBuffer;

		m_pkSubMesh = new PatchSurfaceSubMesh( kData );
	}
	else
	{
		neolog << LogLevel( ERROR ) << "*** Invalid submesh chunk: Invalid type" << endl;
		return -1;
	}

	//Locate polygon buffer chunk
	if( ( pkChunk = FindChunk( "", ChunkType::POLYGONBUFFER, -1 ) ) )
		m_pkSubMesh->SetPolygonBuffer( dynamic_cast< PolygonBufferChunk* >( pkChunk )->m_pkBuffer );

	//Locate material subchunk
	if( ( pkChunk = FindChunk( "", ChunkType::MATERIAL, -1 ) ) )
		m_pkSubMesh->m_pkMaterial = dynamic_cast< MaterialChunk* >( pkChunk )->m_pkMaterial;

	//Check for shadow vertex buffer
	if( ( pkChunk = FindChunk( "shadow", ChunkType::VERTEXBUFFER, -1 ) ) )
		m_pkSubMesh->SetShadowVertexBuffer( dynamic_cast< VertexBufferChunk* >( pkChunk )->m_pkBuffer );

	//Check for shadow polygon buffer
	if( ( pkChunk = FindChunk( "shadow", ChunkType::POLYGONBUFFER, -1 ) ) )
		m_pkSubMesh->SetShadowPolygonBuffer( dynamic_cast< PolygonBufferChunk* >( pkChunk )->m_pkBuffer );

	//Make all node types findable with a FindChunk search for scene nodes
	m_usFindType = ChunkType::SUBMESH;

	return 1;
}


}; // namespace NeoChunkIO


