/***************************************************************************
                   technique.cpp  -  Material technique chunk
                             -------------------
    begin                : Fri Jan 9 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, technique.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "technique.h"
#include "shader.h"
#include "texlayer.h"
#include "color.h"
#include "stdstring.h"
#include "bool.h"
#include "integer.h"
#include "float.h"

#include <neoengine/render.h>
#include <neoengine/core.h>
#include <neoengine/strutil.h>

using namespace std;
using namespace NeoEngine;


namespace NeoChunkIO
{


TechniquePassChunk::~TechniquePassChunk()
{
	delete m_pkPass;
}


int TechniquePassChunk::ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager )
{
	if( !m_pkPass )
		m_pkPass = new MaterialPass;

	vector< Chunk* > vpkChunks;

	FindChunks( "", ChunkType::SHADER, &vpkChunks, false, true );

	vector< Chunk* >::iterator ppkChunk    = vpkChunks.begin();
	vector< Chunk* >::iterator ppkChunkEnd = vpkChunks.end();

	for( ; ppkChunk != ppkChunkEnd; ++ppkChunk )
	{
		ShaderChunk *pkChunk = dynamic_cast< ShaderChunk* >( *ppkChunk );

		if( !m_pkPass->m_pkVertexShader && pkChunk->m_pkShader && ( pkChunk->m_pkShader->GetType() == Shader::VERTEXSHADER ) )
			m_pkPass->m_pkVertexShader = pkChunk->m_pkShader;
		else if( !m_pkPass->m_pkFragmentShader && pkChunk->m_pkShader && ( pkChunk->m_pkShader->GetType() == Shader::FRAGMENTSHADER ) )
			m_pkPass->m_pkFragmentShader = pkChunk->m_pkShader;

		if( m_pkPass->m_pkVertexShader && m_pkPass->m_pkFragmentShader )
			break;
	}

	vpkChunks.clear();

	FindChunks( "", ChunkType::TEXTURELAYER, &vpkChunks, false, true );

	ppkChunk    = vpkChunks.begin();
	ppkChunkEnd = vpkChunks.end();
	unsigned int uiLayer = 0;

	for( ; ppkChunk != ppkChunkEnd; ++ppkChunk, ++uiLayer )
	{
		TextureLayerChunk *pkLayerChunk = dynamic_cast< TextureLayerChunk* >( *ppkChunk );

		if( !uiLayer )
			*((TextureLayer*)m_pkPass) = *(pkLayerChunk->m_pkTexLayer);
		else
		{
			m_pkPass->m_vpkTextureLayers.push_back( pkLayerChunk->m_pkTexLayer );
			pkLayerChunk->m_pkTexLayer = 0;
		}
	}

	Chunk *pkChunk = 0;

	if( ( pkChunk = FindChunk( "zbuffer", ChunkType::STRING, -1 ) ) )
	{
		string &rstrMode = dynamic_cast< StringChunk* >( pkChunk )->m_strData;
	
		//Split on whitespace and convert to enums
		m_pkPass->m_kZBufferMode.Set(
		  ZBufferMode::GetTestFuncFromString( rstrMode.substr( 0, rstrMode.find_first_of( " \t" ) ) ),
		  ZBufferMode::GetWriteModeFromString( rstrMode.substr( rstrMode.find_last_of( " \t" ) + 1, string::npos ) )
		);
	}
		
	if( ( pkChunk = FindChunk( "ambient", ChunkType::COLOR, -1 ) ) )
		m_pkPass->m_kAmbient = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;
	if( ( pkChunk = FindChunk( "diffuse", ChunkType::COLOR, -1 ) ) )
		m_pkPass->m_kDiffuse = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;
	if( ( pkChunk = FindChunk( "specular", ChunkType::COLOR, -1 ) ) )
		m_pkPass->m_kSpecular = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;
	if( ( pkChunk = FindChunk( "emission", ChunkType::COLOR, -1 ) ) )
		m_pkPass->m_kEmission = dynamic_cast< ColorChunk* >( pkChunk )->m_kData;
	if( ( pkChunk = FindChunk( "shininess", ChunkType::FLOAT, -1 ) ) )
		m_pkPass->m_fShininess = dynamic_cast< FloatChunk* >( pkChunk )->m_fData;

	//Multipass for each X number of lights
	if( ( pkChunk = FindChunk( "maxlights", ChunkType::INTEGER, -1 ) ) )
		m_pkPass->m_uiMaxLights = dynamic_cast< IntChunk* >( pkChunk )->m_iData;

	return 1;
}



TechniqueChunk::~TechniqueChunk()
{
}


int TechniqueChunk::ReadData( NeoEngine::File *pkFile, ChunkIO::CHUNKIOMODE eMode, unsigned int uiEnd )
{
	StringChunk *pkTargetChunk[2] = { dynamic_cast< StringChunk* >( FindChunk( "vp", ChunkType::STRING ) ), dynamic_cast< StringChunk* >( FindChunk( "fp", ChunkType::STRING ) ) };
	Shader::SHADERTYPE aeType[2] = { Shader::VERTEXSHADER, Shader::FRAGMENTSHADER };

	for( int i = 0; i < 2; ++i )
	{
		if( pkTargetChunk[i] )
		{
			//Find matching target
			vector< string > vstrTargets;

			Explode( ", ", pkTargetChunk[i]->m_strData, &vstrTargets );

			vector< string >::iterator pstrTarget    = vstrTargets.begin();
			vector< string >::iterator pstrTargetEnd = vstrTargets.end();

			Shader::SHADERTARGET eTarget = Shader::NOTSUPPORTED;

			for( ; pstrTarget != pstrTargetEnd; ++pstrTarget )
			{
				eTarget = Shader::GetTargetFromString( *pstrTarget );

				if( ( eTarget != Shader::NOTSUPPORTED ) && ( eTarget == Shader::GetTarget( aeType[i] ) ) )
					break;

				eTarget = Shader::NOTSUPPORTED;
			}

			if( eTarget == Shader::NOTSUPPORTED )
			{
				//This technique is incompatible
				m_bParse        = false;
				m_bIsCompatible = false;
				return 0;
			}
		}
	}

	Chunk *pkChunk;

	if( ( pkChunk = FindChunk( "texunits", ChunkType::INTEGER ) ) )
	{
		int iTMUs = dynamic_cast< IntChunk* >( pkChunk )->m_iData;

		if( iTMUs > int( ( Core::Get()->GetRenderDevice()->GetCaps().GetGroupFlags( RenderCaps::MULTITEXTURE_GROUP ) & RenderCaps::MULTITEXTURE_UNITS_MASK ) >> RenderCaps::MULTITEXTURE_UNITS_SHIFT ) )
		{
			//This technique is incompatible
			m_bParse        = false;
			m_bIsCompatible = false;
			return 0;
		}
	}

	return 1;
}


int TechniqueChunk::ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager )
{
	//If reaching this point, we know this technique is compatible
	if( !m_pkMaterial )
		m_pkMaterial = new Material( "__tech", 0 );

	//Locate passes
	vector< Chunk* > vpkChunks;

	FindChunks( "", ChunkType::PASS, &vpkChunks, false, true );

	if( !vpkChunks.size() )
	{
		//Parse this chunk as pass
		m_pkPass = m_pkMaterial;
		TechniquePassChunk::ParseData( uiFlags, pkFileManager );
		m_pkPass = 0;
	}
	else
	{
		TechniquePassChunk *pkPassChunk;

		vector< Chunk* >::iterator ppkChunk    = vpkChunks.begin();
		vector< Chunk* >::iterator ppkChunkEnd = vpkChunks.end();

		*((MaterialPass*)(Material*)m_pkMaterial) = *dynamic_cast< TechniquePassChunk* >( *ppkChunk )->m_pkPass;

		for( ++ppkChunk ; ppkChunk != ppkChunkEnd; ++ppkChunk )
		{
			pkPassChunk = dynamic_cast< TechniquePassChunk* >( *ppkChunk );
			m_pkMaterial->m_vpkPasses.push_back( pkPassChunk->m_pkPass );
			pkPassChunk->m_pkPass = 0;
		}
	}

	Chunk *pkChunk = 0;

	if( ( pkChunk = FindChunk( "shadows", ChunkType::BOOL, -1 ) ) )
		m_pkMaterial->m_bDynamicShadows = dynamic_cast< BoolChunk* >( pkChunk )->m_bData;

	return 1;
}


};


