/***************************************************************************
                 texmatrixgen.cpp  -  Texture matrix gen chunk
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoChunkIO, texmatrixgen.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "texmatrixgen.h"
#include "chunktype.h"
#include "matrix.h"
#include "stdstring.h"
#include "float.h"

#include <neoengine/texmatrixgen.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;

namespace NeoChunkIO
{



TextureMatrixGenChunk::~TextureMatrixGenChunk()
{
	delete m_pkGen;
}


int TextureMatrixGenChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	Chunk *pkChunk = 0;
	
	if( !( pkChunk = FindChunk( "type", ChunkType::STRING, -1 ) ) )
	{
		neolog << LogLevel( ERROR ) << "*** Invalid texmatrixgen chunk: no type subchunk" << endl;
		return -1;
	}

	string &rstrType = dynamic_cast< StringChunk* >( pkChunk )->m_strData;

	if( rstrType == "scroll" )
	{
		TextureMatrixGenScroll *pkScroll = new TextureMatrixGenScroll;

		pkScroll->m_afScrollSpeed[0] = dynamic_cast< FloatChunk* >( FindChunk( "x", ChunkType::FLOAT, -1 ) )->m_fData;
		pkScroll->m_afScrollSpeed[1] = dynamic_cast< FloatChunk* >( FindChunk( "y", ChunkType::FLOAT, -1 ) )->m_fData;

		m_pkGen = pkScroll;
	}
	else if( rstrType == "scale" )
	{
		TextureMatrixGenScale *pkScroll = new TextureMatrixGenScale;

		pkScroll->m_afScale[0] = dynamic_cast< FloatChunk* >( FindChunk( "x", ChunkType::FLOAT, -1 ) )->m_fData;
		pkScroll->m_afScale[1] = dynamic_cast< FloatChunk* >( FindChunk( "y", ChunkType::FLOAT, -1 ) )->m_fData;

		m_pkGen = pkScroll;
	}
	else if( rstrType == "stretch" )
	{
		TextureMatrixGenStretch *pkStretch = new TextureMatrixGenStretch;

		const std::string &rstrFunc = dynamic_cast< StringChunk* >( FindChunk( "func", ChunkType::STRING, -1 ) )->m_strData;

		if( rstrFunc == "sin" )
			pkStretch->m_pfTable = TextureMatrixGen::s_afSinTable;
		else if( rstrFunc == "square" )
			pkStretch->m_pfTable = TextureMatrixGen::s_afSquareTable;
		else if( rstrFunc == "triangle" )
			pkStretch->m_pfTable = TextureMatrixGen::s_afTriangleTable;
		else if( rstrFunc == "sawtooth" )
			pkStretch->m_pfTable = TextureMatrixGen::s_afSawToothTable;
		else if( rstrFunc == "inversesawtooth" )
			pkStretch->m_pfTable = TextureMatrixGen::s_afInverseSawToothTable;
		else
		{
			neolog << LogLevel( ERROR ) << "*** Invalid texmatrixgen chunk: Invalid func name for stretch" << endl;
			delete pkStretch;
			return -1;
		}

		pkStretch->m_fBase      = dynamic_cast< FloatChunk* >( FindChunk( "base",      ChunkType::FLOAT, -1 ) )->m_fData;
		pkStretch->m_fAmplitude = dynamic_cast< FloatChunk* >( FindChunk( "amplitude", ChunkType::FLOAT, -1 ) )->m_fData;
		pkStretch->m_fPhase     = dynamic_cast< FloatChunk* >( FindChunk( "phase",     ChunkType::FLOAT, -1 ) )->m_fData;
		pkStretch->m_fFrequency = dynamic_cast< FloatChunk* >( FindChunk( "frequency", ChunkType::FLOAT, -1 ) )->m_fData;

		m_pkGen = pkStretch;
	}
	else if( rstrType == "transform" )
	{
		TextureMatrixGenTransform *pkTransform = new TextureMatrixGenTransform;

		Matrix &rkMat = dynamic_cast< MatrixChunk* >( FindChunk( "", ChunkType::MATRIX, -1 ) )->m_kData;
		
		for( int v = 0; v < 2; ++v )
			for( int u = 0; u < 2; ++u )
				pkTransform->m_aafRotate[v][u] = rkMat[v][u];

		for( int i = 0; i < 2; ++i )
			pkTransform->m_afTranslate[i] = rkMat[i][2];

		m_pkGen = pkTransform;
	}
	else if( rstrType == "turbulence" )
	{
		TextureMatrixGenTurbulence *pkTurb = new TextureMatrixGenTurbulence;

		pkTurb->m_fBase      = dynamic_cast< FloatChunk* >( FindChunk( "base",      ChunkType::FLOAT, -1 ) )->m_fData;
		pkTurb->m_fAmplitude = dynamic_cast< FloatChunk* >( FindChunk( "amplitude", ChunkType::FLOAT, -1 ) )->m_fData;
		pkTurb->m_fPhase     = dynamic_cast< FloatChunk* >( FindChunk( "phase",     ChunkType::FLOAT, -1 ) )->m_fData;
		pkTurb->m_fFrequency = dynamic_cast< FloatChunk* >( FindChunk( "frequency", ChunkType::FLOAT, -1 ) )->m_fData;

		m_pkGen = pkTurb;
	}
	else if( rstrType == "rotate" )
	{
		TextureMatrixGenRotate *pkRot = new TextureMatrixGenRotate;

		pkRot->m_fRotateSpeed = dynamic_cast< FloatChunk* >( FindChunk( "speed", ChunkType::FLOAT, -1 ) )->m_fData;

		m_pkGen = pkRot;
	}
	else
	{
		neolog << LogLevel( ERROR ) << "*** Invalid texmatrixgen chunk: Invalid type [" << rstrType << "]" << endl;
		return -1;
	}

	return 1;
}


}; // namespace NeoChunkIO


