/***************************************************************************
                       device.h  -  aRts sound device
                             -------------------
    begin                : Sun Jun 06 2004
    copyright            : (C) 2004 by Adam Butcher
    email                : adam.j.butcher@virgin.net
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevALSA, device.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEARTS_DEVICE_H
#define __NEARTS_DEVICE_H


#include <neoengine/audio.h>
#include <pthread.h>

namespace NeoArts
{


/**
  * \brief aRts device implementation
  * \author Adam Butcher (adam.j.butcher@virgin.net)
  */
class AudioDevice : public NeoEngine::AudioDevice
{
	friend class Sound;

	protected:
	
		bool											   m_bInitialized;
		pthread_mutex_t									   m_kMutex;

	public:

		/**
		* \param pkFileManager                             File manager, will use core global object if null (default)
		*/
		                                                   AudioDevice( NeoEngine::FileManager *pkFileManager = 0 );

		/**
		*/
		virtual                                           ~AudioDevice();

		/**
		* Open the audio device
		* \param rkWindow                                  Window to initialize sound device in
		*/
		virtual bool                                       Open( const NeoEngine::RenderWindow &rkWindow );

		/**
		* Close the audio device
		*/
		virtual void                                       Close();

		/**
		 * Mutex Lock/Unlock
		 */
		inline void									       Lock()   { pthread_mutex_lock( &m_kMutex ); }
		inline void									       Unlock() { pthread_mutex_unlock( &m_kMutex ); }
		
		/**
		* Load a sound file. Implement in derived class
		* \param rstrFilename                              File name
		* \return                                          Ptr to sound
		*/
		virtual NeoEngine::Sound                          *LoadSound( const std::string &rstrFilename );
};


};

#endif
