/***************************************************************************
                    device.cpp  -  Direct3D9 render device
                             -------------------
    begin                : Sun Mar 9 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, device.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "device.h"
#include "buffermanager.h"
#include "framebuffertarget.h"
#include "pixelbuffertarget.h"

#include <neoengine/font.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoD3D9
{


extern bool g_bProcessMsg;

bool Device::s_bWindowClassRegistered = false;


Device::Device( FileManager *pkFileManager, InputManager *pkInputManager ) :
	RenderDevice( pkFileManager, pkInputManager ),
	m_pkD3D( 0 ),
	m_pkDevice( 0 ),
	m_ppkTMU( 0 ),
	m_uiNumTMUs( 0 ),
	m_pkZBufferState( 0 ),
	m_pkVertexBufferManager( 0 ),
	m_pkVertexBufferStore( 0 ),
	m_pkPolygonBufferManager( 0 ),
	m_pkPolygonBufferStore( 0 ),
	m_pkVertexShader( 0 ),
	m_pkFragmentShader( 0 ),
	m_ppkLights( new Light*[ 128 ] ),
	m_uiAllocatedLights( 128 ),
	m_uiCurLight( 0 ),
	m_uiMaxLights( 0 ),
	m_uiNumLights( 0 ),
	m_pkCurTarget( 0 ),
	m_pkFrameBufferTarget( 0 ),
	m_pkPixelBufferTarget( 0 ),
	m_bForceDefaultMaterial( false ),
	m_bIgnoreKill( false ),
	m_bShowCursor( true ),
	m_bDrawOutlines( false ),
	m_bDrawPolygons( true ),
	m_bDrawNormals( false ),
	m_bDrawTangents( false )
{
	//Setup BGRA color ordering
	Color32::s_uiShift[0]     = 16;
	Color32::s_uiShift[1]     = 8;
	Color32::s_uiShift[2]     = 0;
	Color32::s_uiShift[3]     = 24;

	Color32::s_uiComponent[0] = 2;
	Color32::s_uiComponent[1] = 1;
	Color32::s_uiComponent[2] = 0;
	Color32::s_uiComponent[3] = 3;

	//Set font subpixel alignment
	Font::s_fSubPixelOffset   = 0.5f;

	//Set default values of 3Mb and 2Mb
	m_pkVertexBufferManager  = new BufferManager( 3145728 );
	m_pkPolygonBufferManager = new BufferManager( 2097152 );
	
	g_bProcessMsg = true;
}


Device::~Device()
{
	Close();

	g_bProcessMsg = false;

	delete m_pkFrameBufferTarget;
	delete m_pkPixelBufferTarget;

	delete [] m_ppkLights;

	delete m_pkVertexBufferManager;
	delete m_pkPolygonBufferManager;
}


RenderDevice::CULLMODE Device::SetCullMode( CULLMODE eCullMode )
{
	CULLMODE ePrevMode = m_eCullMode;

	if( eCullMode == CULLNONE )
		m_pkDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
	else if( eCullMode == CULLBACK )
		m_pkDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW );
	else if( eCullMode == CULLFRONT )
		m_pkDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
	else
		neolog << LogLevel( ERROR ) << "*** cull mode not implemented" << endl;

	return ePrevMode;
}


}; // namespace NeoD3D9

