/***************************************************************************
                 execute.cpp  -  Execute render queue operations
                             -------------------
    begin                : Mon Mar 10 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, execute.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#define NOMINMAX

#include "device.h"
#include "renderqueue.h"
#include "op.h"
#include "vertexbuffer.h"
#include "polygonbuffer.h"
#include "polygonstripbuffer.h"
#include "texture.h"
#include "textureunit.h"
#include "zbufferstate.h"
#include "bufferregion.h"
#include "shader.h"
#include "rendertarget.h"

#include <neoengine/vertex.h>
#include <neoengine/polygon.h>
#include <neoengine/light.h>
#include <neoengine/scenenode.h>

#include <limits>

using namespace std;
using namespace NeoEngine;


namespace NeoD3D9
{


//D3DBLEND aeSrcBlendTable[16]  = { D3DBLEND_ONE,  D3DBLEND_ZERO, D3DBLEND_ZERO, D3DBLEND_ZERO, D3DBLEND_DESTCOLOR, D3DBLEND_INVDESTCOLOR, D3DBLEND_ZERO, D3DBLEND_SRCCOLOR, D3DBLEND_ZERO,     D3DBLEND_SRCALPHA,    D3DBLEND_INVSRCALPHA, D3DBLEND_ZERO,      D3DBLEND_DESTALPHA,    D3DBLEND_INVDESTALPHA, D3DBLEND_ZERO, D3DBLEND_SRCALPHASAT };
//D3DBLEND aeDestBlendTable[16] = { D3DBLEND_ZERO, D3DBLEND_ONE , D3DBLEND_ZERO, D3DBLEND_ZERO, D3DBLEND_SRCCOLOR,  D3DBLEND_INVSRCCOLOR,  D3DBLEND_ZERO, D3DBLEND_ZERO,     D3DBLEND_SRCALPHA, D3DBLEND_INVSRCALPHA, D3DBLEND_ZERO,        D3DBLEND_DESTALPHA, D3DBLEND_INVDESTALPHA, D3DBLEND_ZERO,         D3DBLEND_ZERO, D3DBLEND_ZERO        };

//TEMPORARY TEST: Disable destalpha blends
//                                0x0            0x1            0x2            0x3            0x4                 0x5                    0x6            0x7                0x8                0x9                   0xa                   0xb            0xc            0xd                   0xe            0xf
D3DBLEND aeSrcBlendTable[16]  = { D3DBLEND_ONE,  D3DBLEND_ZERO, D3DBLEND_ZERO, D3DBLEND_ZERO, D3DBLEND_DESTCOLOR, D3DBLEND_INVDESTCOLOR, D3DBLEND_ZERO, D3DBLEND_SRCCOLOR, D3DBLEND_ZERO,     D3DBLEND_SRCALPHA,    D3DBLEND_INVSRCALPHA, D3DBLEND_ONE,  D3DBLEND_ZERO, D3DBLEND_SRCALPHASAT, D3DBLEND_ONE,  D3DBLEND_ONE  };
D3DBLEND aeDestBlendTable[16] = { D3DBLEND_ZERO, D3DBLEND_ONE , D3DBLEND_ZERO, D3DBLEND_ZERO, D3DBLEND_SRCCOLOR,  D3DBLEND_INVSRCCOLOR,  D3DBLEND_ZERO, D3DBLEND_ZERO,     D3DBLEND_SRCALPHA, D3DBLEND_INVSRCALPHA, D3DBLEND_ZERO,        D3DBLEND_ONE,  D3DBLEND_ZERO, D3DBLEND_ZERO,        D3DBLEND_ZERO, D3DBLEND_ZERO };


#ifndef NED3DPOLYGONBUFFER_USEMINMAXINDEX

#define RENDERPASS()                                                                                                                  \
                                                                                                                                      \
if( pkOp->m_ePrimitive == RenderPrimitive::TRIANGLESTRIP )                                                                            \
{                                                                                                                                     \
	PolygonStripBufferD3D *pkStripBuffer = (PolygonStripBufferD3D*)((NeoEngine::PolygonStripBuffer*)pkOp->m_pkPolygonStripBuffer);    \
                                                                                                                                      \
	m_pkDevice->m_pkDevice->DrawIndexedPrimitive( D3DPT_TRIANGLESTRIP,                                                                \
	                                              pkVertexBuffer->m_pkRegion->m_uiRegionOffset / pkVertexBuffer->GetVertexSize(),     \
	                                              0,                                                                                  \
	                                              pkVertexBuffer->GetNumVertices(),                                                   \
	                                              pkStripBuffer->m_pkRegion->m_uiRegionOffset / 2,                                    \
	                                              pkOp->m_uiNumPrimitives );                                                          \
}                                                                                                                                     \
else                                                                                                                                  \
{                                                                                                                                     \
	m_pkDevice->m_pkDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST,                                                                 \
	                                              pkVertexBuffer->m_pkRegion->m_uiRegionOffset / pkVertexBuffer->GetVertexSize(),     \
	                                              0,                                                                                  \
                                                  pkVertexBuffer->GetNumVertices(),                                                   \
                                                  pkPolygonBuffer->m_pkRegion->m_uiRegionOffset / 2, pkOp->m_uiNumPrimitives );       \
}

#else

#define RENDERPASS()                                                                                                                  \
                                                                                                                                      \
if( pkOp->m_ePrimitive == RenderPrimitive::TRIANGLESTRIP )                                                                            \
{                                                                                                                                     \
	PolygonStripBufferD3D *pkStripBuffer = (PolygonStripBufferD3D*)((NeoEngine::PolygonStripBuffer*)pkOp->m_pkPolygonStripBuffer);    \
	                                                                                                                                  \
	m_pkDevice->m_pkDevice->DrawIndexedPrimitive( D3DPT_TRIANGLESTRIP,                                                                \
	                                              pkVertexBuffer->m_pkRegion->m_uiRegionOffset / pkVertexBuffer->GetVertexSize(),     \
	                                              pkStripBuffer->m_uiMinIndex,                                                        \
	                                              pkStripBuffer->m_uiMaxIndex - pkStripBuffer->m_uiMinIndex,                          \
	                                              pkStripBuffer->m_pkRegion->m_uiRegionOffset / 2,                                    \
	                                              pkOp->m_uiNumPrimitives );                                                          \
}                                                                                                                                     \
else                                                                                                                                  \
{                                                                                                                                     \
	m_pkDevice->m_pkDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST,                                                                 \
	                                              pkVertexBuffer->m_pkRegion->m_uiRegionOffset / pkVertexBuffer->GetVertexSize(),     \
	                                              pkPolygonBuffer->m_uiMinIndex,                                                      \
                                                  pkPolygonBuffer->m_uiMaxIndex - pkPolygonBuffer->m_uiMinIndex,                      \
                                                  pkPolygonBuffer->m_pkRegion->m_uiRegionOffset / 2, pkOp->m_uiNumPrimitives );       \
}

#endif


void RenderQueue::ExecuteOps()
{
	if( !m_iNumOps )
		return;

	static D3DVIEWPORT9 kViewport;
	static D3DMATERIAL9 kD3DMaterial;

	int iOp = 0;
	bool bLighting = false;

	static Matrix kView;
	static Matrix kNativeModel;

	kView = m_pkDevice->GetViewMatrix();

	if( m_bIsSky )
	{
		kView[0][3] = kView[1][3] = kView[2][3] = 0.0f;

		const Viewport &rkViewport = m_pkDevice->GetViewport();

		kViewport.X      = rkViewport.m_iX;
		kViewport.Y      = rkViewport.m_iY;
		kViewport.Width  = rkViewport.m_iWidth;
		kViewport.Height = rkViewport.m_iHeight;
		kViewport.MinZ   = 1.0f;
		kViewport.MaxZ   = 1.0f;

		m_pkDevice->m_pkDevice->SetViewport( &kViewport );
	}

	kView.Transpose();

	m_pkDevice->m_pkDevice->SetTransform( D3DTS_VIEW, (D3DMATRIX*)((float*)kView) );

	kView.Transpose();


	if( !m_bIsSky && ( m_pkDevice->m_uiNumLights != 0 ) )
	{
		bLighting = true;

		unsigned int uiLight = 0;
		unsigned int uiLights = MIN< unsigned int >( m_pkDevice->m_uiNumLights, m_pkDevice->m_uiMaxLights );
		
		for( uiLight = 0; uiLight < uiLights; ++uiLight )
		{
			D3DLIGHT9 kLight;
			Light *pkLight = m_pkDevice->m_ppkLights[ uiLight ];

			memset( &kLight, 0, sizeof( D3DLIGHT9 ) );

			kLight.Diffuse.r = pkLight->m_kDiffuse.r;
			kLight.Diffuse.g = pkLight->m_kDiffuse.g;
			kLight.Diffuse.b = pkLight->m_kDiffuse.b;
			kLight.Diffuse.a = pkLight->m_kDiffuse.a;

			kLight.Specular.r = pkLight->m_kSpecular.r;
			kLight.Specular.g = pkLight->m_kSpecular.g;
			kLight.Specular.b = pkLight->m_kSpecular.b;
			kLight.Specular.a = pkLight->m_kSpecular.a;

			kLight.Ambient.r = pkLight->m_kAmbient.r;
			kLight.Ambient.g = pkLight->m_kAmbient.g;
			kLight.Ambient.b = pkLight->m_kAmbient.b;
			kLight.Ambient.a = pkLight->m_kAmbient.a;

			kLight.Falloff      = 1.0f;
			kLight.Attenuation0 = pkLight->m_fConstantAttenuation;
			kLight.Attenuation1 = pkLight->m_fLinearAttenuation;
			kLight.Attenuation2 = pkLight->m_fQuadraticAttenuation;

			if( pkLight->GetType() == Light::DIRECTIONAL )
			{
				Vector3d kDir = pkLight->GetSceneNode() ? -pkLight->GetSceneNode()->GetWorldTransform().GetColumn(2) : Vector3d( 0.0f, 0.0f, -1.0f );

				kLight.Type = D3DLIGHT_DIRECTIONAL;
				kLight.Direction.x = kDir.x;
				kLight.Direction.y = kDir.y;
				kLight.Direction.z = kDir.z;
			}
			else if( pkLight->GetType() == Light::POINT )
			{
				Vector3d kPos = pkLight->GetSceneNode() ? pkLight->GetSceneNode()->GetWorldTranslation() : Vector3d::ORIGO;
				
				kLight.Type = D3DLIGHT_POINT;
				kLight.Position.x = kPos.x;
				kLight.Position.y = kPos.y;
				kLight.Position.z = kPos.z;
				kLight.Range = sqrtf( numeric_limits< float >::max() );
			}

			m_pkDevice->m_pkDevice->SetLight( uiLight, &kLight );
			m_pkDevice->m_pkDevice->LightEnable( uiLight, true );
		}

		for( ; uiLight < m_pkDevice->m_uiMaxLights; ++uiLight )
			m_pkDevice->m_pkDevice->LightEnable( uiLight, false );

		m_pkDevice->m_pkDevice->SetRenderState( D3DRS_AMBIENT, Color32( m_pkDevice->m_kAmbientLight ) );
		m_pkDevice->m_pkDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	}
	else
	{
		bLighting = false;
		m_pkDevice->m_pkDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	}

	unsigned int *puiOpIndex = m_kSorter.GetIndices();

	bool bUseSort = !( m_bIsSky || ( m_pkDevice->m_pkCurTarget->m_uiFrameFlags & RenderDevice::NOSORT ) );

	for( iOp = 0; iOp < m_iNumOps; ++iOp )
	{
		int iIndex =  !bUseSort ? iOp : puiOpIndex[ iOp ];

		RenderPrimitive *pkOp = &m_ppkOpArray[ iIndex >> OpArray::ARRAYSHIFT ]->m_akOps[ iIndex & OpArray::ARRAYMASK ];

		if( ( pkOp->m_ePrimitive != RenderPrimitive::TRIANGLES ) && ( pkOp->m_ePrimitive != RenderPrimitive::TRIANGLESTRIP ) )
			continue;

		if( m_bIsSky )
			pkOp->m_kModelMatrix[0][3] = pkOp->m_kModelMatrix[1][3] = pkOp->m_kModelMatrix[2][3] = 0.0f;

		kNativeModel = pkOp->m_kModelMatrix;

		kNativeModel.Transpose();


		VertexBufferD3D  *pkVertexBuffer  = static_cast< VertexBufferD3D*  >( (NeoEngine::VertexBuffer* )pkOp->m_pkVertexBuffer  );
		PolygonBufferD3D *pkPolygonBuffer = static_cast< PolygonBufferD3D* >( (NeoEngine::PolygonBuffer*)pkOp->m_pkPolygonBuffer );

		assert( !( pkVertexBuffer->m_pkRegion->m_uiRegionOffset % pkVertexBuffer->GetVertexSize() ) );

		m_pkDevice->m_pkDevice->SetStreamSource( 0, m_pkDevice->GetVertexBufferStore(), 0, pkVertexBuffer->GetVertexSize() );

		MaterialPass *pkMat = !pkOp->m_uiPass ? pkOp->m_pkMaterial : pkOp->m_pkMaterial->m_vpkPasses[ pkOp->m_uiPass - 1 ];

		if( pkMat->m_pkVertexShader )
		{
			//Programmable pipeline
			m_pkDevice->m_uiCurLight = 0;

			D3D9Shader *pkShader = static_cast< D3D9Shader* >( (NeoEngine::Shader*)pkMat->m_pkVertexShader );
			
			pkShader->Bind( pkOp, kView );

			m_pkDevice->m_pkDevice->SetVertexDeclaration( pkVertexBuffer->m_pkVertexDeclaration );

			m_pkDevice->m_pkVertexShader = pkShader;
		}
		else
		{
			if( m_pkDevice->m_pkVertexShader )
			{
				m_pkDevice->m_pkVertexShader->Unbind();
				m_pkDevice->m_pkVertexShader = 0;
			}

			//Fixed function pipeline
			m_pkDevice->m_pkDevice->SetTransform( D3DTS_WORLD, (D3DMATRIX*)((float*)kNativeModel) );
			m_pkDevice->m_pkDevice->SetFVF( pkVertexBuffer->m_uiD3DFVF );
		}

		TextureLayer       *pkLayer   = (TextureLayer*)pkMat;
		NeoEngine::Texture *pkTexture = pkLayer->m_pkTexture;

		if( pkTexture )
		{
			m_pkDevice->m_ppkTMU[0]->SetTexture( ((Texture*)pkTexture)->m_pkD3DTexture );
			m_pkDevice->m_ppkTMU[0]->SetFiltering( pkTexture->GetFiltering() );
			m_pkDevice->m_ppkTMU[0]->SetMaxAnisotropy( pkTexture->GetMaxAnisotropy() );
			m_pkDevice->m_ppkTMU[0]->SetAddressingMode( pkLayer->m_uiUVAddress );

			switch( pkLayer->m_eTexCoordGen )
			{
				case TextureLayer::REFLECTION:
					m_pkDevice->m_ppkTMU[0]->SetUVLayer( D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR | pkLayer->m_uiUVLayer );
					break;

				case TextureLayer::NORMAL:
					m_pkDevice->m_ppkTMU[0]->SetUVLayer( D3DTSS_TCI_CAMERASPACENORMAL | pkLayer->m_uiUVLayer );
					break;

				default:
					m_pkDevice->m_ppkTMU[0]->SetUVLayer( pkLayer->m_uiUVLayer );
					break;
			}
		}
		else
			m_pkDevice->m_ppkTMU[0]->SetTexture( 0 );

		if( pkMat->m_vpkTexMatrixGen.size() )
		{
			Matrix kMat;

			vector< TextureMatrixGen* >::iterator ppkGen    = pkMat->m_vpkTexMatrixGen.begin();
			vector< TextureMatrixGen* >::iterator ppkGenEnd = pkMat->m_vpkTexMatrixGen.end();

			for( ; ppkGen != ppkGenEnd; ++ppkGen )
				(*ppkGen)->GenerateMatrix( &kMat );

			m_pkDevice->m_ppkTMU[0]->SetTransform( &kMat, pkVertexBuffer->m_auiTexLayerDims[ pkMat->m_uiUVLayer ] );
		}
		else
			m_pkDevice->m_ppkTMU[0]->SetTransform( 0, pkVertexBuffer->m_auiTexLayerDims[ pkMat->m_uiUVLayer ] );

		if( m_bIsSky )
			m_pkDevice->m_pkZBufferState->SetMode( ZBufferMode::LEQUAL, ZBufferMode::DISABLED );
		else
			m_pkDevice->m_pkZBufferState->SetMode( pkMat->m_kZBufferMode.Get() & ZBufferMode::FUNCBITS, pkMat->m_kZBufferMode.Get() & ZBufferMode::MODEBITS );

		m_pkDevice->m_pkDevice->SetRenderState( D3DRS_SRCBLEND, aeSrcBlendTable[ pkMat->m_kBlendMode.Get() & BlendMode::SRCBITS ] );
		m_pkDevice->m_pkDevice->SetRenderState( D3DRS_DESTBLEND, aeDestBlendTable[ ( pkMat->m_kBlendMode.Get() & BlendMode::DESTBITS ) >> 4 ] );

		if( pkMat->m_pkFragmentShader )
		{
			D3D9Shader *pkShader = static_cast< D3D9Shader* >( (NeoEngine::Shader*)pkMat->m_pkFragmentShader );
			
			pkShader->Bind( pkOp, kView );

			m_pkDevice->m_pkFragmentShader = pkShader;

			vector< TextureLayer* >::iterator ppkLayer    = pkMat->m_vpkTextureLayers.begin();
			vector< TextureLayer* >::iterator ppkLayerEnd = pkMat->m_vpkTextureLayers.end();

			int iLayer = 1;

			//We assume enough textureunits are available!
			for( ; ppkLayer != ppkLayerEnd; ++iLayer, ++ppkLayer )
			{
				if( (*ppkLayer)->m_pkTexture )
				{
					m_pkDevice->m_ppkTMU[ iLayer ]->SetTexture( ((Texture*)((NeoEngine::Texture*)(*ppkLayer)->m_pkTexture))->m_pkD3DTexture );
					m_pkDevice->m_ppkTMU[ iLayer ]->SetFiltering( (*ppkLayer)->m_pkTexture->GetFiltering() );
					m_pkDevice->m_ppkTMU[ iLayer ]->SetMaxAnisotropy( (*ppkLayer)->m_pkTexture->GetMaxAnisotropy() );
				}
				else
					m_pkDevice->m_ppkTMU[ iLayer ]->SetTexture( 0 );
			}
		}
		else
		{
			if( m_pkDevice->m_pkFragmentShader )
			{
				m_pkDevice->m_pkFragmentShader->Unbind();
				m_pkDevice->m_pkFragmentShader = 0;
			}

			//If vertex format has color data, set alpha to modulate
			if( !( pkVertexBuffer->m_uiD3DFVF & D3DFVF_DIFFUSE ) )
			{
				if( bLighting )
				{
					kD3DMaterial.Ambient.r = pkMat->m_kAmbient.r;
					kD3DMaterial.Ambient.g = pkMat->m_kAmbient.g;
					kD3DMaterial.Ambient.b = pkMat->m_kAmbient.b;
					kD3DMaterial.Ambient.a = pkMat->m_kAmbient.a;

					kD3DMaterial.Diffuse.r = pkMat->m_kDiffuse.r;
					kD3DMaterial.Diffuse.g = pkMat->m_kDiffuse.g;
					kD3DMaterial.Diffuse.b = pkMat->m_kDiffuse.b;
					kD3DMaterial.Diffuse.a = pkMat->m_kDiffuse.a;

					kD3DMaterial.Specular.r = pkMat->m_kSpecular.r;
					kD3DMaterial.Specular.g = pkMat->m_kSpecular.g;
					kD3DMaterial.Specular.b = pkMat->m_kSpecular.b;
					kD3DMaterial.Specular.a = pkMat->m_kSpecular.a;

					kD3DMaterial.Emissive.r = pkMat->m_kEmission.r;
					kD3DMaterial.Emissive.g = pkMat->m_kEmission.g;
					kD3DMaterial.Emissive.b = pkMat->m_kEmission.b;
					kD3DMaterial.Emissive.a = pkMat->m_kEmission.a;

					kD3DMaterial.Power = pkMat->m_fShininess * 128.0f;

					m_pkDevice->m_pkDevice->SetMaterial( &kD3DMaterial );

					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_AMBIENTMATERIALSOURCE,  D3DMCS_MATERIAL );
					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_DIFFUSEMATERIALSOURCE,  D3DMCS_MATERIAL );
					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_SPECULARMATERIALSOURCE, D3DMCS_MATERIAL );
					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_COLORVERTEX,            FALSE           );
					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_SPECULARENABLE,         TRUE            );
				
					m_pkDevice->m_pkDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
				}
			}
			else
			{
				m_pkDevice->m_pkDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );

				m_pkDevice->m_pkDevice->SetRenderState( D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_COLOR1 );
				m_pkDevice->m_pkDevice->SetRenderState( D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_COLOR1 );
				m_pkDevice->m_pkDevice->SetRenderState( D3DRS_COLORVERTEX,           TRUE          );
				m_pkDevice->m_pkDevice->SetRenderState( D3DRS_SPECULARENABLE,        FALSE         );
			}

			vector< TextureLayer* >::iterator ppkLayer    = pkMat->m_vpkTextureLayers.begin();
			vector< TextureLayer* >::iterator ppkLayerEnd = pkMat->m_vpkTextureLayers.end();

			int iLayer = 0;
			
			for( iLayer = 0; ppkLayer != ppkLayerEnd; ++iLayer, ++ppkLayer )
			{
				pkLayer   = *ppkLayer;
				pkTexture = pkLayer->m_pkTexture;

				if( ( ( iLayer + 1 ) >= (int)m_pkDevice->m_uiNumTMUs ) || !m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetBlendMode( pkLayer->m_kBlendMode.Get() & BlendMode::SRCBITS, pkLayer->m_kBlendMode.Get() & BlendMode::DESTBITS, pkLayer->m_kBlendMode.m_fFactor ) )
				{
					for( ++iLayer; iLayer < (int)m_pkDevice->m_uiNumTMUs; ++iLayer )
						m_pkDevice->m_ppkTMU[ iLayer ]->Disable();

					//Render up to this layer
					RENDERPASS()

					//Set new base layer
					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_SRCBLEND, aeSrcBlendTable[ pkLayer->m_kBlendMode.Get() & BlendMode::SRCBITS ] );
					m_pkDevice->m_pkDevice->SetRenderState( D3DRS_DESTBLEND, aeDestBlendTable[ ( pkLayer->m_kBlendMode.Get() & BlendMode::DESTBITS ) >> 4 ] );

					if( !m_bIsSky )
						m_pkDevice->m_pkZBufferState->SetMode( ZBufferMode::EQUAL, ZBufferMode::DISABLED );

					iLayer = -1;
				}

				if( pkTexture )
				{
					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetTexture( ((Texture*)pkTexture)->m_pkD3DTexture );
					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetFiltering( pkTexture->GetFiltering() );
					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetMaxAnisotropy( pkTexture->GetMaxAnisotropy() );
					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetAddressingMode( pkLayer->m_uiUVAddress );

					switch( pkLayer->m_eTexCoordGen )
					{
						case TextureLayer::REFLECTION:
							m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetUVLayer( D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR | pkLayer->m_uiUVLayer );
							break;

						case TextureLayer::NORMAL:
							m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetUVLayer( D3DTSS_TCI_CAMERASPACENORMAL | pkLayer->m_uiUVLayer );
							break;

						default:
							m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetUVLayer( pkLayer->m_uiUVLayer );
							break;
					}
				}
				else
					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetTexture( 0 );

				if( pkLayer->m_vpkTexMatrixGen.size() )
				{
					Matrix kMat;

					vector< TextureMatrixGen* >::iterator ppkGen    = pkLayer->m_vpkTexMatrixGen.begin();
					vector< TextureMatrixGen* >::iterator ppkGenEnd = pkLayer->m_vpkTexMatrixGen.end();

					for( ; ppkGen != ppkGenEnd; ++ppkGen )
						(*ppkGen)->GenerateMatrix( &kMat );

					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetTransform( &kMat, pkVertexBuffer->m_auiTexLayerDims[ pkLayer->m_uiUVLayer ] );
				}
				else
					m_pkDevice->m_ppkTMU[ iLayer + 1 ]->SetTransform( 0, pkVertexBuffer->m_auiTexLayerDims[ pkLayer->m_uiUVLayer ] );
			}

			for( ++iLayer; iLayer < (int)m_pkDevice->m_uiNumTMUs; ++iLayer )
				m_pkDevice->m_ppkTMU[ iLayer ]->Disable();
		}

		if( m_pkDevice->m_pkVertexShader && ( pkMat->m_uiMaxLights > 0 ) && ( m_pkDevice->m_uiNumLights > pkMat->m_uiMaxLights ) )
		{
			bool bFinal = false;

			do
			{
				RENDERPASS()

				if( bFinal )
					break;

				m_pkDevice->m_pkDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_ONE );
				m_pkDevice->m_pkDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );
				
				//m_pkDevice->m_kZBufferState.SetMode( ZBufferMode::EQUAL, ZBufferMode::DISABLED );

				m_pkDevice->m_uiCurLight += pkMat->m_uiMaxLights;

				m_pkDevice->m_pkVertexShader->BindLightParams( pkOp );

				if( m_pkDevice->m_pkFragmentShader )
					m_pkDevice->m_pkFragmentShader->BindLightParams( pkOp );

				bFinal = ( ( m_pkDevice->m_uiCurLight + pkMat->m_uiMaxLights ) >= m_pkDevice->m_uiNumLights );

			} while( true );
		}
		else
		{
			//Render final pass
			RENDERPASS()
		}
	}

	if( m_bIsSky )
	{
		kViewport.MinZ   = 0.0f;
		kViewport.MaxZ   = 1.0f;

		m_pkDevice->m_pkDevice->SetViewport( &kViewport );
	}
}


};

