/***************************************************************************
                    glue.cpp  -  Export symbols for engine
                             -------------------
    begin                : Sun Mar 9 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, glue.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "device.h"
#include "link.h"

using namespace NeoEngine;
using namespace NeoD3D9;


namespace NeoD3D9
{

#ifdef BUILD_STATIC
#  define D3D9EXPORT

ModuleStatic *g_pkStaticModule = 0;

#else
#  ifdef WIN32
#    define D3D9EXPORT __declspec(dllexport)
#  else
#    define D3D9EXPORT
#  endif

extern "C"
{

#endif

D3D9EXPORT RenderDevice *CreateDevice( FileManager *pkFileManager, InputManager *pkInputManager )
{
	return new Device( pkFileManager, pkInputManager );
}


D3D9EXPORT void GetVersion( std::string *pstrName, int *piMajor, int *piMinor, int *piRevision )
{
	std::string strDevInfo =
#  ifdef _DEBUG
	"D3D9 [DEBUG]";
#  else
	"D3D9";
#  endif

	*pstrName   = strDevInfo;
	*piMajor    = NEOENGINEVERSION_MAJOR;
	*piMinor    = NEOENGINEVERSION_MINOR;
	*piRevision = NEOENGINEVERSION_REVISION;
}

D3D9EXPORT int Initialize()
{
	return 0;
}

D3D9EXPORT int Shutdown()
{
#ifdef BUILD_STATIC
	delete g_pkStaticModule;
#endif
	return 0;
}

#ifdef BUILD_STATIC

#else

#  ifndef WIN32_LEAN_AND_MEAN
#    define WIN32_LEAN_AND_MEAN
#  endif

#  include <windows.h>

BOOL APIENTRY DllMain( HANDLE hModule, DWORD dwReason, LPVOID pReserved )
{
	switch( dwReason )
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}

	return TRUE;
}

}

#endif



#ifdef BUILD_STATIC


Link::Link()
{
#if defined(WIN32) && defined(_DEBUG)
	g_pkStaticModule = new ModuleStatic( "neodevd3d9_debug" );
#else
	g_pkStaticModule = new ModuleStatic( "neodevd3d9" );
#endif

	g_pkStaticModule->m_Symbols.Insert( "GetVersion",   (void**)GetVersion   );
	g_pkStaticModule->m_Symbols.Insert( "CreateDevice", (void**)CreateDevice );
	g_pkStaticModule->m_Symbols.Insert( "Initialize",   (void**)Initialize   );
	g_pkStaticModule->m_Symbols.Insert( "Shutdown",     (void**)Shutdown     );

	NeoEngine::Core::GetModuleManager()->InsertModule( g_pkStaticModule );
}


#endif


};

