/***************************************************************************
                 open.cpp  -  Open and initialize render window
                             -------------------
    begin                : Sun Mar 9 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, open.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "device.h"
#include "windowproc.h"
#include "framebuffertarget.h"
#include "pixelbuffertarget.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


#define D3D9DEVICE_WINDOWCLASSNAME        "ned3d9devicewnd"


namespace NeoD3D9
{


bool Device::Open( const RenderWindow &rkWindow )
{
	Close();

	m_kWindow.m_strWindowName = rkWindow.m_strWindowName;
	m_kWindow.m_uiAdapter     = rkWindow.m_uiAdapter;
	m_kWindow.m_kResolution   = rkWindow.m_kResolution;

	if( m_kWindow.m_kResolution.m_uiBPP < 15 )
		m_kWindow.m_kResolution.m_uiBPP = 16;
	else if( m_kWindow.m_kResolution.m_uiBPP > 32 )
		m_kWindow.m_kResolution.m_uiBPP = 32;

	m_kWindow.m_hInstance = GetModuleHandle( 0 );

	if( !m_kWindow.m_hWnd )
	{
		neolog << LogLevel( INFO ) << "Creating rendering window" << endl;

		//Store dimensions and flags
		m_kWindow.m_uiFlags  |= RenderWindow::DEVICECREATED;

		if( rkWindow.m_kCaps.IsSet( RenderCaps::FULLSCREEN ) )
			m_kWindow.m_kCaps.Set( RenderCaps::FULLSCREEN );

		if( !s_bWindowClassRegistered )
		{
			WNDCLASS wc;
			wc.style          = CS_OWNDC;
			wc.lpfnWndProc    = (WNDPROC)WindowProc;
			wc.cbClsExtra     = 0;
			wc.cbWndExtra     = 0;
			wc.hInstance      = (HINSTANCE)m_kWindow.m_hInstance;
			wc.hIcon          = LoadIcon( 0, IDI_WINLOGO );
			wc.hCursor        = LoadCursor( 0, IDC_ARROW );
			wc.hbrBackground  = 0;
			wc.lpszMenuName   = 0;
			wc.lpszClassName  = D3D9DEVICE_WINDOWCLASSNAME;

			if( !RegisterClass( &wc ) )
			{
				neolog << LogLevel( ERROR ) << "*** Unable to register window class" << endl;
				return false;
			}

			s_bWindowClassRegistered = true;
		}

		RECT kRect;

		kRect.left   = 0;
		kRect.top    = 0;
		kRect.right  = m_kWindow.m_kResolution.m_uiWidth;
		kRect.bottom = m_kWindow.m_kResolution.m_uiHeight;

		if( !m_kWindow.m_kCaps.IsSet( RenderCaps::FULLSCREEN ) )
		{
			AdjustWindowRect( &kRect, WS_OVERLAPPEDWINDOW, FALSE );

			neolog << LogLevel( DEBUG ) << "  rect [ " << kRect.left << " " << kRect.top << " -> " << kRect.right << " " << kRect.bottom << " ]" << endl;
		}

		m_kWindow.m_hWnd = CreateWindowEx( ( m_kWindow.m_kCaps.IsSet( RenderCaps::FULLSCREEN ) ? WS_EX_TOPMOST : 0 ), D3D9DEVICE_WINDOWCLASSNAME, m_kWindow.m_strWindowName.c_str(), ( !m_kWindow.m_kCaps.IsSet( RenderCaps::FULLSCREEN ) ? WS_OVERLAPPEDWINDOW : WS_POPUP ) | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, CW_USEDEFAULT, CW_USEDEFAULT, kRect.right - kRect.left, kRect.bottom - kRect.top, 0, 0, (HINSTANCE)m_kWindow.m_hInstance, this );

		if( !m_kWindow.m_hWnd )
		{
			neolog << LogLevel( ERROR ) << "*** Unable to create window" << endl;
			return false;
		}

		SetWindowLong( (HWND)m_kWindow.m_hWnd, GWL_USERDATA, (long)this );

		//Store desktop resolution
		HDC hDeskDC = GetDC( 0 );
		
		m_kWindow.m_kDesktopResolution.m_uiWidth   = GetDeviceCaps( hDeskDC, HORZRES );
		m_kWindow.m_kDesktopResolution.m_uiHeight  = GetDeviceCaps( hDeskDC, VERTRES );
		m_kWindow.m_kDesktopResolution.m_uiBPP     = GetDeviceCaps( hDeskDC, BITSPIXEL );

		ReleaseDC( 0, hDeskDC );
	}
	else
	{
		if( !( m_kWindow.m_uiFlags & RenderWindow::USEDIMENSIONS ) )
		{
			RECT kRect;

			GetClientRect( (HWND)m_kWindow.m_hWnd, &kRect );

			m_kWindow.m_kResolution.m_uiWidth  = kRect.right  - kRect.left;
			m_kWindow.m_kResolution.m_uiHeight = kRect.bottom - kRect.top;
		}
	}

	neolog << LogLevel( INFO ) << "Opening render device" << endl;

	if( !m_pkFrameBufferTarget )
		m_pkFrameBufferTarget = new FrameBufferRenderTarget( this );

	//if( pkCaps && pkCaps->IsSet( RenderCaps::PIXELBUFFER ) )
	{
		if( !m_pkPixelBufferTarget )
			m_pkPixelBufferTarget = new PixelBufferRenderTarget( this );

		m_kWindow.m_kCaps.Set( RenderCaps::PIXELBUFFER   );
		m_kWindow.m_kCaps.Set( RenderCaps::RENDERTEXTURE );
	}

	if( m_kWindow.m_kCaps.IsSet( RenderCaps::FULLSCREEN ) )
	{
		HDC hDC = GetDC( (HWND)m_kWindow.m_hWnd );

		m_kWindow.m_kResolution.m_uiBPP = GetDeviceCaps( hDC, BITSPIXEL );

		ReleaseDC( (HWND)m_kWindow.m_hWnd, hDC );
	}

	if( rkWindow.m_kCaps.IsSet( RenderCaps::STENCILBUFFER ) )
		m_kWindow.m_kCaps.Set( RenderCaps::STENCILBUFFER );

	return Initialize( rkWindow.m_kCaps );
}


}; // namespace NeoD3D9



