/***************************************************************************
               pixelbuffertarget.cpp  -  Pixel buffer render target
                             -------------------
    begin                : Fri May 23 2003
    copyright            : (C) 2003 by Keaton Mullis
    email                : kmullis@zianet.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, pixelbuffertarget.cpp

 The Initial Developer of the Original Code is Keaton Mullis.
 Portions created by Keaton Mullis are Copyright (C) 2003
 Keaton Mullis. All Rights Reserved.

 ***************************************************************************/

#include "pixelbuffertarget.h"
#include "pixelbuffer.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoD3D9
{


PixelBufferRenderTarget::PixelBufferRenderTarget( Device *pkDevice ) :
	RenderTarget( pkDevice ),
	m_pkCurBuffer( 0 )
{
	Deactivate();
}


PixelBufferRenderTarget::~PixelBufferRenderTarget()
{
	Deactivate();
}


void PixelBufferRenderTarget::Activate()
{
	if( IsActive() )
		return;

	if( m_pkCurBuffer )
		m_pkCurBuffer->Activate();

	RenderTarget::Activate();
}


void PixelBufferRenderTarget::Deactivate()
{
	if( !IsActive() )
		return;

	if( m_pkCurBuffer )
		m_pkCurBuffer->Deactivate();

	RenderTarget::Deactivate();
}


bool PixelBufferRenderTarget::SetPixelBuffer( unsigned int uiPixelBuffer )
{
	if( m_pkDevice->m_vpkPixelBuffers.size() )
	{
		vector< PixelBuffer* >::iterator ppkBuffer    = m_pkDevice->m_vpkPixelBuffers.begin();
		vector< PixelBuffer* >::iterator ppkBufferEnd = m_pkDevice->m_vpkPixelBuffers.end();

		for( ; ppkBuffer != ppkBufferEnd; ++ppkBuffer )
		{
			if( ( (*ppkBuffer)->GetTargetID() & 0x00FFFFFF ) == uiPixelBuffer )
			{
				m_pkCurBuffer = *ppkBuffer;
				return true;
			}
		}
	}

	neolog << LogLevel( ERROR ) << "*** unable to set pixel buffer: unable to locate buffer [" << uiPixelBuffer << "]" << endl;

	return false;
}


}; // namespace NeoD3D9

