/***************************************************************************
                  query.cpp  -  Queries for adapters and modes
                             -------------------
    begin                : Tue Oct 28 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, query.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#define NEEDGDI
#define COMPILE_MULTIMON_STUBS

#include <windows.h>
#include "multimon.h"

#include "device.h"

#include <neoengine/renderadapter.h>

#include <algorithm>

using namespace std;
using namespace NeoEngine;


namespace NeoD3D9
{


void Device::QueryAdapters( std::vector< RenderAdapter > *pvkAdapters )
{
	if( !pvkAdapters )
		return;

	if( !m_pkD3D && !( m_pkD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to query adapters: Unable to create Direct3D 9 object" << endl;
		return;
	}

	unsigned int uiAdapters = m_pkD3D->GetAdapterCount();

	for( unsigned int uiAdapter = 0; uiAdapter < uiAdapters; ++uiAdapter )
	{
		D3DADAPTER_IDENTIFIER9 kIdentifier;
		RenderAdapter          kAdapter;

		kAdapter.m_uiID = uiAdapter;

		if( D3D_OK != m_pkD3D->GetAdapterIdentifier( uiAdapter, 0, &kIdentifier ) )
		{
			neolog << LogLevel( WARNING ) << "*** Unable to query adapter [" << uiAdapter << "]";
			continue;
		}

		kAdapter.m_strName  = kIdentifier.Description;
		kAdapter.m_hMonitor = m_pkD3D->GetAdapterMonitor( uiAdapter );

		MONITORINFOEX kMonitorInfo;

		kMonitorInfo.cbSize = sizeof( MONITORINFOEX );

		GetMonitorInfo( (HMONITOR)kAdapter.m_hMonitor, (MONITORINFO*)&kMonitorInfo );

		kAdapter.m_aiMonitorRect[0] = kMonitorInfo.rcMonitor.left;
		kAdapter.m_aiMonitorRect[1] = kMonitorInfo.rcMonitor.top;
		kAdapter.m_aiMonitorRect[2] = kMonitorInfo.rcMonitor.right;
		kAdapter.m_aiMonitorRect[3] = kMonitorInfo.rcMonitor.bottom;

		pvkAdapters->push_back( kAdapter );
	}
}


void Device::QueryResolutions( vector< RenderResolution > *pvkResolutions, unsigned int uiAdapter )
{
	if( !pvkResolutions )
		return;

	if( !m_pkD3D && !( m_pkD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to query resolutions: Unable to create Direct3D 9 object" << endl;
		return;
	}

	unsigned int uiAdapters = m_pkD3D->GetAdapterCount();

	if( uiAdapter >= uiAdapters )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to query resolutions: Invalid adapter identifer" << endl;
		return;
	}

	unsigned int uiModes;

	// Alpha/non-alpha modes are treated equally, as is 555/565 modes (MSDN)
	D3DFORMAT aeFormats[2]  = { D3DFMT_X8R8G8B8, D3DFMT_X1R5G5B5 };
	unsigned int uiDepth[2] = { 32,              16              };

	for( unsigned int uiFormat = 0; uiFormat < 2; ++uiFormat )
	{
		uiModes = m_pkD3D->GetAdapterModeCount( uiAdapter, aeFormats[ uiFormat ] );

		D3DDISPLAYMODE kMode;
		
		for( unsigned int uiMode = 0; uiMode < uiModes; ++uiMode )
		{
			if( D3D_OK == m_pkD3D->EnumAdapterModes( uiAdapter, aeFormats[ uiFormat ], uiMode, &kMode ) )
				pvkResolutions->push_back( RenderResolution( kMode.Width, kMode.Height, uiDepth[ uiFormat ], 0, 0, kMode.RefreshRate ) );
		}
	}

	sort( pvkResolutions->begin(), pvkResolutions->end() );
}


};



