/***************************************************************************
               rendertarget.cpp  -  Base class for render targets
                             -------------------
    begin                : Fri Apr 25 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, rendertarget.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 Contributors: Keaton Mullis (kmullis@zianet.com)

 ***************************************************************************/

#include "device.h"
#include "rendertarget.h"
#include "framebuffertarget.h"
#include "pixelbuffertarget.h"
#include "renderqueue.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoD3D9
{


unsigned int Device::SetRenderTarget( unsigned int uiTarget )
{
	unsigned int uiCurTarget = INVALIDBUFFER;

	if( uiTarget == INVALIDBUFFER )
		return uiCurTarget;

	if( m_pkCurTarget )
	{
		if( m_pkCurTarget->GetID() == uiTarget )
			return uiCurTarget;

		uiCurTarget = m_pkCurTarget->GetID();

		m_pkCurTarget->Deactivate();
	}

	if( uiTarget == FRAMEBUFFER )
		m_pkCurTarget = m_pkFrameBufferTarget;
	else if( uiTarget & PIXELBUFFER )
	{
		if( m_pkPixelBufferTarget->SetPixelBuffer( uiTarget & 0x00FFFFFF ) )
			m_pkCurTarget = m_pkPixelBufferTarget;
		else
			neolog << LogLevel( ERROR ) << "[Device::SetRenderTarget] unable to set pixel buffer [" << ( ( uiTarget & 0x00FFFFFF ) >> 24 ) << "] to target" << endl;
	}

	m_pkCurTarget->Activate();

	if( !m_pkCurTarget->IsActive() )
	{
		m_pkCurTarget = 0;
		return SetRenderTarget( uiCurTarget );
	}

	return uiCurTarget;
}


void Device::SetTargetMask( bool bWrite )
{
	if( bWrite )
		m_pkDevice->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALPHA | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_RED );
	else
		m_pkDevice->SetRenderState( D3DRS_COLORWRITEENABLE, 0 );
}


RenderTarget::RenderTarget( Device *pkDevice ) :
	m_pkDevice( pkDevice )
{
	//Create render groups
	m_pkNormalQueue  = new RenderQueue( pkDevice, 50, 5, false );
	m_pkNoTransQueue = new RenderQueue( pkDevice,  5, 1, true  );
}


RenderTarget::~RenderTarget()
{
	delete m_pkNormalQueue;
	delete m_pkNoTransQueue;
}


}; // namespace NeoD3D9

