/***************************************************************************
     windowproc.cpp  -  Window procedure for internally created windows
                             -------------------
    begin                : Sun Mar 9 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevD3D9, windowproc.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "device.h"

#include <neoengine/input.h>
#include <neoengine/logstream.h>

#ifndef WIN32_LEAN_AND_MEAN
#	define WIN32_LEAN_AND_MEAN
#endif
#include <windowsx.h>

using namespace std;
using namespace NeoEngine;
using namespace NeoD3D9;


namespace NeoD3D9
{


bool g_bProcessMsg = true;


#ifdef MB_RIGHT
#  undef MB_RIGHT
#endif

LONG WINAPI WindowProc( HWND hWnd, UINT uiMsg, WPARAM wParam, LPARAM lParam )
{
	static Device *pkDevice            = 0;
	static int     iMouseButtonBitmask = 0;

	if( g_bProcessMsg )
	{
		if( !pkDevice )
			pkDevice = (Device*)GetWindowLong( hWnd, GWL_USERDATA );

		switch( uiMsg )
		{
			case WM_MOUSEMOVE:
			{
				if( pkDevice->HasInputEventGroup( InputDevice::MOUSEINPUT ) )
				{
					int x = GET_X_LPARAM( lParam );
					int y = GET_Y_LPARAM( lParam );

					pkDevice->m_vpkEvents.push_back( new InputEvent( IE_MOUSEMOVE, x, y, 0, iMouseButtonBitmask ) );
				}
				break;
			}

			case WM_LBUTTONDOWN:
				iMouseButtonBitmask |= ( 1 << NeoEngine::MB_LEFT );
				if( pkDevice->HasInputEventGroup( InputDevice::MOUSEINPUT ) )
					pkDevice->m_vpkEvents.push_back( new InputEvent( IE_MOUSEDOWN, NeoEngine::MB_LEFT, LOWORD( lParam ), HIWORD( lParam ) ) );
				break;

			case WM_LBUTTONUP:
				iMouseButtonBitmask &= ~( 1 << NeoEngine::MB_LEFT );
				if( pkDevice->HasInputEventGroup( InputDevice::MOUSEINPUT ) )
					pkDevice->m_vpkEvents.push_back( new InputEvent( IE_MOUSEUP, NeoEngine::MB_LEFT, LOWORD( lParam ), HIWORD( lParam ) ) );
				break;

			case WM_RBUTTONDOWN:
				iMouseButtonBitmask |= ( 1 << NeoEngine::MB_RIGHT );
				if( pkDevice->HasInputEventGroup( InputDevice::MOUSEINPUT ) )
					pkDevice->m_vpkEvents.push_back( new InputEvent( IE_MOUSEDOWN, NeoEngine::MB_RIGHT, LOWORD( lParam ), HIWORD( lParam ) ) );
				break;

			case WM_RBUTTONUP:
				iMouseButtonBitmask |= ~( 1 << NeoEngine::MB_RIGHT );
				if( pkDevice->HasInputEventGroup( InputDevice::MOUSEINPUT ) )
					pkDevice->m_vpkEvents.push_back( new InputEvent( IE_MOUSEUP, NeoEngine::MB_RIGHT, LOWORD( lParam ), HIWORD( lParam ) ) );
				break;

			case WM_KEYDOWN:
			{
				if( pkDevice->HasInputEventGroup( InputDevice::KEYBOARDINPUT ) )
				{
					if( ( (int)wParam >= VK_F1 ) && ( (int)wParam <= (int)VK_F12 ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_F1 + (int)wParam - VK_F1 ) );
					else if( (int)wParam == VK_ESCAPE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_ESCAPE ) );
					else if( (int)wParam == VK_SPACE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_SPACE, ' ' ) );
					else if( (int)wParam == VK_RETURN )
     				{
     					/* If the key is extended (24th bit), it is the keypad ENTER */
     					if (lParam & 1<<24)
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_ENTER ) );
     					else
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RETURN ) );
					}
					else if( (int)wParam == VK_UP )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_UP ) );
					else if( (int)wParam == VK_DOWN )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_DOWN ) );
					else if( (int)wParam == VK_LEFT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LEFT ) );
					else if( (int)wParam == VK_RIGHT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RIGHT ) );
					else if( (int)wParam == VK_BACK )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_BACKSPACE ) );

					else if( ( (int)wParam >= VK_NUMPAD0 ) && ( (int)wParam <= (int)VK_NUMPAD9 ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_0 + (int)wParam - VK_NUMPAD0 ) );
					else if( (int)wParam == VK_ADD )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_PLUS ) );
					else if( (int)wParam == VK_SUBTRACT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_MINUS ) );
					else if( (int)wParam == VK_DECIMAL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_DECIMAL ) );
					else if( (int)wParam == VK_DIVIDE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_DIVIDE ) );
					else if( (int)wParam == VK_MULTIPLY )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_ASTERISK ) );
					else if( (int)wParam == VK_NUMLOCK )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_KP_NUMLOCK ) );
					else if( (int)wParam == VK_TAB )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_TAB, '\t' ) );
					else if( (int)wParam == VK_CAPITAL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_CAPSLOCK ) );
					else if( (int)wParam == VK_SHIFT )
					{
						/* For most keyboards, 36 in the raw OEM code is the right SHIFT */
						if ( ( lParam & 0x00ff0000 ) == 0x00360000)
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RSHIFT ) );
						else
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LSHIFT ) );
					}
					else if( (int)wParam == VK_CONTROL )
     				{
     					/* If the key is extended (24th bit), it is the right CTRL */
     					if (lParam & 1<<24)     
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RCTRL ) );
     					else
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LCTRL ) );
					}
					else if( (int)wParam == VK_MENU )
						/* It would be too tricky (and long) to determine whether
						   it was the left or right one, so we just take it as left here.
						   Use neodevdinput8 if you really need to distinguish them. */
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LALT ) );
					else if( (int)wParam == VK_LWIN )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LWIN ) );
					else if( (int)wParam == VK_RWIN )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RWIN ) );
					else if( (int)wParam == VK_INSERT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_INSERT ) );
					else if( (int)wParam == VK_DELETE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_DELETE ) );
					else if( (int)wParam == VK_HOME )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_HOME ) );
					else if( (int)wParam == VK_END )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_END ) );
					else if( (int)wParam == VK_PRIOR )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_PAGEUP ) );
					else if( (int)wParam == VK_NEXT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_PAGEDOWN ) );
					else if( (int)wParam == VK_SCROLL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_SCROLLLOCK ) );
					else if( (int)wParam == VK_PAUSE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_PAUSE ) );
				}
				break;
			}

			case WM_KEYUP:
			{
				if( pkDevice->HasInputEventGroup( InputDevice::KEYBOARDINPUT ) )
				{
					if( ( (int)wParam >= (int)'A' ) && ( (int)wParam <= (int)'Z' ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_A + (int)wParam - (int)'A' ) );
					else if( ( (int)wParam >= (int)'0' ) && ( (int)wParam <= (int)'9' ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_0 + (int)wParam - (int)'0' ) );
					else if( ( (int)wParam >= VK_F1 ) && ( (int)wParam <= (int)VK_F12 ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_F1 + (int)wParam - VK_F1 ) );
					else if( (int)wParam == VK_ESCAPE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_ESCAPE ) );
					else if( (int)wParam == VK_SPACE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_SPACE ) );
					else if( (int)wParam == VK_RETURN )
     				{
     					/* If the key is extended (24th bit), it is the keypad ENTER */
     					if (lParam & 1<<24)
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_ENTER ) );
     					else
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_RETURN ) );
					}
					else if( (int)wParam == VK_UP )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_UP ) );
					else if( (int)wParam == VK_DOWN )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_DOWN ) );
					else if( (int)wParam == VK_LEFT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_LEFT ) );
					else if( (int)wParam == VK_RIGHT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_RIGHT ) );
					else if( (int)wParam == VK_DECIMAL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_DOT ) );
					else if( (int)wParam == VK_BACK )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_BACKSPACE ) );

					else if( ( (int)wParam >= VK_NUMPAD0 ) && ( (int)wParam <= (int)VK_NUMPAD9 ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_0 + (int)wParam - VK_NUMPAD0 ) );
					else if( (int)wParam == VK_ADD )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_PLUS ) );
					else if( (int)wParam == VK_SUBTRACT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_MINUS ) );
					else if( (int)wParam == VK_DECIMAL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_DECIMAL ) );
					else if( (int)wParam == VK_DIVIDE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_DIVIDE ) );
					else if( (int)wParam == VK_MULTIPLY )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_ASTERISK ) );
					else if( (int)wParam == VK_NUMLOCK )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_NUMLOCK ) );
					else if( (int)wParam == VK_SEPARATOR )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_KP_ENTER ) );
					else if( (int)wParam == VK_TAB )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_TAB ) );
					else if( (int)wParam == VK_CAPITAL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_CAPSLOCK ) );
					else if( (int)wParam == VK_SHIFT )
					{
						/* For most keyboards, 36 in the raw OEM code is the right SHIFT */
						if ( ( lParam & 0x00ff0000 ) == 0x00360000)
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_RSHIFT ) );
						else
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_LSHIFT ) );
					}
					else if( (int)wParam == VK_CONTROL )
     				{
     					/* If the key is extended (24th bit), it is the right CTRL */
     					if (lParam & 1<<24)     
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_RCTRL ) );
     					else
							pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_LCTRL ) );
					}
					else if( (int)wParam == VK_MENU )
						/* It would be too tricky (and long) to determine whether
						   it was the left or right one, so we just take it as left here.
						   Use neodevdinput8 if you really need to distinguish them. */
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_LALT ) );
					else if( (int)wParam == VK_LWIN )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_LWIN ) );
					else if( (int)wParam == VK_RWIN )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_RWIN ) );
					else if( (int)wParam == VK_INSERT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_INSERT ) );
					else if( (int)wParam == VK_DELETE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_DELETE ) );
					else if( (int)wParam == VK_HOME )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_HOME ) );
					else if( (int)wParam == VK_END )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_END ) );
					else if( (int)wParam == VK_PRIOR )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_PAGEUP ) );
					else if( (int)wParam == VK_NEXT )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_PAGEDOWN ) );
					else if( (int)wParam == VK_SCROLL )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_SCROLLLOCK ) );
					else if( (int)wParam == VK_PAUSE )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYUP, KC_PAUSE ) );
				}
				break;
			}

			case WM_CHAR:
			{
				if( pkDevice->HasInputEventGroup( InputDevice::KEYBOARDINPUT ) )
				{
					if( ( (int)wParam >= (int)'A' ) && ( (int)wParam <= (int)'Z' ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_A + int( wParam - 'A' ), 'A' + int( wParam - 'A' ) ) );
					else if( ( (int)wParam >= (int)'a' ) && ( (int)wParam <= (int)'z' ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_A + int( wParam - 'a' ), 'a' + int( wParam - 'a' ) ) );
					else if( ( (int)wParam >= (int)'0' ) && ( (int)wParam <= (int)'9' ) )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_0 + int( wParam - '0' ), '0' + int( wParam - '0' ) ) );
					else if( wParam == '!' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_EXCLAMATION, '!' ) );
					else if( wParam == '"' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_QUOTEDOUBLE, '"' ) );
					else if( wParam == '#' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_HASH, '#' ) );
					else if( wParam == '%' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_PERCENT, '%' ) );
					else if( wParam == '&' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_AMPERSAND, '&' ) );
					else if( wParam == '/' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_SLASH, '/' ) );
					else if( wParam == '(' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LEFTPARENTHESIS, '(' ) );
					else if( wParam == ')' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RIGHTPARENTHESIS, ')' ) );
					else if( wParam == '[' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LEFTBRACKET, '[' ) );
					else if( wParam == ']' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RIGHTBRACKET, ']' ) );
					else if( wParam == '{' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LEFTCURL, '{' ) );
					else if( wParam == '}' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_RIGHTCURL, '}' ) );
					else if( wParam == '+' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_PLUS, '+' ) );
					else if( wParam == '-' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_MINUS, '-' ) );
					else if( wParam == '_' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_UNDERSCORE, '_' ) );
					else if( wParam == '.' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_DOT, '.' ) );
					else if( wParam == ',' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_COMMA, ',' ) );
					else if( wParam == ':' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_COLON, ':' ) );
					else if( wParam == ';' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_SEMICOLON, ';' ) );
					else if( wParam == '<' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_LESS, '<' ) );
					else if( wParam == '>' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_GREATER, '>' ) );
					else if( wParam == '|' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_BAR, '|' ) );
					else if( wParam == '\\' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_BACKSLASH, '\\' ) );
					else if( wParam == '=' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_EQUAL, '=' ) );
					else if( wParam == '\'' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_QUOTE, '\'' ) );
					else if( wParam == '?' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_QUESTION, '?' ) );
					else if( wParam == '$' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_DOLLAR, '$' ) );
					else if( wParam == '' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_POUND, '' ) );
					else if( wParam == '@' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_AT, '@' ) );
					else if( wParam == '*' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_ASTERISK, '*' ) );
					else if( wParam == '~' )
						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_KEYDOWN, KC_TILDE, '~' ) );
				}
				break;
			}

			case WM_SIZE:
			{
				if( pkDevice && pkDevice->HasInputEventGroup( InputDevice::SYSTEMINPUT ) )
				{
					if( ( wParam == SIZE_RESTORED ) || ( wParam == SIZE_MAXIMIZED ) )
					{
						//m_iDeviceWidth = LOWORD(lParam);
						//m_iDeviceHeight = HIWORD(lParam);
	
						//Resize();
					}
				}
				break;
			}
			
			case WM_SETCURSOR:
			{
				if( pkDevice )
					pkDevice->ShowCursor( pkDevice->m_bShowCursor );
				return true;
				break;
			}

			case WM_DESTROY:
			{
				if( pkDevice && !pkDevice->m_bIgnoreKill )
				{
					if( pkDevice->HasInputEventGroup( InputDevice::SYSTEMINPUT ) )
					{
						neolog << LogLevel( DEBUG ) << "[WindowProc] Got WM_DESTROY, terminating" << endl;

						pkDevice->m_vpkEvents.push_back( new InputEvent( IE_SYSEVENT, IE_SYSEVENT_KILL ) );
					}
					
					g_bProcessMsg   = false;
				}

				pkDevice->m_bIgnoreKill = true;

				break;
			}

			default:
				break;
		}
	}

	return DefWindowProc( hWnd, uiMsg, wParam, lParam );
}


};
