/***************************************************************************
                    glue.cpp  -  Export symbols for engine
                             -------------------
    begin                : Wed Sep 24 2003
    copyright            : (C) 2003 by emedia-solutions wolf
    email                : markus@emedia-solutions-wolf.de
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevDInput8, glue.cpp

 The Initial Developer of the Original Code is Markus Wolf.
 Portions created by Markus Wold are Copyright (C) 2003
 emedia-solutions wolf. All Rights Reserved.

 ***************************************************************************/

#include "link.h"
#include "device.h"

#include <neoengine/core.h>
#include <neoengine/renderwindow.h>

using namespace NeoEngine;
using namespace NeoDI;

namespace NeoDI
{

#ifdef BUILD_STATIC
#  define DIEXPORT

ModuleStatic *g_pkStaticModule = 0;

#else
#  ifdef WIN32
#    define DIEXPORT __declspec(dllexport)
#  else
#    define DIEXPORT
#  endif

extern "C"
{

#endif

DIEXPORT InputDevice *CreateDevice( RenderWindow *pkWnd, InputManager *pkInputManager )
{
	return new Device( (HWND)pkWnd->m_hWnd, pkInputManager );
}


DIEXPORT void GetVersion( std::string *pstrName, int *piMajor, int *piMinor, int *piRevision )
{
	std::string strDevInfo =
#  ifdef _DEBUG
	"DI [DEBUG]";
#  else
	"DI";
#  endif

	*pstrName   = strDevInfo;
	*piMajor    = NEOENGINEVERSION_MAJOR;
	*piMinor    = NEOENGINEVERSION_MINOR;
	*piRevision = NEOENGINEVERSION_REVISION;
}

DIEXPORT int Initialize()
{
	return 0;
}

DIEXPORT int Shutdown()
{
#ifdef BUILD_STATIC
	delete g_pkStaticModule;
#endif
	return 0;
}

#ifdef BUILD_STATIC

#else

#  ifndef WIN32_LEAN_AND_MEAN
#    define WIN32_LEAN_AND_MEAN
#  endif

#  include <windows.h>

BOOL APIENTRY DllMain( HANDLE hModule, DWORD dwReason, LPVOID pReserved )
{
	switch( dwReason )
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
    Device::s_hInstance = (HINSTANCE)hModule;
	return TRUE;
}

}

#endif



#ifdef BUILD_STATIC


Link::Link()
{
#if defined(WIN32) && defined(_DEBUG)
	g_pkStaticModule = new ModuleStatic( "neodevdi_debug" );
#else
	g_pkStaticModule = new ModuleStatic( "neodevdi" );
#endif

	g_pkStaticModule->m_Symbols.Insert( "GetVersion",   (void**)GetVersion   );
	g_pkStaticModule->m_Symbols.Insert( "CreateDevice", (void**)CreateDevice );
	g_pkStaticModule->m_Symbols.Insert( "Initialize",   (void**)Initialize   );
	g_pkStaticModule->m_Symbols.Insert( "Shutdown",     (void**)Shutdown     );

	NeoEngine::Core::GetModuleManager()->InsertModule( g_pkStaticModule );
}


#endif


};

