/***************************************************************************
                    device.cpp  -  DirectSound sound device
                             -------------------
    begin                : Wed Mar 17 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevDSound, device.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "device.h"

#include <neoengine/core.h>
#include <neoengine/logstream.h>

using namespace NeoEngine;
using namespace std;


namespace NeoDSound
{


AudioDevice::AudioDevice( FileManager *pkFileManager ) :
	NeoEngine::AudioDevice( pkFileManager ),
	m_pkDS( 0 ),
	m_pkPrimaryBuffer( 0 )
{
}


AudioDevice::~AudioDevice()
{
}


bool AudioDevice::Open( const RenderWindow &rkWindow )
{
	Close();

	HRESULT res;

	if( FAILED( ( res = DirectSoundCreate8( 0, &m_pkDS, 0 ) ) ) )
	{
		neolog << "*** Unable to open audio device: failed to create DirectSound8 object" << endl;
	    return false;
	}

	if( FAILED( ( res = m_pkDS->SetCooperativeLevel( (HWND)rkWindow.m_hWnd, DSSCL_EXCLUSIVE ) ) ) )
	{
		neolog << "*** unable to open audio device: failed to set cooperative level" << endl;
		return false;
	}

	DSCAPS caps;

	memset( &caps, 0, sizeof( DSCAPS ) );
	caps.dwSize = sizeof( DSCAPS );
	
	if( FAILED( ( res = m_pkDS->GetCaps( &caps ) ) ) )
	{
		neolog << "*** unable to open audio device: failed to get caps" << endl;
		return false;
	}

	DSBUFFERDESC bufferDesc;

	memset( &bufferDesc, 0, sizeof( DSBUFFERDESC ) );
	bufferDesc.dwSize  = sizeof( bufferDesc );
	bufferDesc.dwFlags = DSBCAPS_PRIMARYBUFFER | DSBCAPS_CTRL3D;

	if( FAILED( ( res = m_pkDS->CreateSoundBuffer( &bufferDesc, (LPDIRECTSOUNDBUFFER*)&m_pkPrimaryBuffer, 0 ) ) ) )
	{
		neolog << "*** unable to open audio device: failed to create primary buffer" << endl;
		return false;
	}

	return true;
}


void AudioDevice::Close()
{
	if( m_pkPrimaryBuffer )
		m_pkPrimaryBuffer->Release(), m_pkPrimaryBuffer = 0;

	if( m_pkDS )
		m_pkDS->Release(), m_pkDS = 0;
}


};

