/***************************************************************************
                    device.cpp  -  OpenGL render device
                             -------------------
    begin                : Sun Mar 9 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, device.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "device.h"
#include "framebuffertarget.h"
#include "pixelbuffertarget.h"
#include "statistics.h"

#include <neoengine/font.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{

#ifdef WIN32

bool Device::s_bWindowClassRegistered = false;

#endif


Device::Device( FileManager *pkFileManager, InputManager *pkInputManager ) :
	RenderDevice( pkFileManager, pkInputManager ),
#ifdef WIN32
	m_bIgnoreKill( false ),
#endif
	m_pkVertexBufferManager( 0 ),
	m_uiVertexStorageSize( 3145728 ),
	m_pkFrameBufferTarget( 0 ),
	m_pkPixelBufferTarget( 0 ),
	m_pkCurTarget( 0 ),
	m_ppkTMU( 0 ),
	m_uiNumTMUs( 0 ),
	m_uiMaxTextureSize( 0 ),
	m_pkStats( 0 ),
	m_bResetStats( true ),
	m_bDrawOutlines( false ),
	m_bForceDefaultMaterial( false ),
	m_bDrawPolygons( true ),
	m_bDrawNormals( false ),
	m_bDrawTangents( false ),
	m_bIsRenderingTexture( false ),
	m_pkVertexShader( 0 ),
	m_pkFragmentShader( 0 ),
	m_ppkLights( new Light*[ 128 ] ),
	m_uiAllocatedLights( 128 ),
	m_uiMaxLights( 0 ),
	m_uiNumLights( 0 ),
	m_uiCurLight( 0 )
{
#ifdef WIN32

#elif defined( __APPLE__ )

#elif defined( POSIX )

	m_FSWindow                  = 0;
	m_WMDeleteWindow            = 0;

#else
#  error "Platform not implemented"
#endif
	
	//Setup RGBA color ordering
	Color32::s_uiShift[0]     = 0;
	Color32::s_uiShift[1]     = 8;
	Color32::s_uiShift[2]     = 16;
	Color32::s_uiShift[3]     = 24;

	Color32::s_uiComponent[0] = 0;
	Color32::s_uiComponent[1] = 1;
	Color32::s_uiComponent[2] = 2;
	Color32::s_uiComponent[3] = 3;

	//Set font subpixel alignment
	NeoEngine::Font::s_fSubPixelOffset   = 0.0f;

	m_pkStats                 = new FrameStatistics( this );

#ifdef __APPLE__

	for( int i = 0; i < 256; ++i )
	    m_aiKeyMap[i] = -1;

	m_aiKeyMap[ MK_ESCAPE       ] = KC_ESCAPE;
	m_aiKeyMap[ MK_F1           ] = KC_F1;
	m_aiKeyMap[ MK_F2           ] = KC_F2;
	m_aiKeyMap[ MK_F3           ] = KC_F3;
	m_aiKeyMap[ MK_F4           ] = KC_F4;
	m_aiKeyMap[ MK_F5           ] = KC_F5;
	m_aiKeyMap[ MK_F6           ] = KC_F6;
	m_aiKeyMap[ MK_F7           ] = KC_F7;
	m_aiKeyMap[ MK_F8           ] = KC_F8;
	m_aiKeyMap[ MK_F9           ] = KC_F9;
	m_aiKeyMap[ MK_F10          ] = KC_F10;
	m_aiKeyMap[ MK_F11          ] = KC_F11;
	m_aiKeyMap[ MK_F12          ] = KC_F12;
	//m_aiKeyMap[MK_PRINT]        = KC_PRINT;
	//m_aiKeyMap[MK_SCROLLOCK]    = KC_SCROLLOCK;
	//m_aiKeyMap[MK_PAUSE]        = KC_PAUSE;
	//m_aiKeyMap[MK_POWER]        = KC_POWER;
	//m_aiKeyMap[MK_BACKQUOTE]    = KC_BACKQUOTE;
	m_aiKeyMap[ MK_1            ] = KC_1;
	m_aiKeyMap[ MK_2            ] = KC_2;
	m_aiKeyMap[ MK_3            ] = KC_3;
	m_aiKeyMap[ MK_4            ] = KC_4;
	m_aiKeyMap[ MK_5            ] = KC_5;
	m_aiKeyMap[ MK_6            ] = KC_6;
	m_aiKeyMap[ MK_7            ] = KC_7;
	m_aiKeyMap[ MK_8            ] = KC_8;
	m_aiKeyMap[ MK_9            ] = KC_9;
	m_aiKeyMap[ MK_0            ] = KC_0;
	m_aiKeyMap[ MK_MINUS        ] = KC_MINUS;
	//m_aiKeyMap[MK_EQUALS]       = KC_EQUALS;
	m_aiKeyMap[MK_BACKSPACE]      = KC_BACKSPACE;
	//m_aiKeyMap[MK_INSERT]       = KC_INSERT;
	//m_aiKeyMap[MK_HOME]         = KC_HOME;
	//m_aiKeyMap[MK_PAGEUP]       = KC_PAGEUP;
	//m_aiKeyMap[MK_NUMLOCK]      = KC_NUMLOCK;
	//m_aiKeyMap[MK_KP_EQUALS]    = KC_KP_EQUALS;
	//m_aiKeyMap[MK_KP_DIVIDE]    = KC_KP_DIVIDE;
	//m_aiKeyMap[MK_KP_MULTIPLY]  = KC_KP_MULTIPLY;
	//m_aiKeyMap[MK_TAB]          = KC_TAB;
	m_aiKeyMap[ MK_q            ] = KC_Q;
	m_aiKeyMap[ MK_w            ] = KC_W;
	m_aiKeyMap[ MK_e            ] = KC_E;
	m_aiKeyMap[ MK_r            ] = KC_R;
	m_aiKeyMap[ MK_t            ] = KC_T;
	m_aiKeyMap[ MK_y            ] = KC_Y;
	m_aiKeyMap[ MK_u            ] = KC_U;
	m_aiKeyMap[ MK_i            ] = KC_I;
	m_aiKeyMap[ MK_o            ] = KC_O;
	m_aiKeyMap[ MK_p            ] = KC_P;
	m_aiKeyMap[ MK_LEFTBRACKET  ] = KC_LEFTBRACKET;
	m_aiKeyMap[ MK_RIGHTBRACKET ] = KC_RIGHTBRACKET;
	m_aiKeyMap[ MK_BACKSLASH    ] = KC_BACKSLASH;
	m_aiKeyMap[ MK_DELETE       ] = KC_DELETE;
	//m_aiKeyMap[MK_END]          = KC_END;
	//m_aiKeyMap[MK_PAGEDOWN]     = KC_PAGEDOWN;
	//m_aiKeyMap[MK_KP7]          = KC_KP7;
	//m_aiKeyMap[MK_KP8]          = KC_KP8;
	//m_aiKeyMap[MK_KP9]          = KC_KP9;
	//m_aiKeyMap[MK_KP_MINUS]     = KC_KP_MINUS;
	//m_aiKeyMap[MK_CAPSLOCK]     = KC_CAPSLOCK;
	m_aiKeyMap[ MK_a            ] = KC_A;
	m_aiKeyMap[ MK_s            ] = KC_S;
	m_aiKeyMap[ MK_d            ] = KC_D;
	m_aiKeyMap[ MK_f            ] = KC_F;
	m_aiKeyMap[ MK_g            ] = KC_G;
	m_aiKeyMap[ MK_h            ] = KC_H;
	m_aiKeyMap[ MK_j            ] = KC_J;
	m_aiKeyMap[ MK_k            ] = KC_K;
	m_aiKeyMap[ MK_l            ] = KC_L;
	m_aiKeyMap[ MK_SEMICOLON    ] = KC_SEMICOLON;
	m_aiKeyMap[ MK_QUOTE        ] = KC_QUOTE;
	m_aiKeyMap[ MK_RETURN       ] = KC_RETURN;
	//m_aiKeyMap[MK_KP4]          = KC_KP4;
	//m_aiKeyMap[MK_KP5]          = KC_KP5;
	//m_aiKeyMap[MK_KP6]          = KC_KP6;
	//m_aiKeyMap[MK_KP_PLUS]      = KC_KP_PLUS;
	//m_aiKeyMap[MK_LSHIFT]       = KC_LSHIFT;
	m_aiKeyMap[ MK_z            ] = KC_Z;
	m_aiKeyMap[ MK_x            ] = KC_X;
	m_aiKeyMap[ MK_c            ] = KC_C;
	m_aiKeyMap[ MK_v            ] = KC_V;
	m_aiKeyMap[ MK_b            ] = KC_B;
	m_aiKeyMap[ MK_n            ] = KC_N;
	m_aiKeyMap[ MK_m            ] = KC_M;
	m_aiKeyMap[ MK_COMMA        ] = KC_COMMA;
	//m_aiKeyMap[MK_PERIOD]       = KC_PERIOD;
	m_aiKeyMap[ MK_SLASH        ] = KC_SLASH;
	//m_aiKeyMap[MK_RSHIFT]       = KC_RSHIFT;
	m_aiKeyMap[ MK_UP           ] = KC_UP;
	//m_aiKeyMap[MK_KP1]          = KC_KP1;
	//m_aiKeyMap[MK_KP2]          = KC_KP2;
	//m_aiKeyMap[MK_KP3]          = KC_KP3;
	//m_aiKeyMap[MK_KP_ENTER]     = KC_KP_ENTER;
	//m_aiKeyMap[MK_LCTRL]        = KC_LCTRL;
	//m_aiKeyMap[MK_LALT]         = KC_LALT;
	//m_aiKeyMap[MK_LMETA]        = KC_LMETA;
	m_aiKeyMap[ MK_SPACE        ] = KC_SPACE;
	//m_aiKeyMap[MK_RMETA]        = KC_RMETA;
	//m_aiKeyMap[MK_RALT]         = KC_RALT;
	//m_aiKeyMap[MK_RCTRL]        = KC_RCTRL;
	m_aiKeyMap[ MK_LEFT         ] = KC_LEFT;
	m_aiKeyMap[ MK_DOWN         ] = KC_DOWN;
	m_aiKeyMap[ MK_RIGHT        ] = KC_RIGHT;
	//m_aiKeyMap[MK_KP0]          = KC_KP0;
	//m_aiKeyMap[MK_KP_PERIOD]    = KC_KP_PERIOD;
#endif
}


Device::~Device()
{
	Close();

	delete m_pkFrameBufferTarget;
	delete m_pkPixelBufferTarget;

	delete m_pkStats;

	delete [] m_ppkLights;
}


};

