/***************************************************************************
               pixelbuffertarget.cpp  -  Pixel buffer render target
                             -------------------
    begin                : Fri Apr 25 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, pixelbuffertarget.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "pixelbuffertarget.h"
#include "pixelbuffer.h"
#include "textureunit.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{


PixelBufferRenderTarget::PixelBufferRenderTarget( Device *pkDevice ) :
	RenderTarget( pkDevice ),
	m_pkCurBuffer( 0 )
{
	Deactivate();
}


PixelBufferRenderTarget::~PixelBufferRenderTarget()
{
	Deactivate();
}


void PixelBufferRenderTarget::Activate()
{
	if( IsActive() )
		return;

	if( m_pkCurBuffer )
		m_pkCurBuffer->Activate();

	if( m_pkCurBuffer->IsActive() )
		RenderTarget::Activate();
}


void PixelBufferRenderTarget::Deactivate()
{
	if( !IsActive() )
		return;

	if( m_pkCurBuffer )
		m_pkCurBuffer->Deactivate();

	RenderTarget::Deactivate();
}


bool PixelBufferRenderTarget::SetPixelBuffer( unsigned int uiPixelBuffer )
{
	if( m_pkDevice->m_vpkPixelBuffers.size() )
	{
		vector< PixelBuffer* >::iterator ppkBuffer    = m_pkDevice->m_vpkPixelBuffers.begin();
		vector< PixelBuffer* >::iterator ppkBufferEnd = m_pkDevice->m_vpkPixelBuffers.end();

		for( ; ppkBuffer != ppkBufferEnd; ++ppkBuffer )
		{
			if( ( (*ppkBuffer)->GetTargetID() & 0x00FFFFFF ) == uiPixelBuffer )
			{
				m_pkCurBuffer = *ppkBuffer;
				return true;
			}
		}
	}

	neolog << LogLevel( ERROR ) << "*** Unable to set pixel buffer: Unable to locate buffer [" << uiPixelBuffer << "]" << endl;

	return false;
}


int PixelBufferRenderTarget::GetWidth()
{
        return m_pkCurBuffer->GetWidth();
}


int PixelBufferRenderTarget::GetHeight()
{
        return m_pkCurBuffer->GetHeight();
}


}; // namespace NeoOGL

