/***************************************************************************
              pixelbuffertarget.h  -  Pixel buffer render target
                             -------------------
    begin                : Fri Apr 25 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, pixelbuffertarget.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEOGLPIXELBUFFERTARGET_H
#define __NEOGLPIXELBUFFERTARGET_H


/**
  * \file pixelbuffertarget.h
  * Pixel buffer render target
  */


#include "device.h"
#include "rendertarget.h"


namespace NeoOGL
{


// External classes
class PixelBuffer;






/**
  * \brief Pixel buffer render target
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class PixelBufferRenderTarget : public RenderTarget
{
	public:

		/*! Pixel buffer */
		PixelBuffer                                 *m_pkCurBuffer;

		/**
		* Initialize data
		* \param pkDevice                             Device
		*/
		                                              PixelBufferRenderTarget( Device *pkDevice );

		/**
		* Deallocate memory
		*/
		virtual                                      ~PixelBufferRenderTarget();

		/**
		* \return                                     Render target identifier
		*/
		virtual unsigned int                          GetID() { return NeoEngine::RenderDevice::PIXELBUFFER; }

		/**
		* Activate
		*/
		inline virtual void                           Activate();

		/**
		* Deactivate
		*/
		inline virtual void                           Deactivate();

		/**
		* Set active pixel buffer
		* \param uiPixelBuffer                        Pixel buffer ID
		* \return                                     true if successful, false if error
		*/
		bool                                          SetPixelBuffer( unsigned int uiPixelBuffer );

		virtual int                                   GetHeight();

		virtual int                                   GetWidth();
};


}; // namespace NeoOGL


#endif

