/***************************************************************************
              combiners.cpp  -  nVidia register combiners parser
                             -------------------
    begin       : Wed Jan 7 2004
    copyright   : (C) 2004 by Reality Rift Studios
    email       : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, rcparse/combiners.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.
 ***************************************************************************/

#include "combiners.h"
#include "parser.h"
#include "../device.h"

#include <neoengine/shader.h>
#include <neoengine/light.h>
#include <neoengine/scenenode.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


char *myin;
int line_number = 0;

int ParseSource();


namespace NeoOGL
{


bool NVRegisterCombiners::Validate()
{
	if( ( m_uiNumConst == 2 ) && ( m_kConstColor[0].reg.bits.name == m_kConstColor[1].reg.bits.name ) )
		neolog << LogLevel( ERROR ) << "Unable to compile fragment shader: Global constant set twice" << endl;

	if( !m_kGenerals.Validate( m_uiNumConst, m_kConstColor ) )
		return false;

	return m_kFinal.Validate();
}


void NVRegisterCombiners::Invoke( Device *pkDevice, NeoEngine::RenderPrimitive *pkPrimitive )
{
	for( unsigned int i = 0; i < m_uiNumConst; ++i )
		BindParam( pkDevice, -1, m_kConstColor[i].reg.bits.name, m_pkParams[i], pkPrimitive );

	m_kGenerals.Invoke( pkDevice, pkPrimitive );
	m_kFinal.Invoke();
}


void NVRegisterCombiners::BindLightParams( Device *pkDevice, RenderPrimitive *pkPrimitive )
{
	for( unsigned int i = 0; i < m_uiNumConst; ++i )
	{
		if( ( m_pkParams[i]->m_eType == ShaderParam::PPT_SYSTEM ) && ( m_pkParams[i]->m_Value.m_eSystemID & ShaderParam::LIGHTPARAM ) )
			BindParam( pkDevice, -1, m_kConstColor[i].reg.bits.name, m_pkParams[i], pkPrimitive );
	}

	m_kGenerals.BindLightParams( pkDevice, pkPrimitive );
}


void NVRegisterCombiners::BindParam( Device *pkDevice, int iStage, unsigned int uiName, ShaderParam *pkParam, RenderPrimitive *pkPrimitive )
{
	static float afIdentity[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	float afVal[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	unsigned int uiLight = pkDevice->m_uiCurLight + pkParam->m_uiLight;

	switch( pkParam->m_eType )
	{
		case ShaderParam::PPT_SYSTEM:
		{
			switch( pkParam->m_Value.m_eSystemID )
			{
				case ShaderParam::PSP_LIGHT_POSITION:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTranslation();
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIRECTION:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = -pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTransform().GetColumn(2);
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_POSITION_OBJECTSPACE:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkPrimitive->m_kInvModelMatrix * pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTranslation();
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIRECTION_OBJECTSPACE:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkPrimitive->m_kInvModelMatrix * (-pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTransform().GetColumn(2));
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_AMBIENT_COLOR:
				{
					const float *pfVal = ( uiLight < pkDevice->m_uiNumLights ) ? (const float*)pkDevice->m_ppkLights[ uiLight ]->m_kAmbient : afIdentity;
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, pfVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, pfVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIFFUSE_COLOR:
				{
					const float *pfVal = ( uiLight < pkDevice->m_uiNumLights ) ? (const float*)pkDevice->m_ppkLights[ uiLight ]->m_kDiffuse : afIdentity;
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, pfVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, pfVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_SPECULAR_COLOR:
				{
					const float *pfVal = ( uiLight < pkDevice->m_uiNumLights ) ? (const float*)pkDevice->m_ppkLights[ uiLight ]->m_kSpecular : afIdentity;
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, pfVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, pfVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_ATTENUATION:
				{
					if( uiLight < pkDevice->m_uiNumLights )
					{
						afVal[0] = pkDevice->m_ppkLights[ uiLight ]->m_fConstantAttenuation;
						afVal[1] = pkDevice->m_ppkLights[ uiLight ]->m_fLinearAttenuation;
						afVal[2] = pkDevice->m_ppkLights[ uiLight ]->m_fQuadraticAttenuation;
						afVal[3] = 1.0f;
					}
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_REP_ATTENUATION:
				{
					if( uiLight < pkDevice->m_uiNumLights )
					{
						afVal[0] = 1.0f / pkDevice->m_ppkLights[ uiLight ]->m_fConstantAttenuation;
						afVal[1] = 1.0f / pkDevice->m_ppkLights[ uiLight ]->m_fLinearAttenuation;
						afVal[2] = 1.0f / pkDevice->m_ppkLights[ uiLight ]->m_fQuadraticAttenuation;
						afVal[3] = 1.0f;
					}
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				case ShaderParam::PSP_TOTAL_LIGHT_AMBIENT_COLOR:
				{
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, (const float*)pkDevice->m_kTotalAmbientLight );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, (const float*)pkDevice->m_kTotalAmbientLight );
					break;
				}

				case ShaderParam::PSP_TOTAL_AMBIENTLIGHT_AMBIENT_COLOR:
				{
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, (const float*)pkDevice->m_kAmbientLight );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, (const float*)pkDevice->m_kAmbientLight );
					break;
				}

				case ShaderParam::PSP_EYE_POSITION_OBJECTSPACE:
				{
					*((Vector3d*)afVal) = ( pkDevice->GetViewMatrix() * pkPrimitive->m_kInvModelMatrix ).GetColumn(3);
					if( iStage < 0 )
						neoglCombinerParameterfvNV( uiName, afVal );
					else
						neoglCombinerStageParameterfvNV( GL_COMBINER0_NV + iStage, uiName, afVal );
					break;
				}

				default:
				{
					neolog << LogLevel( ERROR ) << "*** Unable to bind system parameter: Unsupported type" << endl;
					break;
				}
			}
		}

		case ShaderParam::PPT_FLOAT4:
		{
			if( iStage < 0 )
				neoglCombinerParameterfvNV( uiName, pkParam->m_Value.m_afFloat );
			break;
		}

		case ShaderParam::PPT_COLOR:
		{
			if( iStage < 0 )
				neoglCombinerParameterfvNV( uiName, (const float*)pkParam->m_Value.m_pkColor );
			break;
		}

		case ShaderParam::PPT_VECTOR:
		{
			if( iStage < 0 )
				neoglCombinerParameterfvNV( uiName, (const float*)pkParam->m_Value.m_pkVector );
			break;
		}

		case ShaderParam::PPT_QUAT:
		{
			if( iStage < 0 )
				neoglCombinerParameterfvNV( uiName, (const float*)pkParam->m_Value.m_pkQuaternion );
			break;
		}
/*
		case ShaderParam::PPT_MATRIX:
		{
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex    , (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[0][0] );
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex + 1, (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[1][0] );
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex + 2, (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[2][0] );
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex + 3, (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[3][0] );
			break;
		}
*/
		default:
		{
			neolog << LogLevel( ERROR ) << "*** Unable to bind fragment shader: Invalid parameter type" << endl;
			break;
		}
	}
}


bool NVRegisterCombiners::Compile( const char *pszSource )
{
	char *pszDup = myin = strdup( pszSource );

	line_number  = 0;
	pFinalResult = this;

	bool bSuccess = ( ParseSource() == 0 );

	free( pszDup );

	return bSuccess;
}


};

