/***************************************************************************
               rendertarget.cpp  -  Base class for render targets
                             -------------------
    begin                : Fri Apr 25 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, rendertarget.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "rendertarget.h"
#include "framebuffertarget.h"
#include "pixelbuffertarget.h"
#include "renderqueue.h"
#include "device.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{



unsigned int Device::SetRenderTarget( unsigned int uiTarget )
{
	unsigned int uiOldTarget = INVALIDBUFFER;

	if( uiTarget == INVALIDBUFFER )
		return uiOldTarget;

	RenderTarget *pkOldTarget = 0;

	if( m_pkCurTarget )
	{
		if( m_pkCurTarget->GetID() == uiTarget )
			return uiTarget;

		uiOldTarget = m_pkCurTarget->GetID();
		pkOldTarget = m_pkCurTarget;
	}

	if( uiTarget == FRAMEBUFFER )
		m_pkCurTarget = m_pkFrameBufferTarget;
	else if( ( uiTarget & 0xFF000000 ) == PIXELBUFFER )
	{
		if( m_pkPixelBufferTarget->SetPixelBuffer( uiTarget & 0x00FFFFFF ) )
			m_pkCurTarget = m_pkPixelBufferTarget;
		else
			neolog << LogLevel( ERROR ) << "*** Unable to set render target: Pixel buffer [" << ( ( uiTarget & 0x00FFFFFF ) >> 24 ) << "] failed" << endl;
	}
	else
		neolog << LogLevel( ERROR ) << "*** Unable to set render target: Unimplemented target [" << uiTarget << "]" << endl;

	m_pkCurTarget->Activate();

	if( !m_pkCurTarget->IsActive() )
	{
		m_pkCurTarget = 0;
		return SetRenderTarget( uiOldTarget );
	}
	else if( pkOldTarget )
		pkOldTarget->Deactivate();

	return uiOldTarget;
}


void Device::SetTargetMask( bool bWrite )
{
	if( bWrite )
		glColorMask( GL_TRUE,  GL_TRUE,  GL_TRUE,  GL_TRUE  );
	else
		glColorMask( GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE );
}




RenderTarget::RenderTarget( Device *pkDevice ) :
	m_pkDevice( pkDevice )
{
	//Create render groups
	m_pkNormalQueue  = new RenderQueue( pkDevice, 50, 5, false );
	m_pkNoTransQueue = new RenderQueue( pkDevice,  5, 1, true  );
}


RenderTarget::~RenderTarget()
{
	delete m_pkNormalQueue;
	delete m_pkNoTransQueue;
}


void RenderTarget::DeleteQueues()
{
	delete m_pkNormalQueue, m_pkNormalQueue = 0;
	delete m_pkNoTransQueue, m_pkNoTransQueue = 0;
}


};
