/***************************************************************************
               shader-arb.cpp  -  ARB vertex/fragment shaders
                             -------------------
    begin       : Mon Jan 5 2004
    copyright   : (C) 2004 by Reality Rift Studios
    email       : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, shader-arb.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.
 ***************************************************************************/

#include "shader-arb.h"

#include <neoengine/light.h>
#include <neoengine/scenenode.h>
#include <neoengine/strutil.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{


inline void BindParameter( Device *pkDevice, GLenum eTarget, ShaderParam *pkParam, RenderPrimitive *pkPrimitive )
{
	static float afIdentity[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	float afVal[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	unsigned int uiLight = pkDevice->m_uiCurLight + pkParam->m_uiLight;

	switch( pkParam->m_eType )
	{
		case ShaderParam::PPT_SYSTEM:
		{
			switch( pkParam->m_Value.m_eSystemID )
			{
				case ShaderParam::PSP_MODEL_MATRIX:
				{
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &pkPrimitive->m_kModelMatrix.m_aafMatrix[0][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &pkPrimitive->m_kModelMatrix.m_aafMatrix[1][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &pkPrimitive->m_kModelMatrix.m_aafMatrix[2][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &pkPrimitive->m_kModelMatrix.m_aafMatrix[3][0] );
					break;
				};

				case ShaderParam::PSP_VIEW_MATRIX:
				{
					const Matrix &kMat = pkDevice->GetViewMatrix();

					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &kMat.m_aafMatrix[0][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &kMat.m_aafMatrix[1][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &kMat.m_aafMatrix[2][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &kMat.m_aafMatrix[3][0] );
					break;
				};

				case ShaderParam::PSP_PROJECTION_MATRIX:
				{
					const Matrix &kMat = pkDevice->GetProjectionMatrix();

					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &kMat.m_aafMatrix[0][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &kMat.m_aafMatrix[1][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &kMat.m_aafMatrix[2][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &kMat.m_aafMatrix[3][0] );
					break;
				};

				case ShaderParam::PSP_MODELVIEW_MATRIX:
				{
					Matrix kMat( pkDevice->GetViewMatrix() * pkPrimitive->m_kModelMatrix );

					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &kMat.m_aafMatrix[0][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &kMat.m_aafMatrix[1][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &kMat.m_aafMatrix[2][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &kMat.m_aafMatrix[3][0] );
					break;
				}

				case ShaderParam::PSP_MODELVIEWPROJ_MATRIX:
				{
					Matrix kMat( ( pkDevice->GetProjectionMatrix() * pkDevice->GetViewMatrix() ) * pkPrimitive->m_kModelMatrix );

					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &kMat.m_aafMatrix[0][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &kMat.m_aafMatrix[1][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &kMat.m_aafMatrix[2][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &kMat.m_aafMatrix[3][0] );
					break;
				}

				case ShaderParam::PSP_INV_MODEL_MATRIX:
				{
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &pkPrimitive->m_kInvModelMatrix.m_aafMatrix[0][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &pkPrimitive->m_kInvModelMatrix.m_aafMatrix[1][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &pkPrimitive->m_kInvModelMatrix.m_aafMatrix[2][0] );
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &pkPrimitive->m_kInvModelMatrix.m_aafMatrix[3][0] );
					break;
				};

				case ShaderParam::PSP_LIGHT_POSITION:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTranslation();
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIRECTION:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = -pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTransform().GetColumn(2);
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_POSITION_OBJECTSPACE:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkPrimitive->m_kInvModelMatrix * pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTranslation();
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIRECTION_OBJECTSPACE:
				{
					if( ( uiLight < pkDevice->m_uiNumLights ) && pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkPrimitive->m_kInvModelMatrix * (-pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTransform().GetColumn(2));
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_AMBIENT_COLOR:
				{
					if( uiLight < pkDevice->m_uiNumLights )
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, pkDevice->m_ppkLights[ uiLight ]->m_kAmbient );
					else
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIFFUSE_COLOR:
				{
					if( uiLight < pkDevice->m_uiNumLights )
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, pkDevice->m_ppkLights[ uiLight ]->m_kDiffuse );
					else
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_SPECULAR_COLOR:
				{
					if( uiLight < pkDevice->m_uiNumLights )
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, pkDevice->m_ppkLights[ uiLight ]->m_kSpecular );
					else
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_ATTENUATION:
				{
					if( uiLight < pkDevice->m_uiNumLights )
					{
						afVal[0] = pkDevice->m_ppkLights[ uiLight ]->m_fConstantAttenuation;
						afVal[1] = pkDevice->m_ppkLights[ uiLight ]->m_fLinearAttenuation;
						afVal[2] = pkDevice->m_ppkLights[ uiLight ]->m_fQuadraticAttenuation;
						afVal[3] = 1.0f;
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					}
					else
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_REP_ATTENUATION:
				{
					if( uiLight < pkDevice->m_uiNumLights )
					{
						afVal[0] = 1.0f / pkDevice->m_ppkLights[ uiLight ]->m_fConstantAttenuation;
						afVal[1] = 1.0f / pkDevice->m_ppkLights[ uiLight ]->m_fLinearAttenuation;
						afVal[2] = 1.0f / pkDevice->m_ppkLights[ uiLight ]->m_fQuadraticAttenuation;
						afVal[3] = 1.0f;
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					}
					else
						neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_TOTAL_LIGHT_AMBIENT_COLOR:
				{
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, (const float*)pkDevice->m_kTotalAmbientLight );
					break;
				}

				case ShaderParam::PSP_TOTAL_AMBIENTLIGHT_AMBIENT_COLOR:
				{
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, (const float*)pkDevice->m_kAmbientLight );
					break;
				}

				case ShaderParam::PSP_EYE_POSITION_OBJECTSPACE:
				{
					*((Vector3d*)afVal) = ( pkDevice->GetViewMatrix() * pkPrimitive->m_kInvModelMatrix ).GetColumn(3);
					neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
					break;
				}

				default:
				{
					neolog << LogLevel( ERROR ) << "*** Unable to bind system parameter: Unsupported type" << endl;
					break;
				}
			}

			break;
		}

		case ShaderParam::PPT_FLOAT4:
		{
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, pkParam->m_Value.m_afFloat );
			break;
		}

		case ShaderParam::PPT_VECTOR:
		{
			float afVal[4] = { pkParam->m_Value.m_pkVector->x, pkParam->m_Value.m_pkVector->y, pkParam->m_Value.m_pkVector->z, 1.0f };
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, afVal );
			break;
		}

		case ShaderParam::PPT_COLOR:
		{
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, (const GLfloat*)pkParam->m_Value.m_pkColor );
			break;
		}

		case ShaderParam::PPT_QUAT:
		{
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex, (const GLfloat*)&pkParam->m_Value.m_pkQuaternion->qx );
			break;
		}

		case ShaderParam::PPT_MATRIX:
		{
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex    , &pkParam->m_Value.m_pkMatrix->m_aafMatrix[0][0] );
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 1, &pkParam->m_Value.m_pkMatrix->m_aafMatrix[1][0] );
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 2, &pkParam->m_Value.m_pkMatrix->m_aafMatrix[2][0] );
			neoglProgramLocalParameter4fvARB( eTarget, pkParam->m_uiIndex + 3, &pkParam->m_Value.m_pkMatrix->m_aafMatrix[3][0] );
			
			break;
		}

		default:
		{
			neolog << LogLevel( ERROR ) << "*** Unable to bind parameter: Unsupported type" << endl;
			break;
		}
	}
}


ShaderARB::ShaderARB( Device *pkDevice, SHADERTYPE eType, FileManager *pkFileManager ) :
	NeoOGL::Shader( pkDevice, eType, pkFileManager ),
	m_uiShader( 0 ),
	m_eType( eType == VERTEXSHADER ? GL_VERTEX_PROGRAM_ARB : GL_FRAGMENT_PROGRAM_ARB )
{
}


ShaderARB::~ShaderARB()
{
	if( ( ( m_eShaderType == VERTEXSHADER ) && ( m_pkDevice->m_pkVertexShader == this ) ) ||
	    ( ( m_eShaderType == VERTEXSHADER ) && ( m_pkDevice->m_pkVertexShader == this ) ) )
		glDisable( m_eType );

	if( m_uiShader )
		neoglDeleteProgramsARB( 1, &m_uiShader );
}


bool ShaderARB::Compile( const string *pstrSource )
{
	if( m_bIsCompiled && !pstrSource )
		return true;

	ClearParams();

	if( pstrSource )
		m_strSource = *pstrSource;

	if( m_uiShader )
		neoglDeleteProgramsARB( 1, &m_uiShader ), m_uiShader = 0;

	neoglGenProgramsARB( 1, &m_uiShader );

	if( neoglCheckError( "*** Unable to generate program name: " ) )
		return false;

	neoglBindProgramARB( m_eType, m_uiShader );

	neoglProgramStringARB( m_eType, GL_PROGRAM_FORMAT_ASCII_ARB, m_strSource.length(), m_strSource.c_str() );

	if( neoglCheckError( string( "*** Unable to compile " ) + ( m_eType == GL_VERTEX_PROGRAM_ARB ? "vertex" : "fragment" ) + " program: Error loading program source, " ) )
	{
		neolog << LogLevel( ERROR ) << "  Program source:" << endl << m_strSource << endl;

		GLint iErrorPos;

		glGetIntegerv( GL_PROGRAM_ERROR_POSITION_ARB, &iErrorPos );

		const GLubyte *pucError;

		pucError = glGetString( GL_PROGRAM_ERROR_STRING_ARB );

		neolog << "  Position: " << iErrorPos << endl
		       << "  Error: "    << pucError  << endl;

		return false;
	}

	int iInstructions, iTemporaries, iParameters;

	neoglGetProgramivARB( m_eType, GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB, &iInstructions );
	neoglGetProgramivARB( m_eType, GL_PROGRAM_NATIVE_TEMPORARIES_ARB,  &iTemporaries  );
	neoglGetProgramivARB( m_eType, GL_PROGRAM_NATIVE_PARAMETERS_ARB,   &iParameters   );

	neolog << LogLevel( DEBUG ) << "Program [" << m_uiShader << "] loaded" << endl
	                            << "  [" << iInstructions << "] instructions" << endl
	                            << "  [" << iTemporaries  << "] temporaries" << endl
	                            << "  [" << iParameters   << "] parameters" << endl;

	ParseParams();

	m_bIsCompiled = true;

	return true;
}


bool ShaderARB::Bind( RenderPrimitive *pkPrimitive )
{
#ifdef _DEBUG
	if( !m_bIsCompiled )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to bind " << ( ( m_eShaderType == VERTEXSHADER ) ? "vertex" : "fragment" ) << " program: Program not compiled" << endl;
		return false;
	}
#endif

	neoglBindProgramARB( m_eType, m_uiShader );

	glEnable( m_eType );

	if( neoglCheckError( string( "*** Unable to bind " ) + ( ( m_eShaderType == VERTEXSHADER ) ? "vertex" : "fragment" ) + " program: " ) )
	{
		GLint iErrorPos;

		glGetIntegerv( GL_PROGRAM_ERROR_POSITION_ARB, &iErrorPos );

		const GLubyte *pucError;

		pucError = glGetString( GL_PROGRAM_ERROR_STRING_ARB );

		neolog << LogLevel( ERROR ) 
		       << "  Position: " << iErrorPos << endl
		       << "  Error: " << pucError << endl;

		Unbind();

		return false;
	}

	//Bind all params
	vector< ShaderParam* >::iterator ppkParam = m_vpkParams.begin();
	vector< ShaderParam* >::iterator ppkEnd   = m_vpkParams.end();

	for( ; ppkParam != ppkEnd; ++ppkParam )
		BindParameter( m_pkDevice, m_eType, *ppkParam, pkPrimitive );

	//We don't use updated params for now, clear to avoid stockpiling pointers
	m_vpkUpdatedParams.clear();

	neoglCheckError( "*** Error binding parameters: " );

	return true;
}


bool ShaderARB::Unbind()
{
	glDisable( m_eType );
	return true;
}


bool ShaderARB::BindLightParams( RenderPrimitive *pkPrimitive )
{
	//Bind all params
	vector< ShaderParam* >::iterator ppkParam = m_vpkParams.begin();
	vector< ShaderParam* >::iterator ppkEnd   = m_vpkParams.end();

	for( ; ppkParam != ppkEnd; ++ppkParam )
	{
		if( ( (*ppkParam)->m_eType == ShaderParam::PPT_SYSTEM ) && ( (*ppkParam)->m_Value.m_eSystemID & ShaderParam::LIGHTPARAM ) )
			BindParameter( m_pkDevice, m_eType, *ppkParam, pkPrimitive );
	}

	return true;
}


};

