/***************************************************************************
                   shader-ati.cpp  -  ATI fragment shaders
                             -------------------
    begin       : Mon Jan 6 2004
    copyright   : (C) 2004 by Reality Rift Studios
    email       : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, shader-ati.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.
 ***************************************************************************/

#include "shader-ati.h"

#include <neoengine/light.h>
#include <neoengine/scenenode.h>
#include <neoengine/strutil.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{


enum ATIFSPARSERSTAGE
{
	HEADER      = 0,
	CONSTANTS   = 1,
	PRELIM      = 2,
	OUTPUT      = 3,
	ENTERSTAGE  = 4,
	DONEPARSING = 5
};

enum ATIFSREGUSE
{
	DST,
	SRC,
	TEX
};

//Simplify parsing
#define GL_ALPHA_BIT_ATI                   0x00000008


bool ParseArg( const string &rstrToken, int *piReg, int *piSwizzle, int *piMod, ATIFSREGUSE eUse )
{
	vector< string > vstrTokens;

	Explode( ".", rstrToken, &vstrTokens );

	if( ( vstrTokens[0].length() == 2 ) && ( vstrTokens[0][0] == 'r' ) )
	{
		if( ( vstrTokens[0][1] < '0' ) || ( vstrTokens[0][1] > '7' ) )
			return false;

		*piReg = GL_REG_0_ATI + ( (unsigned int)vstrTokens[0][1] - '0' );
	}
	else if( ( vstrTokens[0].length() == 2 ) && ( vstrTokens[0][0] == 'c' ) )
	{
		if( ( vstrTokens[0][1] < '0' ) || ( vstrTokens[0][1] > '7' ) )
			return false;

		*piReg = GL_CON_0_ATI + ( (unsigned int)vstrTokens[0][1] - '0' );
	}
	else if( ( vstrTokens[0].length() == 2 ) && ( vstrTokens[0][0] == 't' ) )
	{
		if( ( vstrTokens[0][1] < '0' ) || ( vstrTokens[0][1] > '9' ) )
			return false;

		*piReg = GL_TEXTURE0_ARB + ( (unsigned int)vstrTokens[0][1] - '0' );
	}
	else if( vstrTokens[0] == "color0" )
		*piReg = GL_PRIMARY_COLOR;
	else if( vstrTokens[0] == "color1" )
		*piReg = GL_SECONDARY_INTERPOLATOR_ATI;
	else if( vstrTokens[0] == "0" )
		*piReg = GL_ZERO;
	else if( vstrTokens[0] == "1" )
		*piReg = GL_ONE;
	else
		return false;

	if( ( eUse == DST ) && ( ( *piReg < GL_REG_0_ATI ) || ( *piReg > GL_REG_7_ATI ) ) )
		return false;
	if( ( eUse == TEX ) && ( ( *piReg < GL_REG_0_ATI ) && ( *piReg > GL_REG_7_ATI ) && ( *piReg < GL_TEXTURE0_ARB ) && ( *piReg > GL_TEXTURE7_ARB ) ) )
		return false;
	if( ( eUse == SRC ) && ( *piReg >= GL_TEXTURE0_ARB ) && ( *piReg <= GL_TEXTURE7_ARB ) )
		return false;

	if( !piSwizzle && !piMod && ( vstrTokens.size() > 1 ) )
		return false;

	if( piSwizzle )
		*piSwizzle = 0;

	if( piMod )
		*piMod = GL_NONE;

	if( piSwizzle )
	{
		if( vstrTokens.size() == 1 )
			return( eUse != TEX );

		if( eUse == TEX )
		{
			if( vstrTokens[1] == "str" )
				*piSwizzle = GL_SWIZZLE_STR_ATI;
			else if( vstrTokens[1] == "str_dr" )
				*piSwizzle = GL_SWIZZLE_STR_DR_ATI;
			else if( vstrTokens[1] == "stq" )
				*piSwizzle = GL_SWIZZLE_STQ_ATI;
			else if( vstrTokens[1] == "stq_dq" )
				*piSwizzle = GL_SWIZZLE_STQ_DQ_ATI;
			else
				return false;

			vstrTokens.erase( vstrTokens.begin() + 1 );
		}
		else if( eUse == DST )
		{
			unsigned int i = 0;

			if( ( i < vstrTokens[1].length() ) && ( vstrTokens[1][i] == 'r' ) )
			{ *piSwizzle |= GL_RED_BIT_ATI; ++i; }
			if( ( i < vstrTokens[1].length() ) && ( vstrTokens[1][i] == 'g' ) )
			{ *piSwizzle |= GL_GREEN_BIT_ATI; ++i; }
			if( ( i < vstrTokens[1].length() ) && ( vstrTokens[1][i] == 'b' ) )
			{ *piSwizzle |= GL_BLUE_BIT_ATI; ++i; }
			if( ( i < vstrTokens[1].length() ) && ( vstrTokens[1][i] == 'a' ) )
			{ *piSwizzle |= GL_ALPHA_BIT_ATI; ++i; }
			
			if( i == vstrTokens[1].length() )
				vstrTokens.erase( vstrTokens.begin() + 1 );
			else
				*piSwizzle = 0;
		}
		else if( eUse == SRC )
		{
			if( vstrTokens[1] == "r" )
				*piSwizzle = GL_RED;
			else if( vstrTokens[1] == "g" )
				*piSwizzle = GL_GREEN;
			else if( vstrTokens[1] == "b" )
				*piSwizzle = GL_BLUE;
			else if( vstrTokens[1] == "a" )
				*piSwizzle = GL_ALPHA;
			
			if( *piSwizzle != 0 )
				vstrTokens.erase( vstrTokens.begin() + 1 );
		}
	}

	if( piMod )
	{
		while( vstrTokens.size() > 1 )
		{
			if( eUse == DST )
			{
				if( ( ( *piMod != GL_NONE ) && ( vstrTokens[1] != "sat" ) ) || ( *piMod & GL_SATURATE_BIT_ATI ) )
					return false;

				if( vstrTokens[1] == "2x" )
					*piMod = GL_2X_BIT_ATI;
				else if( vstrTokens[1] == "4x" )
					*piMod = GL_4X_BIT_ATI;
				else if( vstrTokens[1] == "8x" )
					*piMod = GL_8X_BIT_ATI;
				else if( vstrTokens[1] == "half" )
					*piMod = GL_HALF_BIT_ATI;
				else if( vstrTokens[1] == "quarter" )
					*piMod = GL_QUARTER_BIT_ATI;
				else if( vstrTokens[1] == "eighth" )
					*piMod = GL_EIGHTH_BIT_ATI;
				else if( vstrTokens[1] == "sat" )
					*piMod |= GL_SATURATE_BIT_ATI;
				else
					return false;
			}
			else // SRC
			{
				if( vstrTokens[1] == "neg" )
					*piMod |= GL_NEGATE_BIT_ATI;
				else if( vstrTokens[1] == "comp" )
					*piMod |= GL_COMP_BIT_ATI;
				else if( vstrTokens[1] == "bias" )
					*piMod |= GL_BIAS_BIT_ATI;
				else if( vstrTokens[1] == "2x" )
					*piMod |= GL_2X_BIT_ATI;
				else
					return false;
			}

			vstrTokens.erase( vstrTokens.begin() + 1 );
		}
	}

	if( vstrTokens.size() > 1 )
		return false;

	return true;
}



ShaderATIFragmentShader::ShaderATIFragmentShader( Device *pkDevice, FileManager *pkFileManager ) :
	NeoOGL::Shader( pkDevice, FRAGMENTSHADER, pkFileManager ),
	m_uiShader( 0 )
{
}


ShaderATIFragmentShader::~ShaderATIFragmentShader()
{
	if( m_pkDevice->m_pkFragmentShader == this )
		glDisable( GL_FRAGMENT_SHADER_ATI );

	if( m_uiShader )
		neoglDeleteFragmentShaderATI( m_uiShader ), m_uiShader = 0;
}


bool ShaderATIFragmentShader::Compile( const string *pstrSource )
{
	if( m_bIsCompiled && !pstrSource )
		return true;

	ClearParams();

	if( pstrSource )
		m_strSource = *pstrSource;

	if( m_uiShader )
		neoglDeleteFragmentShaderATI( m_uiShader ), m_uiShader = 0;

	m_uiShader = neoglGenFragmentShadersATI( 1 );

	if( neoglCheckError( "*** Unable to generate shader name: " ) )
		return false;

	neoglBindFragmentShaderATI( m_uiShader );

	neoglBeginFragmentShaderATI();

	//Parse source into shader commands
	string           strError;
	string           strLine;
	string           strToken;
	ATIFSPARSERSTAGE eStage     = HEADER;
	ATIFSPARSERSTAGE eLastStage = HEADER;

	string::size_type uiPos     = 0;
	string::size_type uiLine    = 0;
	string::size_type uiToken   = 0;

	unsigned int uiPasses   = 0;
	unsigned int uiNumPairs = 0;
	unsigned int uiRegSampled[10];
	int          iTexSwizzle[16];
	bool         bLastWasColor = false;

	memset( iTexSwizzle, 0xFF, sizeof( int ) * 16 );

	while( !strError.size() && ( eStage != DONEPARSING ) )
	{
		if( neoglCheckError( string( "*** Unable to compile fragment shader, line [" ) + strLine + "] cause error: " ) )
			break;

		//Skip whitespace
		uiPos = m_strSource.find_first_not_of( " \t\n\r;", uiPos );

		if( m_strSource[ uiPos ] == '#' )
		{
			//Comment, skip line
			uiPos = m_strSource.find_first_of( "\n\r", uiPos );
			continue;
		}
		else if( uiPos == string::npos )
			break;

		strLine = Strip( m_strSource.substr( uiPos, ( uiLine = m_strSource.find_first_of( ";\n\r", uiPos ) ) - uiPos ) );

		uiPos = m_strSource.find_first_not_of( "; \t\n\r", uiLine );

		switch( eStage )
		{
			case HEADER:
			{
				if( strLine != "!!ATIfs1.0" )
					strError = "Invalid header";
				else
				{
					eLastStage = eStage;
					eStage     = ENTERSTAGE;
				}
				break;
			}

			case ENTERSTAGE:
			{
				if( strLine == "StartConstants" )
					eStage = CONSTANTS;
				else if( ( strLine == "StartPrelimPass" ) || ( strLine == "StartOutputPass" ) )
				{
					bLastWasColor = false;
					uiNumPairs = 0;
					eStage = ( ( strLine == "StartPrelimPass" ) ? PRELIM : OUTPUT );
					++uiPasses;
					memset( uiRegSampled, 0, sizeof( unsigned int ) * 10 );
				}
				else
				{
					strError = "Syntax error near [" + strLine + "] : expected block start";
					break;
				}

				if( eStage <= eLastStage )
					strError = "Invalid stage order : " + strLine + " not allowed";

				break;
			}

			case CONSTANTS:
			{
				if( strLine == "EndConstants" )
				{
					eLastStage = eStage;
					eStage     = ENTERSTAGE;
					break;
				}

				strToken = strLine.substr( 0, ( uiToken = strLine.find_first_of( " \t" ) ) );

				if( strToken == "CONSTANT" )
				{
					uiToken = strLine.find_first_not_of( " \t", uiToken );

					strToken = strLine.substr( uiToken, strLine.find_first_of( " \t", uiToken ) - uiToken );

					if( ( strToken.length() != 2 ) || ( strToken[0] != 'c' ) || ( ( strToken[1] < '0' ) || ( strToken[1] > '7' ) ) )
					{
						strError = "Invalid constant register identifier [" + strToken + "] : expected [c0,...,c7]";
						break;
					}

					int iReg = (int)strToken[1] - (int)'0';

					uiToken += 2;
					strToken = strLine.substr( strLine.find_first_not_of( " \t=", uiToken ), string::npos );

					vector< string > vstrVal;

					if( strToken[0] == '{' )
					{
						Explode( "{ \t,};", strToken, &vstrVal );

						float afVal[4];

						unsigned int uiVal = 0;

						for( ; uiVal < vstrVal.size(); ++uiVal )
							afVal[ uiVal ] = (float)atof( vstrVal[ uiVal ].c_str() );

						for( ; uiVal < 4; ++uiVal )
							afVal[ uiVal ] = 0.0f;

						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + iReg, afVal );
					}
					else
					{
						Explode( ".[];", strToken, &vstrVal );

						if( ( vstrVal.size() != 3 ) || ( vstrVal[0] != "program" ) || ( ( vstrVal[1] != "local" ) && ( vstrVal[1] != "env" ) ) )
							strError = "Syntax error, invalid constant specifier [" + strToken + "] : expected [program.<local|env>]";
						else
						{
							if( vstrVal[1] == "env" )
								neolog << LogLevel( WARNING ) << "*** [env] parameters not supported yet, parsing as [local] parameter" << endl;

							ShaderParam *pkParam = new ShaderParam;

							pkParam->m_eType = ShaderParam::PPT_COLOR;
							pkParam->m_uiIndex = (unsigned int)iReg * 4;
							pkParam->m_Value.m_pkColor = &Color::BLACK;

							m_kParams.Insert( "c" + char( '0' + iReg ), pkParam );
							m_vpkParams.push_back( pkParam );
						}
					}
				}
				else
					strError = "Syntax error near [" + strToken + "] : expected CONSTANT";

				break;
			}

			case PRELIM:
			case OUTPUT:
			{
				if( strLine == "EndPass" )
				{
					if( bLastWasColor )
						++uiNumPairs;

					eLastStage = eStage;
					eStage     = ( ( eLastStage == OUTPUT ) ? DONEPARSING : ENTERSTAGE );

					if( !uiNumPairs )
					{
						strError = ( ( eLastStage == OUTPUT ) ? string( "Output" ) : string( "Preliminary" ) ) + " pass must have at least one op";
						break;
					}

					bLastWasColor = false;
					uiNumPairs = 0;
					break;
				}

				vector<string> vstrTokens;

				Explode( " \t,", strLine, &vstrTokens );

				int iReg = 0, iTex = 0, iSwizzle = 0;

				if( ( vstrTokens.size() < 3 ) || ( vstrTokens.size() > 5 ) )
				{
					strError = "Syntax error on line [" + strLine + "]";
					break;
				}

				if( ( vstrTokens[0] == "PassTexCoord" ) || ( vstrTokens[0] == "SampleMap" ) )
				{
					if( ( vstrTokens.size() != 3 ) || !ParseArg( vstrTokens[1], &iReg, 0, 0, DST ) || !ParseArg( vstrTokens[2], &iTex, &iSwizzle, 0, TEX ) )
					{
						strError = "Syntax error, invalid " + vstrTokens[0] + " arguments on line [" + strLine + "]";
						break;
					}
					
					if( ( ( eStage == PRELIM ) || ( uiPasses != 2 ) ) && ( iTex >= GL_REG_0_ATI ) && ( iTex <= GL_REG_9_ATI ) )
					{
						strError = "Invalid " + vstrTokens[0] + " call on line [" + strLine + "] : register only allowed as source in second pass of a two-pass shader";
						break;
					}
					
					if( uiRegSampled[ iReg - GL_REG_0_ATI ] )
					{
						strError = "Invalid " + vstrTokens[0] + " call on line [" + strLine + "] : register already used as destination in previous sample/pass call in pass";
						break;
					}
					
					if( ( iTex >= GL_TEXTURE0_ARB ) && ( iTex <= ( GL_TEXTURE0_ARB + 16 ) ) )
					{
						if( ( ( ( iTexSwizzle[ iTex - GL_TEXTURE0_ARB ] == GL_SWIZZLE_STR_ATI ) || ( iTexSwizzle[ iTex - GL_TEXTURE0_ARB ] == GL_SWIZZLE_STR_DR_ATI ) ) &&
					          ( ( iSwizzle == GL_SWIZZLE_STQ_ATI ) || ( iSwizzle == GL_SWIZZLE_STQ_DQ_ATI ) ) ) ||
						    ( ( ( iTexSwizzle[ iTex - GL_TEXTURE0_ARB ] == GL_SWIZZLE_STQ_ATI ) || ( iTexSwizzle[ iTex - GL_TEXTURE0_ARB ] == GL_SWIZZLE_STQ_DQ_ATI ) ) &&
					          ( ( iSwizzle == GL_SWIZZLE_STR_ATI ) || ( iSwizzle == GL_SWIZZLE_STR_DR_ATI ) ) ) )
						{
							strError = "Invalid " + vstrTokens[0] + " call on line [" + strLine + "] : texture previously used with incompatible swizzle";
							break;
						}

						iTexSwizzle[ iTex - GL_TEXTURE0_ARB ] = iSwizzle;
					}
					
					if( ( iTex >= GL_REG_0_ATI ) && ( iTex <= GL_REG_9_ATI ) && ( ( iSwizzle == GL_SWIZZLE_STQ_ATI ) || ( iSwizzle == GL_SWIZZLE_STQ_DQ_ATI ) ) )
					{
						strError = "Invalid " + vstrTokens[0] + " call on line [" + strLine + "] : incompatible swizzle for register source";
						break;
					}

					uiRegSampled[ iReg - GL_REG_0_ATI ] = 1;

					if( vstrTokens[0] == "PassTexCoord" )
						neoglPassTexCoordATI( iReg, iTex, iSwizzle );
					else
						neoglSampleMapATI( iReg, iTex, iSwizzle );

					break;
				}

				int iArgs = 0;
				GLenum eOp;
				
				if( ( ( vstrTokens[0] == "MOV"     ) && ( eOp = GL_MOV_ATI     , iArgs = 1 ) ) ||
				    ( ( vstrTokens[0] == "ADD"     ) && ( eOp = GL_ADD_ATI     , iArgs = 2 ) ) ||
			        ( ( vstrTokens[0] == "SUB"     ) && ( eOp = GL_SUB_ATI     , iArgs = 2 ) ) ||
			        ( ( vstrTokens[0] == "MUL"     ) && ( eOp = GL_MUL_ATI     , iArgs = 2 ) ) ||
			        ( ( vstrTokens[0] == "DOT3"    ) && ( eOp = GL_DOT3_ATI    , iArgs = 2 ) ) ||
			        ( ( vstrTokens[0] == "DOT4"    ) && ( eOp = GL_DOT4_ATI    , iArgs = 2 ) ) ||
				    ( ( vstrTokens[0] == "MAD"     ) && ( eOp = GL_MAD_ATI     , iArgs = 3 ) ) ||
			        ( ( vstrTokens[0] == "LERP"    ) && ( eOp = GL_LERP_ATI    , iArgs = 3 ) ) ||
			        ( ( vstrTokens[0] == "CND"     ) && ( eOp = GL_CND_ATI     , iArgs = 3 ) ) ||
			        ( ( vstrTokens[0] == "CND0"    ) && ( eOp = GL_CND0_ATI    , iArgs = 3 ) ) ||
			        ( ( vstrTokens[0] == "DOT2ADD" ) && ( eOp = GL_DOT2_ADD_ATI, iArgs = 3 ) ) )
				{
					int iDst = 0;
					int iDstSwizzle = 0;
					int iDstMod = 0;
					int iSrc[3]        = { 0, 0, 0 };
					int iSrcSwizzle[3] = { 0, 0, 0 };
					int iSrcMod[3]     = { 0, 0, 0 };

					if( vstrTokens.size() != ( iArgs + 2 ) )
					{
						strError = "Invalid number of arguments to op [" + vstrTokens[0] + "] on line [" + strLine + "]";
						break;
					}
					
					if( !ParseArg( vstrTokens[1], &iDst, &iDstSwizzle, &iDstMod, DST ) )
					{
						strError = "Invalid destination [" + vstrTokens[1] + "] on line [" + strLine + "]";
						break;
					}

					for( int iArg = 0; iArg < iArgs; ++iArg )
					{
						if( !ParseArg( vstrTokens[iArg+2], &iSrc[iArg], &iSrcSwizzle[iArg], &iSrcMod[iArg], SRC ) )
						{
							strError = "Invalid source argument [" + vstrTokens[iArg+2] + "] on line [" +strLine + "]";
							break;
						}

						if( ( ( iSrc[iArg] == GL_PRIMARY_COLOR ) && ( eStage == PRELIM ) ) ||
						    ( ( iSrc[iArg] == GL_SECONDARY_INTERPOLATOR_ATI ) && ( eStage == PRELIM ) ) )
						{
							strError = "Invalid source argument [" + vstrTokens[iArg+2] + "] on line [" + strLine + "] : color can only be used in output pass";
							break;
						}

						if( ( iSrc[iArg] == GL_SECONDARY_INTERPOLATOR_ATI ) && ( iSrcSwizzle[iArg] == GL_ALPHA ) )
						{
							strError = "Invalid source argument [" + vstrTokens[iArg+2] + "] on line [" + strLine + "] : no alpha on secondary color";
							break;
						}
					}

					if( !iDstSwizzle )
						iDstSwizzle = GL_RED_BIT_ATI | GL_BLUE_BIT_ATI | GL_GREEN_BIT_ATI;

					switch( iArgs )
					{
						case 1:	
						{
							if( iDstSwizzle & ~GL_ALPHA_BIT_ATI )
								neoglColorFragmentOp1ATI( eOp, iDst, iDstSwizzle & ~GL_ALPHA_BIT_ATI, iDstMod, iSrc[0], iSrcSwizzle[0], iSrcMod[0] );

							if( iDstSwizzle & GL_ALPHA_BIT_ATI )
								neoglAlphaFragmentOp1ATI( eOp, iDst, iDstMod, iSrc[0], iSrcSwizzle[0], iSrcMod[0] );
							
							break;
						}

						case 2:	
						{
							if( iDstSwizzle & ~GL_ALPHA_BIT_ATI )
								neoglColorFragmentOp2ATI( eOp, iDst, iDstSwizzle & ~GL_ALPHA_BIT_ATI, iDstMod, iSrc[0], iSrcSwizzle[0], iSrcMod[0], iSrc[1], iSrcSwizzle[1], iSrcMod[1] );

							if( iDstSwizzle & GL_ALPHA_BIT_ATI )
								neoglAlphaFragmentOp2ATI( eOp, iDst, iDstMod, iSrc[0], iSrcSwizzle[0], iSrcMod[0], iSrc[1], iSrcSwizzle[1], iSrcMod[1] );

							break;
						}

						case 3:	
						{
							if( ( iSrc[0] >= GL_CON_0_ATI ) && ( iSrc[0] <= GL_CON_7_ATI ) &&
							    ( iSrc[1] >= GL_CON_0_ATI ) && ( iSrc[1] <= GL_CON_7_ATI ) &&
							    ( iSrc[2] >= GL_CON_0_ATI ) && ( iSrc[2] <= GL_CON_7_ATI ) )
							{
								strError = "Invalid operation on line [" + strLine + "] : max two constants per operation";
								break;
							}

							if( iDstSwizzle & ~GL_ALPHA_BIT_ATI )
								neoglColorFragmentOp3ATI( eOp, iDst, iDstSwizzle & ~GL_ALPHA_BIT_ATI, iDstMod, iSrc[0], iSrcSwizzle[0], iSrcMod[0], iSrc[1], iSrcSwizzle[1], iSrcMod[1], iSrc[2], iSrcSwizzle[2], iSrcMod[2] );

							if( iDstSwizzle & GL_ALPHA_BIT_ATI )
								neoglAlphaFragmentOp3ATI( eOp, iDst, iDstMod, iSrc[0], iSrcSwizzle[0], iSrcMod[0], iSrc[1], iSrcSwizzle[1], iSrcMod[1], iSrc[2], iSrcSwizzle[2], iSrcMod[2] );

							break;
						}
					}

					if( bLastWasColor )
						uiNumPairs++;
					else if( iDstSwizzle & GL_ALPHA_BIT_ATI )
						uiNumPairs++;

					bLastWasColor = ( ( iDstSwizzle & GL_ALPHA_BIT_ATI ) == 0 );
				}
				else
					strError = "Syntax error on line [" + strLine + "] : invalid op";

				break;
			}
		}
	}

	neoglEndFragmentShaderATI();

	if( eStage != DONEPARSING )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to compile fragment shader" << endl << "  " << ( strError.length() ? strError : string( "No completed output pass" ) ) << endl;
		return false;
	}

	if( neoglCheckError( "*** Unable to compile fragment shader: " ) )
		return false;

	if( eStage == DONEPARSING ) //uiPos >= m_strSource.length() )
		m_bIsCompiled = true;

	ParseParams();

	return m_bIsCompiled;
}


bool ShaderATIFragmentShader::Bind( RenderPrimitive *pkPrimitive )
{
#ifdef _DEBUG
	if( !m_bIsCompiled )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to bind fragment shader: Shader not compiled" << endl;
		return false;
	}
#endif

	vector< ShaderParam* >::iterator ppkParam = m_vpkParams.begin();
	vector< ShaderParam* >::iterator ppkEnd   = m_vpkParams.end();

	for( ; ppkParam != ppkEnd; ++ppkParam )
		BindParam( *ppkParam, pkPrimitive );

	neoglBindFragmentShaderATI( m_uiShader );
	glEnable( GL_FRAGMENT_SHADER_ATI );

	neoglCheckError( "*** Error binding parameters: " );

	return true;
}


bool ShaderATIFragmentShader::BindParam( ShaderParam *pkParam, RenderPrimitive *pkPrimitive )
{
	static float afIdentity[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	float afVal[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	unsigned int uiLight = m_pkDevice->m_uiCurLight + pkParam->m_uiLight;

	switch( pkParam->m_eType )
	{
		case ShaderParam::PPT_SYSTEM:
		{
			switch( pkParam->m_Value.m_eSystemID )
			{
				case ShaderParam::PSP_LIGHT_POSITION:
				{
					if( ( uiLight < m_pkDevice->m_uiNumLights ) && m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTranslation();
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIRECTION:
				{
					if( ( uiLight < m_pkDevice->m_uiNumLights ) && m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = -m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTransform().GetColumn(2);
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_POSITION_OBJECTSPACE:
				{
					if( ( uiLight < m_pkDevice->m_uiNumLights ) && m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkPrimitive->m_kInvModelMatrix * m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTranslation();
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIRECTION_OBJECTSPACE:
				{
					if( ( uiLight < m_pkDevice->m_uiNumLights ) && m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode() )
						*((Vector3d*)afVal) = pkPrimitive->m_kInvModelMatrix * (-m_pkDevice->m_ppkLights[ uiLight ]->GetSceneNode()->GetWorldTransform().GetColumn(2));
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					break;
				}

				case ShaderParam::PSP_LIGHT_AMBIENT_COLOR:
				{
					if( uiLight < m_pkDevice->m_uiNumLights )
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, m_pkDevice->m_ppkLights[ uiLight ]->m_kAmbient );
					else
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_DIFFUSE_COLOR:
				{
					if( uiLight < m_pkDevice->m_uiNumLights )
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, m_pkDevice->m_ppkLights[ uiLight ]->m_kDiffuse );
					else
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_SPECULAR_COLOR:
				{
					if( uiLight < m_pkDevice->m_uiNumLights )
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, m_pkDevice->m_ppkLights[ uiLight ]->m_kSpecular );
					else
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_ATTENUATION:
				{
					if( uiLight < m_pkDevice->m_uiNumLights )
					{
						afVal[0] = m_pkDevice->m_ppkLights[ uiLight ]->m_fConstantAttenuation;
						afVal[1] = m_pkDevice->m_ppkLights[ uiLight ]->m_fLinearAttenuation;
						afVal[2] = m_pkDevice->m_ppkLights[ uiLight ]->m_fQuadraticAttenuation;
						afVal[3] = 1.0f;
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					}
					else
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_LIGHT_REP_ATTENUATION:
				{
					if( uiLight < m_pkDevice->m_uiNumLights )
					{
						afVal[0] = 1.0f / m_pkDevice->m_ppkLights[ uiLight ]->m_fConstantAttenuation;
						afVal[1] = 1.0f / m_pkDevice->m_ppkLights[ uiLight ]->m_fLinearAttenuation;
						afVal[2] = 1.0f / m_pkDevice->m_ppkLights[ uiLight ]->m_fQuadraticAttenuation;
						afVal[3] = 1.0f;
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					}
					else
						neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afIdentity );
					break;
				}

				case ShaderParam::PSP_TOTAL_LIGHT_AMBIENT_COLOR:
				{
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, m_pkDevice->m_kTotalAmbientLight );
					break;
				}

				case ShaderParam::PSP_TOTAL_AMBIENTLIGHT_AMBIENT_COLOR:
				{
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, m_pkDevice->m_kAmbientLight );
					break;
				}

				case ShaderParam::PSP_EYE_POSITION_OBJECTSPACE:
				{
					*((Vector3d*)afVal) = ( m_pkDevice->GetViewMatrix() * pkPrimitive->m_kInvModelMatrix ).GetColumn(3);
					neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, afVal );
					break;
				}

				default:
				{
					neolog << LogLevel( ERROR ) << "*** Unable to bind system parameter: Unsupported type" << endl;
					break;
				}
			}
		}

		case ShaderParam::PPT_FLOAT4:
		{
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, pkParam->m_Value.m_afFloat );
			break;
		}

		case ShaderParam::PPT_COLOR:
		{
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, (const float*)pkParam->m_Value.m_pkColor );
			break;
		}

		case ShaderParam::PPT_VECTOR:
		{
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, (const float*)pkParam->m_Value.m_pkVector );
			break;
		}

		case ShaderParam::PPT_QUAT:
		{
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex, (const float*)pkParam->m_Value.m_pkQuaternion );
			break;
		}
/*
		case ShaderParam::PPT_MATRIX:
		{
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex    , (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[0][0] );
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex + 1, (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[1][0] );
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex + 2, (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[2][0] );
			neoglSetFragmentShaderConstantATI( GL_CON_0_ATI + pkParam->m_uiIndex + 3, (const float*)&pkParam->m_Value.m_pkMatrix->m_aafMatrix[3][0] );
			break;
		}
*/
		default:
		{
			neolog << LogLevel( ERROR ) << "*** Unable to bind fragment shader: Invalid parameter type" << endl;
			break;
		}
	}

	return true;
}


bool ShaderATIFragmentShader::Unbind()
{
	glDisable( GL_FRAGMENT_SHADER_ATI );
	return true;
}


bool ShaderATIFragmentShader::BindLightParams( RenderPrimitive *pkPrimitive )
{
	vector< ShaderParam* >::iterator ppkParam = m_vpkParams.begin();
	vector< ShaderParam* >::iterator ppkEnd   = m_vpkParams.end();

	for( ; ppkParam != ppkEnd; ++ppkParam )
	{
		if( ( (*ppkParam)->m_eType == ShaderParam::PPT_SYSTEM ) && ( (*ppkParam)->m_Value.m_eSystemID & ShaderParam::LIGHTPARAM ) )
			BindParam( *ppkParam, pkPrimitive );
	}

	return true;
}


};

