/***************************************************************************
                 shader-nv.cpp  -  nVidia register combiners
                             -------------------
    begin       : Wed Jan 7 2004
    copyright   : (C) 2004 by Reality Rift Studios
    email       : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, shader-nv.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.
 ***************************************************************************/

#include "shader-nv.h"
#include "rcparse/combiners.h"

#include <neoengine/shader.h>
#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{


ShaderNVRegisterCombiners::ShaderNVRegisterCombiners( Device *pkDevice, FileManager *pkFileManager ) :
	NeoOGL::Shader( pkDevice, FRAGMENTSHADER, pkFileManager ),
	m_pkCombiners( 0 )
{
}


ShaderNVRegisterCombiners::~ShaderNVRegisterCombiners()
{
	if( m_pkDevice->m_pkFragmentShader == this )
		glDisable( GL_REGISTER_COMBINERS_NV );

	delete m_pkCombiners;
}


bool ShaderNVRegisterCombiners::Compile( const string *pstrSource )
{
	if( m_bIsCompiled && !pstrSource )
		return true;

	ClearParams();

	if( pstrSource )
		m_strSource = *pstrSource;


	m_pkCombiners = new NVRegisterCombiners;

	if( !m_pkCombiners->Compile( m_strSource.c_str() ) )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to compile fragment shader: Error in parsing source" << endl;
		return false;
	}

	if( !m_pkCombiners->Validate() )
		return false;

	if( neoglCheckError( "*** Unable to compile fragment shader: " ) )
		return false;

	m_bIsCompiled = true;

	unsigned int i = 0;

	for( ; i < m_pkCombiners->m_uiNumConst; ++i )
	{
		ShaderParam *pkParam      = new ShaderParam;
		pkParam->m_eType           = ShaderParam::PPT_COLOR;
		pkParam->m_uiIndex         = i * 4;
		pkParam->m_uiLight         = 0;
		pkParam->m_Value.m_pkColor = &Color::BLACK;

		m_kParams.Insert( "c" + char( '0' + pkParam->m_uiIndex ), pkParam );
		m_vpkParams.push_back( pkParam );

		m_pkCombiners->m_pkParams[i] = pkParam;
	}

	for( int iGeneral = 0; iGeneral < m_pkCombiners->m_kGenerals.num; ++iGeneral )
	{
		for( int j = 0; j < m_pkCombiners->m_kGenerals.num; ++j )
		{
			for( int p = 0; p < m_pkCombiners->m_kGenerals.general[j].numConsts; ++p, ++i )
			{
				ShaderParam *pkParam      = new ShaderParam;
				pkParam->m_eType           = ShaderParam::PPT_COLOR;
				pkParam->m_uiIndex         = i * 4;
				pkParam->m_uiLight         = 0;
				pkParam->m_Value.m_pkColor = &Color::BLACK;

				m_kParams.Insert( "c" + char( '0' + pkParam->m_uiIndex ), pkParam );
				m_vpkParams.push_back( pkParam );

				m_pkCombiners->m_kGenerals.general[j].m_pkParams[p] = pkParam;
			}
		}
	}	

	ParseParams();

	return m_bIsCompiled;
}


bool ShaderNVRegisterCombiners::Bind( RenderPrimitive *pkPrimitive )
{
#ifdef _DEBUG
	if( !m_bIsCompiled || !m_pkCombiners )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to bind fragment shader: Shader not compiled" << endl;
		return false;
	}
#endif

	m_pkCombiners->Invoke( m_pkDevice, pkPrimitive );
	glEnable( GL_REGISTER_COMBINERS_NV );

	return true;
}


bool ShaderNVRegisterCombiners::Unbind()
{
	glDisable( GL_REGISTER_COMBINERS_NV );
	return true;
}


bool ShaderNVRegisterCombiners::BindLightParams( RenderPrimitive *pkPrimitive )
{
	m_pkCombiners->BindLightParams( m_pkDevice, pkPrimitive );
	return true;
}


};

