/***************************************************************************
                       shader.cpp  -  Pipeline shaders
                             -------------------
    begin       : Mon May 05 2003
    copyright   : (C) 2003 by Reality Rift Studios
    email       : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, shader.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 Contributors: Cody Russell (cody [at] jhu.edu)

 ***************************************************************************/

#include "shader.h"
#include "shader-arb.h"
#include "shader-ati.h"
#include "shader-nv.h"

#include <neoengine/logstream.h>

using namespace std;
using namespace NeoEngine;


namespace NeoOGL
{


NeoEngine::ShaderPtr Device::CreateShader( NeoEngine::Shader::SHADERTYPE eType )
{
	if( eType == Shader::VERTEXSHADER ) switch( Shader::s_eVSTarget )
	{
		case Shader::NEPT_GL_ARBVP_1:
			return new ShaderARB( this, Shader::VERTEXSHADER );

		case Shader::NOTSUPPORTED:
			break;

		default:
			neolog << LogLevel( ERROR ) << "*** Unable to create vertex shader: Unknown native vertex shader target [" << Shader::s_eVSTarget << "]" << endl;
			break;
	}
	else if( eType == Shader::FRAGMENTSHADER ) switch( Shader::s_eFSTarget )
	{
		case Shader::NEPT_GL_ATIFS:
			return new ShaderATIFragmentShader( this );

		case Shader::NEPT_GL_NVRC:
		case Shader::NEPT_GL_NVRC_2:
			return new ShaderNVRegisterCombiners( this );

		case Shader::NEPT_GL_ARBFP_1:
			return new ShaderARB( this, Shader::FRAGMENTSHADER );

		case Shader::NOTSUPPORTED:
			break;

		default:
			neolog << LogLevel( ERROR ) << "*** Unable to create fragment shader: Unknown native fragment shader target [" << Shader::s_eFSTarget << "]" << endl;
			break;
	}
	else
		neolog << LogLevel( ERROR ) << "*** Unable to create shader: Unknown type [" << eType << "]" << endl;

	return 0;
}


NeoEngine::ShaderPtr Device::GetShader( NeoEngine::Shader::SHADERTYPE eType, const std::string &rstrName )
{
	return ( ( eType == Shader::VERTEXSHADER ) ? Shader::s_kVSTable : Shader::s_kFSTable ).Find( rstrName );
}



ShaderTable Shader::s_kVSTable;
ShaderTable Shader::s_kFSTable;


Shader::Shader( Device *pkDevice, SHADERTYPE eType, FileManager *pkFileManager ) :
	NeoEngine::Shader( eType, pkFileManager ),
	m_pkDevice( pkDevice ),
	m_bIsCompiled( false )
{
}


Shader::~Shader()
{
	( ( GetType() == VERTEXSHADER ) ? s_kVSTable : s_kFSTable ).Delete( m_strName, this );

	if( m_pkDevice->m_pkVertexShader == this )
		m_pkDevice->m_pkVertexShader = 0;
	else if( m_pkDevice->m_pkFragmentShader == this )
		m_pkDevice->m_pkFragmentShader = 0;
}


void Shader::SetName( const HashString &rstrName )
{
	if( rstrName == GetName() )
		return;

	if( m_bIsCompiled )
	{
		ShaderTable &rkTable = ( ( GetType() == VERTEXSHADER ) ? s_kVSTable : s_kFSTable );
		
		rkTable.Delete( m_strName, this );

		m_strName = rstrName;

		rkTable.Insert( m_strName, this );
	}
	else
		m_strName = rstrName;
}
	

};

