/***************************************************************************
              vertexbuffer-ati.cpp  -  ATI vertex buffer abstraction
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, vertexbuffer-ati.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "vertexbuffer-ati.h"

#include "extensions.h"

using namespace NeoEngine;


namespace NeoOGL
{


VertexBufferATI::VertexBufferATI( Device *pkDevice, VertexBufferManagerATI *pkBufferManager, unsigned int uiType, unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData ) :
	VertexBufferGL( pkDevice, uiType, 0, 0, 0 ),
	m_pkBufferManager( pkBufferManager ),
	m_uiObject( 0 )
{
	AllocateVertices( uiNumVertices, pkFormat, pData );
}


VertexBufferATI::~VertexBufferATI()
{
	if( m_uiObject )
		neoglFreeObjectBufferATI( m_uiObject );
}


void VertexBufferATI::AllocateVertices( unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData )
{
	//Completely override normal allocator to ensure 32 byte alignment AND to allow NOREADWRITE buffers

	assert( !m_uiLock );

	if( ( uiNumVertices == m_uiNumAllocated ) && pkFormat && m_pkVertexFormat && ( *m_pkVertexFormat == *pkFormat ) && !( m_uiType & Buffer::NOREADWRITE ) )
	{
		m_uiNumCurrent = m_uiNumAllocated;

		if( !pData || !uiNumVertices )
			return;

		Lock( Buffer::WRITE );

		LoadVertexData( pData );

		Unlock();

		return;
	}

	delete [] m_pucBuffer;

	if( m_uiObject )
		neoglFreeObjectBufferATI( m_uiObject ), m_uiObject = 0;

	m_pucBuffer = 0;

	if( pkFormat )
	{
		delete m_pkVertexFormat;
		m_pkVertexFormat  = pkFormat->Duplicate();
	}

	m_uiVertexFormatSize  = m_pkVertexFormat ? m_pkVertexFormat->GetVertexSize() : 0;
	m_uiNumAllocated      = m_pkVertexFormat ? uiNumVertices : 0;
	m_uiNumCurrent        = m_uiNumAllocated;

	//Align elements on even 32 byte boundaries
	//if( m_uiVertexFormatSize % 32 )
		//m_uiVertexFormatSize = ( m_uiVertexFormatSize - ( m_uiVertexFormatSize % 32 ) ) + 32;

	if( !m_uiNumAllocated )
		return;

	if( !( m_uiType & Buffer::NOREADWRITE ) )
		m_pucBuffer = new unsigned char[ m_uiVertexFormatSize * m_uiNumAllocated ];

	ParseDeclaration();

	if( !( m_uiType & Buffer::NOREADWRITE ) )
	{
		if( pData )
		{
			Lock( Buffer::WRITE );

			LoadVertexData( pData );

			Unlock();
		}
	}
	else
	{
		m_uiObject = neoglNewObjectBufferATI( m_uiNumAllocated * m_uiVertexFormatSize, pData, GL_STATIC_ATI );
		m_bDirty   = false;
	}
}


void VertexBufferATI::Upload()
{
	if( !m_uiNumCurrent )
		return;

	if( !m_uiObject )
		m_uiObject = neoglNewObjectBufferATI( m_uiNumCurrent * m_uiVertexFormatSize, m_pucBuffer, ( ( m_uiType & Buffer::WRITEPRIORITIZED ) || !( m_uiType & Buffer::STATIC ) ) ? GL_DYNAMIC_ATI : GL_STATIC_ATI );
	else
		neoglUpdateObjectBufferATI( m_uiObject, 0, m_uiNumCurrent * m_uiVertexFormatSize, m_pucBuffer, GL_DISCARD_ATI );

	VertexBufferGL::Upload();
}


}; // namespace NeoOGL
