/***************************************************************************
         vertexbuffer.hcpp -  Base OpenGL vertex buffer abstraction
                             -------------------
    begin                : Fri Aug 1 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, vertexbuffer.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "vertexbuffer.h"

using namespace NeoEngine;


namespace NeoOGL
{


VertexBufferGL::VertexBufferGL( Device *pkDevice, unsigned int uiType, unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData ) :
	VertexBuffer( uiType, uiNumVertices, pkFormat, pData ),
	m_pkDevice( pkDevice ),
	m_pkPositionElement( 0 ),
	m_pkNormalElement( 0 ),
	m_pkDiffuseElement( 0 ),
	m_pkTangentElement( 0 ),
	m_pkBinormalElement( 0 ),
	m_pkFogElement( 0 )
{
	memset( m_pkTexCoordElement, 0, 32 );

	if( uiNumVertices || pkFormat )
		ParseDeclaration();
}


VertexBufferGL::~VertexBufferGL()
{
}


void VertexBufferGL::AllocateVertices( unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData )
{
	VertexBuffer::AllocateVertices( uiNumVertices, pkFormat, pData );

	ParseDeclaration();
}


void VertexBufferGL::ParseDeclaration()
{
	if( !m_pkVertexFormat )
		return;

	for( unsigned int uiElement = 0; uiElement < m_pkVertexFormat->m_uiNumElements; ++uiElement )
	{
		VertexElement *pkElement = &m_pkVertexFormat->m_pkElements[ uiElement ];

		if( pkElement->m_uiUsage == VertexElement::POSITION )
			m_pkPositionElement = pkElement;
		else if( pkElement->m_uiUsage == VertexElement::NORMAL )
			m_pkNormalElement = pkElement;
		else if( pkElement->m_uiUsage == VertexElement::DIFFUSECOLOR )
			m_pkDiffuseElement = pkElement;
		else if( pkElement->m_uiUsage == VertexElement::TEXCOORD )
			m_pkTexCoordElement[ pkElement->m_uiIndex ] = pkElement;
		else if( pkElement->m_uiUsage == VertexElement::TANGENT )
			m_pkTangentElement = pkElement;
		else if( pkElement->m_uiUsage == VertexElement::BINORMAL )
			m_pkBinormalElement = pkElement;
		else if( pkElement->m_uiUsage == VertexElement::FOG )
			m_pkFogElement = pkElement;
	}
}


};

