/***************************************************************************
     vertexbuffermanager-arb.cpp  -  ARB vertex buffer allocation manager
                             -------------------
    begin                : Fri Aug 15 2003
    copyright            : (C) 2003 by Cody Russell
    email                : cody@jhu.edu
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, vertexbuffermanager-arb.cpp

 The Initial Developer of the Original Code is Cody Russell.
 Portions created by Cody Russell are Copyright (C) 2003
 Cody Russell. All Rights Reserved.

 Contributors: Mattias Jansson (mattias@realityrift.com)

 ***************************************************************************/

#include "shader.h"
#include "vertexbuffermanager-arb.h"
#include "vertexbuffer-arb.h"
#include "texture.h"
#include "textureunit.h"

using namespace NeoEngine;


namespace NeoOGL
{


VertexBufferManagerARB::VertexBufferManagerARB( Device *pkDevice ) :
	VertexBufferManager( pkDevice )
{
}


VertexBufferManagerARB::~VertexBufferManagerARB()
{
}


VertexBufferPtr VertexBufferManagerARB::CreateVertexBuffer( unsigned int uiType, unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData )
{
	if( uiType & Buffer::NORENDER )
		return new NeoEngine::VertexBuffer( uiType, uiNumVertices, pkFormat, pData );

	return new VertexBufferARB( m_pkDevice, this, uiType, uiNumVertices, pkFormat, pData );
}


void VertexBufferManagerARB::SetBasePointers( VertexBufferGL *pkVertexBuffer, unsigned int uiUVLayer )
{
	VertexBufferARB *pkBuffer = static_cast< VertexBufferARB* >( pkVertexBuffer );
	int              iStride  = pkBuffer->GetVertexSize();

	BindBuffer( pkBuffer );

	glVertexPointer( 3, GL_FLOAT, iStride, (char*)0 + pkBuffer->m_pkPositionElement->m_uiOffset );

	if( pkBuffer->m_pkNormalElement )
	{
		glNormalPointer( GL_FLOAT, iStride, (char*)0 + pkBuffer->m_pkNormalElement->m_uiOffset );

		if( !m_bNormalArrayEnabled )
		{
			glEnableClientState( GL_NORMAL_ARRAY );
			m_bNormalArrayEnabled = true;
		}
	}
	else if( m_bNormalArrayEnabled )
	{
		glDisableClientState( GL_NORMAL_ARRAY );
		m_bNormalArrayEnabled = false;
	}

	if( pkBuffer->m_pkDiffuseElement )
	{
		if( pkBuffer->m_pkDiffuseElement->m_uiType == VertexElement::COLOR32 )
			glColorPointer( 4, GL_UNSIGNED_BYTE, iStride, (char*)0 + pkBuffer->m_pkDiffuseElement->m_uiOffset );
		else
			glColorPointer( pkBuffer->m_pkDiffuseElement->m_uiType + 1, GL_FLOAT, iStride, (char*)0 + pkBuffer->m_pkDiffuseElement->m_uiOffset );

		if( !m_bDiffuseArrayEnabled )
		{
			glEnableClientState( GL_COLOR_ARRAY );
			m_bDiffuseArrayEnabled = true;
		}
	}
	else if( m_bDiffuseArrayEnabled )
	{
		glDisableClientState( GL_COLOR_ARRAY );
		m_bDiffuseArrayEnabled = false;
	}

	if( pkBuffer->m_pkTexCoordElement[ uiUVLayer ] )
	{
		glTexCoordPointer( pkBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiType + 1, GL_FLOAT, iStride, (char*)0 + pkBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiOffset );

		if( !m_bTexCoordArrayEnabled[ uiUVLayer ] )
		{
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );
			m_bTexCoordArrayEnabled[ uiUVLayer ] = true;
		}
	}
	else if( m_bTexCoordArrayEnabled[ uiUVLayer ] )
	{
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		m_bTexCoordArrayEnabled[ uiUVLayer ] = false;
	}

	if( pkBuffer->m_pkTangentElement )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				if( pkBuffer->m_pkTangentElement->m_uiType == VertexElement::COLOR32 )
					neoglVertexAttribPointerARB( 14, 4, GL_UNSIGNED_BYTE, GL_TRUE, iStride, (char*)0 + pkBuffer->m_pkTangentElement->m_uiOffset );
				else
					neoglVertexAttribPointerARB( 14, pkBuffer->m_pkTangentElement->m_uiType + 1, GL_FLOAT, GL_FALSE, iStride, (char*)0 + pkBuffer->m_pkTangentElement->m_uiOffset );
				neoglEnableVertexAttribArrayARB( 14 );
                break;
			}
/*
			case NEPT_NVVP_30:
			{
				neoglVertexAttribPointerNV( 14, pkBuffer->m_pkTangentElement->m_uiType + 1, GL_FLOAT, iStride, (char*)pkBuffer->m_pkTangentElement->m_uiOffset );
				glEnableClientState( GL_VERTEX_ATTRIB_ARRAY14_NV );
				break;
			}
*/
			default:
				break;
		}

		m_bTangentArrayEnabled = true;
	}
	else if( m_bTangentArrayEnabled )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				neoglDisableVertexAttribArrayARB( 14 );
				break;
			}
/*
			case CG_PROFILE_VP20:
			case CG_PROFILE_VP30:
				glDisableClientState( GL_VERTEX_ATTRIB_ARRAY14_NV );
				break;
*/
			default:
				break;
		}

		m_bTangentArrayEnabled = false;
	}

	if( pkBuffer->m_pkBinormalElement )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				if( pkBuffer->m_pkTangentElement->m_uiType == VertexElement::COLOR32 )
					neoglVertexAttribPointerARB( 15, 4, GL_UNSIGNED_BYTE, GL_TRUE, iStride, (char*)0 + pkBuffer->m_pkTangentElement->m_uiOffset );
				else
					neoglVertexAttribPointerARB( 15, pkBuffer->m_pkTangentElement->m_uiType + 1, GL_FLOAT, GL_FALSE, iStride, (char*)0 + pkBuffer->m_pkTangentElement->m_uiOffset );
				neoglEnableVertexAttribArrayARB( 15 );
                break;
			}
/*
			case NEPT_NVVP_30:
			{
				neoglVertexAttribPointerNV( 15, pkBuffer->m_pkTangentElement->m_uiType + 1, GL_FLOAT, iStride, (char*)0 + pkBuffer->m_pkTangentElement->m_uiOffset );
				glEnableClientState( GL_VERTEX_ATTRIB_ARRAY15_NV );
				break;
			}
*/
			default:
				break;
		}

		m_bBinormalArrayEnabled = true;
	}
	else if( m_bBinormalArrayEnabled )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				neoglDisableVertexAttribArrayARB( 15 );
				break;
			}
/*
			case CG_PROFILE_VP20:
			case CG_PROFILE_VP30:
				glDisableClientState( GL_VERTEX_ATTRIB_ARRAY15_NV );
				break;
*/
			default:
				break;
		}

		m_bBinormalArrayEnabled = false;
	}

	if( pkBuffer->m_pkFogElement )
	{
		neoglFogCoordPointerEXT( GL_FLOAT, iStride, (char*)0 + pkBuffer->m_pkFogElement->m_uiOffset );

		if( !m_bFogCoordArrayEnabled )
		{
			glEnableClientState( GL_FOG_COORDINATE_ARRAY_EXT );
			m_bFogCoordArrayEnabled = true;
		}
	}
	else if( m_bFogCoordArrayEnabled )
	{
		glDisableClientState( GL_FOG_COORDINATE_ARRAY_EXT );
		m_bFogCoordArrayEnabled = false;
	}
}


void VertexBufferManagerARB::SetTexLayerPointer( VertexBufferGL *pkVBuffer, unsigned int uiTexUnit, unsigned int uiUVLayer, bool bDisable )
{
	VertexBufferARB *pkBuffer = static_cast< VertexBufferARB* >( pkVBuffer );

	if( !bDisable && pkVBuffer && pkVBuffer->m_pkTexCoordElement[ uiUVLayer ] )
	{
		BindBuffer( pkBuffer );
		TextureUnit::SetActiveTextureUnit( uiTexUnit );
		glTexCoordPointer( pkVBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiType + 1, GL_FLOAT, pkVBuffer->GetVertexSize(), (char*)0 + pkVBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiOffset );

		if( !m_bTexCoordArrayEnabled[ uiTexUnit ] )
		{
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );
			m_bTexCoordArrayEnabled[ uiTexUnit ] = true;
		}
	}
	else if( m_bTexCoordArrayEnabled[ uiTexUnit ] )
	{
		BindBuffer( pkBuffer );
		TextureUnit::SetActiveTextureUnit( uiTexUnit );
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		m_bTexCoordArrayEnabled[ uiTexUnit ] = false;
	}
}


};



