/***************************************************************************
     vertexbuffermanager-arb.h  -  ARB vertex buffer allocation manager
                             -------------------
    begin                : Fri Aug 15 2003
    copyright            : (C) 2003 by Cody Russell
    email                : cody `at' jhu.edu
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, vertexbuffermanager-arb.h

 The Initial Developer of the Original Code is Cody Russell.
 Portions created by Cody Russell are Copyright (C) 2003
 Cody Russell. All Rights Reserved.

 Contributors: Mattias Jansson (mattias@realityrift.com)

 ***************************************************************************/

#ifndef __NEOGLVERTEXBUFFERMANAGER_ARB_H
#define __NEOGLVERTEXBUFFERMANAGER_ARB_H



/**
  * \file vertexbuffermanager-arb.h
  * ARB vertex buffer allocation manager
  */


#include "vertexbuffermanager.h"
#include "vertexbuffer-arb.h"


namespace NeoOGL
{


//External classes
class VertexBufferARB;


/**
  * \brief Manages vertex buffer allocations using ARB extension
  * \author Cody Russell (cody jhu edu)
  */
class VertexBufferManagerARB : public VertexBufferManager
{
	friend class VertexBufferARB;

	protected:

		/*! Points to the currently bound buffer.  NULL is none are bound. */
		VertexBufferARB                                    *m_pkBoundBuffer;


	public:

		/**
		* \param pkDevice                                   Device object
		*/
		                                                    VertexBufferManagerARB( Device *pkDevice );

		/**
		* Free memory
		*/
		virtual                                            ~VertexBufferManagerARB();

		/**
		* Allocate a vertex buffer
		* \param uiType                                     Buffer type
		* \param uiNumVertices                              Number of vertices
		* \param uiVertexFormat                             Flexible vertex format specifier
		* \param pData                                      Optional pointer to data to load buffer with
		* \return                                           Ptr to new vertex buffer
		*/
		virtual NeoEngine::VertexBufferPtr                  CreateVertexBuffer( unsigned int uiType, unsigned int uiNumVertices, const NeoEngine::VertexDeclaration *pkFormat, const void *pData );

		/**
		* Set base pointers for rendering
		* \param pkVertexBuffer                             Vertex buffer object
		* \param uiUVLayer                                  UV layer
		*/
		virtual void                                        SetBasePointers( VertexBufferGL *pkVertexBuffer, unsigned int uiUVLayer );

		/**
		* Set pointer for texture layer for rendering
		* \param pkVertexBuffer                             Vertex buffer object
		* \param uiTexUnit                                  Active texture unit
		* \param uiUVLayer                                  UV layer
		* \param bDisable                                   Force disable
		*/
		virtual void                                        SetTexLayerPointer( VertexBufferGL *pkVertexBuffer, unsigned int uiTexUnit, unsigned int uiUVLayer, bool bDisable = false );

		/**
		* Bind a particular buffer
		* \param pkVertexBuffer                             Vertex buffer object
		*/
		inline void                                         BindBuffer( VertexBufferARB *pkVertexBuffer )
		{
			if( m_pkBoundBuffer != pkVertexBuffer )
        		{
                		m_pkBoundBuffer = pkVertexBuffer;
                		neoglBindBufferARB( GL_ARRAY_BUFFER_ARB, m_pkBoundBuffer->m_uiObject );
        		}
		}
};


};


#endif
