/***************************************************************************
   vertexbuffermanager-ati.cpp  -  ATI vertex buffer allocation manager
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoDevOpenGL, vertexbuffermanager-ati.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "vertexbuffermanager-ati.h"
#include "vertexbuffer-ati.h"
#include "texture.h"
#include "textureunit.h"
#include "shader.h"

#include <neoengine/vertex.h>

using namespace NeoEngine;


namespace NeoOGL
{


VertexBufferManagerATI::VertexBufferManagerATI( Device *pkDevice ) :
	VertexBufferManager( pkDevice )
{
}


VertexBufferManagerATI::~VertexBufferManagerATI()
{
}


VertexBufferPtr VertexBufferManagerATI::CreateVertexBuffer( unsigned int uiType, unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData )
{
	if( uiType & Buffer::NORENDER )
		return new NeoEngine::VertexBuffer( uiType, uiNumVertices, pkFormat, pData );

	return new VertexBufferATI( m_pkDevice, this, uiType, uiNumVertices, pkFormat, pData );
}


void VertexBufferManagerATI::SetBasePointers( VertexBufferGL *pkVertexBuffer, unsigned int uiUVLayer )
{
	VertexBufferATI   *pkBuffer = static_cast< VertexBufferATI* >( pkVertexBuffer );
	int                iStride  = pkBuffer->GetVertexSize();

	neoglArrayObjectATI( GL_VERTEX_ARRAY, 3, GL_FLOAT, iStride, pkBuffer->m_uiObject, 0 );

	if( pkBuffer->m_pkNormalElement )
	{
		neoglArrayObjectATI( GL_NORMAL_ARRAY, 3, GL_FLOAT, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkNormalElement->m_uiOffset );

		if( !m_bNormalArrayEnabled )
		{
			glEnableClientState( GL_NORMAL_ARRAY );
			m_bNormalArrayEnabled = true;
		}
	}
	else if( m_bNormalArrayEnabled )
	{
		glDisableClientState( GL_NORMAL_ARRAY );
		m_bNormalArrayEnabled = false;
	}

	if( pkBuffer->m_pkDiffuseElement )
	{
		if( pkBuffer->m_pkDiffuseElement->m_uiType == VertexElement::COLOR32 )
			neoglArrayObjectATI( GL_COLOR_ARRAY, 4, GL_UNSIGNED_BYTE, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkDiffuseElement->m_uiOffset );
		else
			neoglArrayObjectATI( GL_COLOR_ARRAY, pkBuffer->m_pkDiffuseElement->m_uiType + 1, GL_FLOAT, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkDiffuseElement->m_uiOffset );

		if( !m_bDiffuseArrayEnabled )
		{
			glEnableClientState( GL_COLOR_ARRAY );
			m_bDiffuseArrayEnabled = true;
		}
	}
	else if( m_bDiffuseArrayEnabled )
	{
		glDisableClientState( GL_COLOR_ARRAY );
		m_bDiffuseArrayEnabled = false;
	}

	if( pkBuffer->m_pkTexCoordElement[ uiUVLayer ] )
	{
		neoglArrayObjectATI( GL_TEXTURE_COORD_ARRAY, pkBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiType + 1, GL_FLOAT, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiOffset );

		if( !m_bTexCoordArrayEnabled[ uiUVLayer ] )
		{
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );
			m_bTexCoordArrayEnabled[ uiUVLayer ] = true;
		}
	}
	else if( m_bTexCoordArrayEnabled[ uiUVLayer ] )
	{
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		m_bTexCoordArrayEnabled[ uiUVLayer ] = false;
	}

	if( pkBuffer->m_pkTangentElement )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				if( pkBuffer->m_pkTangentElement->m_uiType == VertexElement::COLOR32 )
					neoglVertexAttribArrayObjectATI( 14, 4, GL_UNSIGNED_BYTE, GL_TRUE, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkTangentElement->m_uiOffset );
				else
					neoglVertexAttribArrayObjectATI( 14, pkBuffer->m_pkTangentElement->m_uiType + 1, GL_FLOAT, GL_FALSE, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkTangentElement->m_uiOffset );
				neoglEnableVertexAttribArrayARB( 14 );
                break;
			}

			default:
				break;
		}

		m_bTangentArrayEnabled = true;
	}
	else if( m_bTangentArrayEnabled )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				neoglDisableVertexAttribArrayARB( 14 );
				break;
			}

			default:
				break;
		}

		m_bTangentArrayEnabled = false;
	}

	if( pkBuffer->m_pkBinormalElement )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				if( pkBuffer->m_pkTangentElement->m_uiType == VertexElement::COLOR32 )
					neoglVertexAttribArrayObjectATI( 15, 4, GL_UNSIGNED_BYTE, GL_TRUE, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkTangentElement->m_uiOffset );
				else
					neoglVertexAttribArrayObjectATI( 15, pkBuffer->m_pkTangentElement->m_uiType + 1, GL_FLOAT, GL_FALSE, iStride, pkBuffer->m_uiObject, pkBuffer->m_pkTangentElement->m_uiOffset );
				neoglEnableVertexAttribArrayARB( 15 );
                break;
			}

			default:
				break;
		}

		m_bBinormalArrayEnabled = true;
	}
	else if( m_bBinormalArrayEnabled )
	{
		switch( Shader::s_eVSTarget )
		{
			case Shader::NEPT_GL_ARBVP_1:
			{
				neoglDisableVertexAttribArrayARB( 15 );
				break;
			}

			default:
				break;
		}

		m_bBinormalArrayEnabled = false;
	}
}


void VertexBufferManagerATI::SetTexLayerPointer( VertexBufferGL *pkVertexBuffer, unsigned int uiTexUnit, unsigned int uiUVLayer, bool bDisable )
{
	if( !bDisable && pkVertexBuffer && pkVertexBuffer->m_pkTexCoordElement[ uiUVLayer ] )
	{
		TextureUnit::SetActiveTextureUnit( uiTexUnit );
		neoglArrayObjectATI( GL_TEXTURE_COORD_ARRAY, pkVertexBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiType + 1, GL_FLOAT, pkVertexBuffer->GetVertexSize(), static_cast< VertexBufferATI* >( pkVertexBuffer )->m_uiObject, pkVertexBuffer->m_pkTexCoordElement[ uiUVLayer ]->m_uiOffset );

		if( !m_bTexCoordArrayEnabled[ uiTexUnit ] )
		{
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );
			m_bTexCoordArrayEnabled[ uiTexUnit ] = true;
		}
	}
	else if( m_bTexCoordArrayEnabled[ uiTexUnit ] )
	{
		TextureUnit::SetActiveTextureUnit( uiTexUnit );
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		m_bTexCoordArrayEnabled[ uiTexUnit ] = false;
	}
}


};

