/***************************************************************************
          adaptor.cpp  -  Adaptor classes between hierarchy node types
                             -------------------
    begin                : Wed Nov 26 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, adaptor.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "adaptor.h"

using namespace NeoEngine;


namespace NeoEngine
{


BoneAdaptor::BoneAdaptor( Skeleton *pkSkeleton, Bone *pkParent, SceneNode *pkNode ) :
	Bone( 0, "", pkParent ),
	m_pkSkeleton( pkSkeleton ),
	m_pkSceneNode( pkNode )
{
}


BoneAdaptor::BoneAdaptor( BoneAdaptor &rkNode, Skeleton *pkSkeleton, Bone *pkParent ) :
	Bone( rkNode ),
	m_pkSkeleton( pkSkeleton ),
	m_pkSceneNode( rkNode.m_pkSceneNode ? rkNode.m_pkSceneNode->Duplicate() : 0 )
{
	pkParent->AttachNode( this );
	m_pkSkeleton->AttachAdaptor( this );
}


BoneAdaptor::~BoneAdaptor()
{
	m_pkSkeleton->DetachAdaptor( this );

	delete m_pkSceneNode;
}


void BoneAdaptor::NotifyUpdate( bool bRecurse )
{
	Bone::NotifyUpdate( bRecurse );

	m_pkSceneNode->SetTranslation( GetWorldTranslation() );
	m_pkSceneNode->SetRotation( GetWorldRotation() );
}


BoneAdaptor *BoneAdaptor::Duplicate( Skeleton *pkSkeleton, Bone *pkParent )
{
	return new BoneAdaptor( *this, pkSkeleton, pkParent );
}


};

