/***************************************************************************
                animatedsubmesh.cpp  -  Vertex animated submesh
                             -------------------
    begin                : Thu Jan 30 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, animatedsubmesh.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "animatedsubmesh.h"
#include "render.h"


namespace NeoEngine
{


AnimatedSubMesh::AnimatedSubMesh() :
	SubMesh(),
	SubMeshAnimatorController()
{
}


AnimatedSubMesh::AnimatedSubMesh( const AnimatedSubMesh &rkSubMesh ) :
	SubMesh( rkSubMesh ),
	SubMeshAnimatorController( rkSubMesh )
{
	m_pkVertices = 0;
}


AnimatedSubMesh::~AnimatedSubMesh()
{
}


void AnimatedSubMesh::Update( float fDeltaTime )
{
	SubMesh::Update( fDeltaTime );
	SubMeshAnimatorController::Update( fDeltaTime );

	m_bNeedUpdate = true;
}


void AnimatedSubMesh::UpdateData()
{
	if( !m_pkVertices )
		m_pkVertices = Core::Get()->GetRenderDevice()->CreateVertexBuffer( Buffer::DYNAMIC | Buffer::WRITEPRIORITIZED | Buffer::READPRIORITIZED );

	InterpolateVertices( m_pkVertices );

	// FIXME: What if shared polygon buffer ptr with other submeshes?
	if( m_pkPolygons->GetNormals() )
		m_pkPolygons->CalculateNormals( m_pkVertices );

	m_bNeedUpdate = false;
	m_bChanged    = true;
}


SubMesh *AnimatedSubMesh::Duplicate() const
{
	return( new AnimatedSubMesh( *this ) );
}


}; // namespace NeoEngine

