/***************************************************************************
                 animatedsubmesh.h  -  Vertex animated submesh
                             -------------------
    begin                : Thu Jan 30 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, animatedsubmesh.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEANIMATEDSUBMESH_H
#define __NEANIMATEDSUBMESH_H


#include "base.h"
#include "submesh.h"
#include "submeshanimator.h"


/**
  * \file animatedsubmesh.h
  * Vertex animated submesh
  */


namespace NeoEngine
{


/**
  * \brief A vertex animated submesh
  * \author Mattias Jansson (mattias@realityrift.com
  */
class NEOENGINE_API AnimatedSubMesh : public SubMesh, public SubMeshAnimatorController
{
	protected:


		/**
		* Update data
		*/
		virtual void                                  UpdateData();



	public:


		/**
		*/
		                                              AnimatedSubMesh();

		/**
		* \param rkSubMesh                            Reference submesh object to copy data from
		*/
		                                              AnimatedSubMesh( const AnimatedSubMesh &rkSubMesh );

		/**
		*/
		virtual                                      ~AnimatedSubMesh();

		/**
		* Update this submesh
		* \param fDeltaTime                           Delta time passed
		*/
		virtual void                                  Update( float fDeltaTime );

		/**
		* \return                                     new submesh that is duplicate of this
		*/
		virtual SubMesh                              *Duplicate() const;

		/**
		* \return                                     Sub mesh type
		*/
		virtual unsigned int                          GetType() const { return ANIMATEDSUBMESH; }
};


};


#endif

