/***************************************************************************
                        bitparser.h  -  Bit stream parser
                             -------------------
    begin                : Mon Feb 18 2002
    copyright            : (C) 2002 by David Holm
    email                : david@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, bitparser.cpp

 The Initial Developer of the Original Code is David Holm
 Portions created by David Holm are Copyright (C) 2002
 David holm. All Rights Reserved.

 Contributors: Mattias Jansson (mattias@realityrift.com)

 ***************************************************************************/

#include "bitparser.h"
#include "logstream.h"

#include <stdio.h>

using namespace std;


namespace NeoEngine
{


BitParser::~BitParser()
{
	m_pucBase = 0;
	m_pucBits = 0;
}


void BitParser::SetPointer( unsigned char *pucBitstream )
{
	m_pucBase      = pucBitstream;
	m_pucBits      = pucBitstream;
	m_ucBitOffset  = 0;
	m_ulByteOffset = 0;
}


uint32_t BitParser::ReadBits( int iBits )
{
	uint64_t ulRetval = 0;
	
	if( iBits > 32 )
	{
		iBits = 32;
		neolog << LogLevel( ERROR ) << "[BitParser::ReadBits] iBits out of range fixing to 32!" << endl;
	}

	ulRetval = uint32_t( ( (*(uint64_t*)m_pucBits) >> m_ucBitOffset ) & ( ( 1 << iBits ) - 1 ) );

	m_ucBitOffset += iBits;
		
	//Clean up offsets and pointer
	m_ulByteOffset += m_ucBitOffset / 8;
	m_pucBits      += m_ucBitOffset / 8;
	m_ucBitOffset   = m_ucBitOffset % 8;
		
	return (uint32_t)ulRetval;
}


void BitParser::Align()
{
	if( m_ucBitOffset )
	{
		m_ucBitOffset = 0;
		m_pucBits++;
		m_ulByteOffset++;
	}
}


void BitParser::Seek( int iBits, int iMode )
{
	switch( iMode )
	{
		case SEEK_SET:
		{
			m_ucBitOffset  = iBits % 8;
			m_ulByteOffset = iBits / 8;
			m_pucBits      = m_pucBase + m_ulByteOffset;
			break;
		}

		case SEEK_CUR:
		{
			m_ucBitOffset   += iBits % 8;
			m_ulByteOffset  += iBits / 8;

			m_ulByteOffset  += m_ucBitOffset / 8;
			m_ucBitOffset    = m_ucBitOffset % 8;

			m_pucBits        = m_pucBase + m_ulByteOffset;
			break;
		}

		default:
		{
			neolog << LogLevel( ERROR ) << "[BitParser::Seek] invalid seek mode [" << iMode << "]" << endl;
			break;
		}
	}
}

}; // namespace NeoEngine
