/***************************************************************************
                       buffer.cpp  -  Buffer base class
                             -------------------
    begin                : Tue Oct 30 2001
    copyright            : (C) 2001 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, buffer.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2001
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "buffer.h"
#include "strutil.h"

#include <vector>
#include <string>

using namespace std;

namespace NeoEngine
{

string Buffer::GetTypeAsString( unsigned int uiType )
{
    string strType;

    if( uiType & DYNAMIC )
        strType += "dynamic ";
    if( uiType & STATIC )
        strType += "static ";
    if( uiType & READPRIORITIZED )
        strType += "readpri ";
    if( uiType & WRITEPRIORITIZED )
        strType += "writepri ";
    if( uiType & NOREADWRITE )
        strType += "noreadwrite ";
    if( uiType & NORENDER )
        strType += "norender ";
    strType.erase( 0 , strType.find_first_not_of( " " ) ) ;

    return strType;
}

unsigned int Buffer::GetTypeFromString( std::string const &strType )
{
	//Parse string
	vector<string> vstrFlags;

	Explode( " \t", strType, &vstrFlags );

	unsigned int uiFlags = 0;

	vector<string>::iterator pstrFlag = vstrFlags.begin();
	vector<string>::iterator pstrEnd  = vstrFlags.end();

	for( ; pstrFlag != pstrEnd; ++pstrFlag )
	{
		//Strip whitespace
		unsigned int uiStart = pstrFlag->find_first_not_of( " \t" );
		unsigned int uiEnd   = pstrFlag->find_last_not_of( " \t" ) + 1;

		*pstrFlag = pstrFlag->substr( uiStart, uiEnd - uiStart );

		//Ignore empty strings
		if( !pstrFlag->length() )
			continue;

		if( *pstrFlag == "dynamic" )
			uiFlags |= Buffer::DYNAMIC;
		else if( *pstrFlag == "static" )
			uiFlags |= Buffer::STATIC;
		else if( *pstrFlag == "readpri" )
			uiFlags |= Buffer::READPRIORITIZED;
		else if( *pstrFlag == "writepri" )
			uiFlags |= Buffer::WRITEPRIORITIZED;
		else if( *pstrFlag == "norender" )
			uiFlags |= Buffer::NORENDER;
	}

	return uiFlags;
}


}; // namespace NeoEngine
