/***************************************************************************
                  filecodec.h  -  Virtual file I/O filter
                             -------------------
    begin                : Thu Oct 23 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, filecodec.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEFILECODEC_H
#define __NEFILECODEC_H


/**
  * \file filecodec.h
  * Virtual file I/O filters
  */


#include "base.h"
#include "pointer.h"
#include "filetype.h"
#include "module.h"

#include <string>


namespace NeoEngine
{


// External classes
class File;


/**
  * \brief Loads custom file data from a package
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOENGINE_API FileCodec : public FiletypeIdentifier
{
	public:

		/*! Library module */
		ModulePtr                                     m_pkModule;


		/*! File flag we parse */
		int                                           m_iFileFlag;


		/**
		* \param rstrFiletypeName                     File type name
		*/
		                                              FileCodec( const std::string &rstrFiletypeName );

		/**
		* \param rstrFiletypeName                     File type name
		* \param rvstrExtensions                      File type extensions
		*/
													  FileCodec( const std::string &rstrFiletypeName, const std::vector< std::string > &rvstrExtensions );

		/**
		*/
		virtual                                      ~FileCodec();

		/**
		* Loads data from file
		* \param pkFile                               File
		* \param iSrcSize                             Size of source data
		* \param pDest                                Pointer to destination buffer
		* \param piSize                               Pointer to integer holding size of destination buffer
		* \return                                     Error code, 0 if success
		*/
		virtual int                                   LoadFileData( File *pkFile, int iSrcSize, void *pDest, int *piSize ) = 0;
};


};


#endif
