/***************************************************************************
                    filetype.h  -  File type identifiers
                             -------------------
    begin                : Thu Oct 23 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, filetype.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEFILETYPE_H
#define __NEFILETYPE_H


/**
  * \file filetype.h
  * File type identifiers
  */


#ifndef __NEED_VECTOR_STRING
#  define __NEED_VECTOR_STRING
#endif

#include "base.h"

#include <vector>
#include <string>


namespace NeoEngine
{


//External classes
class File;
class FileManager;


/**
  * \brief Base class for file type identifier
  * A file type identifier identifies file type from data and file name
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOENGINE_API FiletypeIdentifier
{
	protected:
	
		/*! File type name */
		std::string                                   m_strTypeName;

		/*! File type extension */
		std::vector< std::string >                    m_vstrExtensions;
					

	public:

		/**
		* \param rstrFiletypeName                     Name of file type we identify
		* \param rvstrExtensions                      Vector of file type extensions
		*/
		                                              FiletypeIdentifier( const std::string &rstrFiletypeName, const std::vector< std::string > &rvstrExtensions );

		/**
		* \param rstrFiletypeName                     Name of file type we identify
		*/
		                                              FiletypeIdentifier( const std::string &rstrFiletypeName );
		
		/**
		*/
		virtual                                      ~FiletypeIdentifier();
	
		/**
		* Check if file is type (must NOT change file offset!)
		* \param pkFile                               File
		* \return                                     true if type, false if not
		*/
		virtual bool                                  IsType( File *pkFile ) = 0;

		/**
		* \return                                     Type name
		*/
		const std::string                            &GetTypeName() const;
		
		/**
		* \return                                     Filetype extension
		*/
		const std::vector< std::string >             &GetExtensions() const;
};


};


#endif
