/***************************************************************************
              keyframe.h  -  Base class for keyframes in animations
                             -------------------
    begin                : Fri Nov 23 2001
    copyright            : (C) 2001 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, keyframe.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2001
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEKEYFRAME_H
#define __NEKEYFRAME_H


#include "base.h"
#include "nemath.h"


/**
  * \file keyframe.h
  * Base class for animation keyframes
  */


namespace NeoEngine
{



/**
  * \class Keyframe
  * \brief Animation keyframe base class
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOENGINE_API Keyframe
{
	public:

		/*! Timestamp in [0,1] range */
		float                              m_fT;


		/**
		* \param fTime                     Timestamp
		*/
		                                   Keyframe( float fTime = 0.0f ) : m_fT( fTime ) {}

		/**
		* \param rkKeyframe                Keyframe reference object
		*/
		                                   Keyframe( const Keyframe &rkKeyframe ) : m_fT( rkKeyframe.m_fT ) {}

		/**
		*/
		virtual                           ~Keyframe() {}

		/**
		* Compare timestamps
		* \param rkKeyframe                Keyframe reference object to compare with
		* \return                          true if timestamp less than reference object timestamp
		*/
		inline bool                        operator < ( const Keyframe &rkKeyframe ) const { return( m_fT < rkKeyframe.m_fT ); }

		/**
		* Compare timestamps
		* \param rkKeyframe                Keyframe reference object to compare with
		* \return                          true if timestamp less are equal (epsilon interval)
		*/
		inline bool                        operator == ( const Keyframe &rkKeyframe ) const { return( fabs( m_fT - rkKeyframe.m_fT ) < 0.0001f ); }
};


}; // namespace NeoEngine
		


#endif
