/***************************************************************************
     loadableentity.cpp  -  Base class for object which can be loaded
                             -------------------
    begin                : Mon Sep 11 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, loadableentity.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "loadableentity.h"
#include "file.h"
#include "filemanager.h"
#include "logstream.h"
#include "core.h"

using namespace std;


namespace NeoEngine
{


LoadableEntity::LoadableEntity( FileManager *pkFileManager ) :
	m_pkFile( 0 ),
	m_bLoaded( false ),
	m_bKeepFile( false ),
	m_pkFileManager( pkFileManager ? pkFileManager : Core::Get()->GetFileManager() )
{
}


LoadableEntity::~LoadableEntity()
{
	delete m_pkFile;
}


void LoadableEntity::SetFileManager( FileManager *pkManager )
{
	m_pkFileManager = pkManager;
}


bool LoadableEntity::Load( const string &rstrFilename, unsigned int uiFlags, bool bForceReload, bool bSearchFileSystem )
{
	if( bForceReload )
		m_bLoaded = false;

	if( !rstrFilename.length() || m_bLoaded )
		return m_bLoaded;

	File *pkFile = 0;

	if( ( m_pkFileManager && ( pkFile = m_pkFileManager->GetByName( rstrFilename ) ) ) ||
	    ( bSearchFileSystem && ( pkFile = new File( File::ExtractPathName( rstrFilename ), File::ExtractFileName( rstrFilename ) ) ) ) )
		Load( pkFile, uiFlags, bForceReload );

	if( !m_bKeepFile || ( m_pkFile != pkFile ) )
		delete pkFile;

	return m_bLoaded;
}


bool LoadableEntity::Load( File *pkFile, unsigned int uiFlags, bool bForceReload )
{
	if( bForceReload )
		m_bLoaded = false;

	if( !pkFile || m_bLoaded )
		return m_bLoaded;

	m_pkFile  = pkFile;

	if( m_pkFile->Open() )
		m_bLoaded = LoadNode( uiFlags );
	else
		neolog << LogLevel( WARNING ) << "[LoadableEntity::Load]: Unable to Open() file [" << ( pkFile->GetPath().length() ? pkFile->GetPath() + "/" : "" ) << pkFile->GetName() << "]" << endl;

	if( !m_bKeepFile )
		m_pkFile = 0;

	return m_bLoaded;
}


}; // namespace NeoEngine

