/***************************************************************************
                massparticle.cpp  -  Massive particle physics node
                             -------------------
    begin                : Sat Jan 3 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, massparticle.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "massparticle.h"

using namespace std;


namespace NeoEngine
{


MassParticle::MassParticle() :
	PhysicsNode(),
	m_fMass( 0.0f ),
	m_fInvMass( 0.0f )
{
}


MassParticle::~MassParticle()
{
}


SceneNode *MassParticle::Duplicate()
{
	assert( !"[MassParticle::Duplicate] Not implemented!" );
	return 0;
}


void MassParticle::SetMass( float fMass )
{
	m_fMass = fMass;

	if( m_fMass != 0.0f )
		m_fInvMass = 1.0f / m_fMass;
	else
		m_fInvMass = 0.0f;
}


const Vector3d &MassParticle::ApplyForce( const Vector3d &rkForce )
{
	return( m_kForce += rkForce );
}


void MassParticle::ApplyImpulse( const Vector3d& rkImpulse )
{
	m_kVelocity += rkImpulse * m_fInvMass;
}


void MassParticle::Update( float fDeltaTime )
{
	SceneNode::Update( fDeltaTime );

	Vector3d kVp = m_kForce * ( m_fInvMass * fDeltaTime );

	TranslateWorld( ( kVp * ( fDeltaTime * 0.5f ) ) + ( m_kVelocity * fDeltaTime ) );

	m_kVelocity += kVp;
}


void MassParticle::ResetForce()
{
	m_kForce.Reset();
}


void MassParticle::PushState()
{
	PhysicsNode::PushState();

	m_vkStateStack.insert( m_vkStateStack.begin(), MassParticleState( m_kForce, m_fMass, m_fInvMass ) );
}


void MassParticle::PopState( bool bSet )
{
	PhysicsNode::PopState( bSet );

	if( m_vkStateStack.size() )
	{
		vector< MassParticleState >::iterator pkState = m_vkStateStack.begin();

		if( bSet )
		{
			m_kForce          = pkState->m_kForce;
			m_fMass           = pkState->m_fMass;
			m_fInvMass        = pkState->m_fInvMass;
		}

		m_vkStateStack.erase( pkState );
	}
}


void MassParticle::ClearState()
{
	PhysicsNode::ClearState();

	m_vkStateStack.clear();
}


};
