/***************************************************************************
         npacpackage.cpp  -  File management in virtual file system
                             -------------------
    begin                : Sat Jun 12 2004
    copyright            : (C) 2004 by emedia-solutions wolf
    email                : markus@emedia-solutions wolf
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, npacpackage.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.
 Portions created by Markus Wolf are Copyright (C) 2004
 emedia-solutions wolf. All Rights Reserved.

 ***************************************************************************/

#include "npacpackage.h"
#include "package.h"
#include "file.h"
#include "logstream.h"

using namespace std;


namespace NeoEngine
{


NpacPackage::NpacPackage( Directory *pkParent, const string &rstrFile, FileManager *pkManager ) :
	Package( pkParent, rstrFile, pkManager )
{
	m_bIsPackage = true;

	if( !Load( pkParent->GetByName( rstrFile, false ), 0, false ) )
	{
		m_bIsPackage = false;
		m_strName    = rstrFile;

		Parse( true );
	}
}


NpacPackage::~NpacPackage()
{
}

void NpacPackage::SkipHeader()
{
	// skip npac header
	char   szID[5] = { 0, 0, 0, 0, 0 };
	int iMajorVersion;
	int iMinorVersion;
	m_pkFile->SetBinary( true );
	m_pkFile->Read( &szID, 4 );
	*m_pkFile >> iMajorVersion >> iMinorVersion;
}

void NpacPackage::ReadCluster( Directory *pkDirectory )
{
	//Read cluster table
	string strName;
	int iSubClusters;
	int iFiles;
	int iFile;

	*m_pkFile >> strName >> iSubClusters >> iFiles;

	//Set name
	pkDirectory->m_strName     = strName;
	pkDirectory->m_strFullPath = "";

	for( int iCluster = 0; iCluster < iSubClusters; ++iCluster )
	{
		Directory *pkDir = new Directory( "", pkDirectory, false );

		ReadCluster( pkDir );
	}

	int iLastFileOffset = 0;
	int iLastFileSize   = 0;

	//Read file data
	for( iFile = 0; iFile < iFiles; ++iFile )
	{
		string strFileName;

		VirtualFileTemplate *pkFile = new VirtualFileTemplate;

		*m_pkFile >> strFileName >> pkFile->m_iOffset >> pkFile->m_iSize >> pkFile->m_iFlags;

		pkFile->m_pkPackage = this;

		if( pkFile->m_iFlags == FILEFLAG_BZIP2_COMPRESSED )
			*m_pkFile >> pkFile->m_aiData[0];

		pkDirectory->m_pkFiles->Insert( strFileName, pkFile );

		iLastFileSize   = pkFile->m_iSize;
		iLastFileOffset = pkFile->m_iOffset;
	}

	if( iFiles )
	{
		//Skip file data
		m_pkFile->Seekg( iLastFileOffset + iLastFileSize, ios_base::beg );
	}
}

}; // namespace NeoEngine
