/***************************************************************************
             package.cpp  -  File management in virtual file system
                             -------------------
    begin                : Wed Sep 18 2002
    copyright            : (C) 2002 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, package.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "package.h"
#include "file.h"
#include "logstream.h"

using namespace std;


namespace NeoEngine
{


Package::Package( Directory *pkParent, const string &rstrFile, FileManager *pkManager ) :
	Directory( "", pkParent, false ),
	LoadableEntity( pkManager )
{
}


Package::~Package()
{
}

bool Package::LoadNode( unsigned int uiFlags )
{
	//Cleanup
	vector< Directory* >::iterator ppkDirectory    = m_vpkDirectories.begin();
	vector< Directory* >::iterator ppkDirectoryEnd = m_vpkDirectories.end();

	for( ; ppkDirectory != ppkDirectoryEnd; ++ppkDirectory )
		delete *ppkDirectory;

	m_vpkDirectories.clear();

	{
		vector< FileTemplate* > vpkFiles;

		m_pkFiles->GetAllNodeData( &vpkFiles );

		vector< FileTemplate* >::iterator ppkNode = vpkFiles.begin();
		vector< FileTemplate* >::iterator ppkEnd  = vpkFiles.end();

		for( ; ppkNode != ppkEnd; ++ppkNode )
			delete (*ppkNode);

		delete m_pkFiles;

		m_pkFiles = new HashTable< FileTemplate >;
	}

	//Loop through all clusters
	SkipHeader();
	ReadCluster( this );

	//Keep file open
	m_bKeepFile  = true;
	m_bIsPackage = true;

	//PrintHierarchy( 0 );

	m_strName = m_pkFile->GetName();
	m_strName = m_strName.substr( 0, m_strName.find_last_of( '.' ) );

	m_strFullPath = "";

	m_bIsParsed = true;

	return true;
}

void Package::SkipHeader()
{
	// nothing to do
}

File *VirtualFileTemplate::AllocateFile( const std::string &rstrName, Directory *pkDirectory )
{
	VirtualFile *pkFile = new VirtualFile( pkDirectory ? pkDirectory->GetFullPath() : "", rstrName, pkDirectory );

	pkFile->m_iFileOffset     = m_iOffset;
	pkFile->m_iFileFlags      = m_iFlags;
	pkFile->m_iStoredFileSize = m_iSize;
	pkFile->m_pkPackage       = m_pkPackage;

	pkFile->m_iFileData[0]    = m_aiData[0];
	pkFile->m_iFileData[1]    = m_aiData[1];
	pkFile->m_iFileData[2]    = m_aiData[2];
	pkFile->m_iFileData[3]    = m_aiData[3];

	return pkFile;
}


}; // namespace NeoEngine
