/***************************************************************************
           perlinimage.cpp  -  Texture generation from perlin noise
                             -------------------
    begin                : Wed Oct 16 2002
    copyright            : (C) 2002 by Rob Wanders
    email                : forteq@softhome.net
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, perlinimage.cpp

 The Initial Developer of the Original Code is Rob Wanders.
 Portions created by Rob Wanders are Copyright (C) 2002
 Rob Wanders. All Rights Reserved.

 ***************************************************************************/

#include "logstream.h"
#include "perlinimage.h"

using namespace std;


namespace NeoEngine
{


PerlinImage::PerlinImage( int iWidth, int iHeight )	:
	ImageData()
{
	m_iWidth     = iWidth;
	m_iHeight    = iHeight;
	m_bGenerated = false;
	m_eFormat    = Texture::RGB;
}


void PerlinImage::GenerateImage( PERLINIMAGETYPE eType, int iData )
{
	m_iType      = eType;

	PerlinNoise kPerlin;
	
	switch( eType )
	{
		case CLOUDS:
		{
			float fNoisePixel;
			int iOffset = 0;

			m_pucData = new unsigned char[ m_iWidth * m_iHeight * 3 ];

			//FIXME: kPerlin.GenerateLookup( m_iWidth, m_iHeight );

			for(int iY = 0; iY < m_iHeight; ++iY )
			{
				float fY = float( iY );

				for( int iX = 0; iX < m_iWidth; ++iX )
				{
					float fX = float( iX );

					// Generate noise
					fNoisePixel = kPerlin.PerlinNoise2d( fX, fY, 1.0f, 0.125f, 0.125f ) +
					              kPerlin.PerlinNoise2d( fX, fY, 1.0f, 0.25f,  0.25f  ) * 0.5f +
					              kPerlin.PerlinNoise2d( fX, fY, 1.0f, 0.5f,   0.5f   ) * 0.25f +
					              kPerlin.PerlinNoise2d( fX, fY, 1.0f, 1.0f,   1.0f   ) * 0.125f;

					// Convert it to RGB (-1,1 => 0,255)
					fNoisePixel = 128.0f * fNoisePixel + 128.0f;

					// Create clouds ( 64 is the amount of clouds, 255 will give a empty sky and 0 total cloudcover
					float fC = ( fNoisePixel - float( iData ) ) / 255.0f;
					if( fC < 0.0f ) fC = 0.0f;
					if( fC > 1.0f ) fC = 1.0f;
					
					fNoisePixel = 255.0f + ( ( 255.0f - ( fC * 255 ) ) * -1.0f );

					unsigned char cData = (unsigned char)fNoisePixel;

					m_pucData[ iOffset++ ] = cData;
					m_pucData[ iOffset++ ] = cData;
					m_pucData[ iOffset++ ] = 255;
				}
			}

			m_bGenerated = true;

			neolog << LogLevel( INFO ) << "[PerlinImage::GenerateImage]: " << m_iWidth << "x" << m_iHeight << " clouds texture created" << endl;

			break;
		}
	
				
		case WOOD:
		{
			float fNoisePixel;
			int iOffset = 0;

			m_pucData = new unsigned char[ m_iWidth * m_iHeight * 3 ];

			//FIXME: kPerlin.GenerateLookup( m_iWidth, m_iHeight );

			for(int iY = 0; iY < m_iHeight; ++iY )
			{
				float fY = float( iY );

				for( int iX = 0; iX < m_iWidth; ++iX )
				{
					float fX = float( iX );

					// Generate noise
					fNoisePixel = kPerlin.PerlinNoise2d( fX, fY, 0.05f, 1.0f, 0.1f );
			
					// Convert it to RGB (-1,1 => 0,255)
					fNoisePixel = 128.0f * fNoisePixel + 128.0f;

					unsigned char cData = (unsigned char)fNoisePixel;

					m_pucData[ iOffset++ ] = cData;
					m_pucData[ iOffset++ ] = cData/2;
					m_pucData[ iOffset++ ] = 0;
				}
			}

			m_bGenerated = true;

			neolog << LogLevel( INFO ) << "[PerlinImage::GenerateImage]: " << m_iWidth << "x" << m_iHeight << " wood texture created" << endl;

			break;
		}

		default:
		{
			neolog << LogLevel( WARNING ) << "[PerlinImage::GenerateImage]: unknown type" << endl;

			return;
		}
	}
}


void PerlinImage::SetWidth( int iWidth )
{
	m_iWidth = iWidth;
}


void PerlinImage::SetHeight( int iHeight )
{
	m_iHeight = iHeight;
}

}; // namespace NeoEngine
