/***************************************************************************
                 physicsnode.cpp  -  Base class for physics nodes
                             -------------------
    begin                : Sat Jan 3 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, physicsnode.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "physicsnode.h"

using namespace std;


namespace NeoEngine
{


PhysicsNode::PhysicsNode() :
	SceneNode()
{
}


PhysicsNode::~PhysicsNode()
{
}


SceneNode *PhysicsNode::Duplicate()
{
	assert( !"[PhysicsNode::Duplicate] Not implemented!" );
	return 0;
}


void PhysicsNode::Update( float fDeltaTime )
{
	SceneNode::Update( fDeltaTime );

	TranslateWorld( m_kVelocity * fDeltaTime );
}


void PhysicsNode::PushState()
{
	m_vkStateStack.insert( m_vkStateStack.begin(), PhysicsNodeState( m_kVelocity, GetTranslation(), GetRotation() ) );
}


void PhysicsNode::PopState( bool bSet )
{
	if( m_vkStateStack.size() )
	{
		vector< PhysicsNodeState >::iterator pkState = m_vkStateStack.begin();

		if( bSet )
		{
			m_kVelocity = pkState->m_kVelocity;

			SetTranslation( pkState->m_kTranslation, false );
			SetRotation( pkState->m_kRotation );
		}

		m_vkStateStack.erase( pkState );
	}
}


void PhysicsNode::ClearState()
{
	m_vkStateStack.clear();
}


};
