/***************************************************************************
skeletonanimator.cpp  -  Animator controller classes for skeletons and bones
                             -------------------
    begin                : Tue Mar 4 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, skeletonanimator.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skeletonanimator.h"
#include "skeleton.h"

using namespace std;


namespace NeoEngine
{


SkeletonAnimation::SkeletonAnimation() :
	m_vpkChannels(),
	m_uiID( 0 ),
	m_strName(),
	m_fLength( 0.0f ),
	m_fCurTime( 0.0f )
{
}


SkeletonAnimation::SkeletonAnimation( const SkeletonAnimation &rkAnimation ) :
	m_vpkChannels(),
	m_uiID( rkAnimation.m_uiID ),
	m_strName( rkAnimation.m_strName ),
	m_fLength( rkAnimation.m_fLength ),
	m_fCurTime( rkAnimation.m_fCurTime )
{
	m_vpkChannels.resize( rkAnimation.m_vpkChannels.size() );

	vector< NodeAnimation* >::const_iterator ppkSrcChannel = rkAnimation.m_vpkChannels.begin();
	vector< NodeAnimation* >::const_iterator ppkEnd        = rkAnimation.m_vpkChannels.end();

	vector< NodeAnimation* >::iterator       ppkChannel    = m_vpkChannels.begin();

	for( ; ppkSrcChannel != ppkEnd; ++ppkChannel, ++ppkSrcChannel )
		*ppkChannel = new NodeAnimation( *(*ppkSrcChannel) );
}


SkeletonAnimation::~SkeletonAnimation()
{
	vector< NodeAnimation* >::iterator ppkChannel = m_vpkChannels.begin();
	vector< NodeAnimation* >::iterator ppkEnd     = m_vpkChannels.end();

	for( ; ppkChannel != ppkEnd; ++ppkChannel )
		delete( *ppkChannel );
}


void SkeletonAnimation::Update( float fDeltaTime )
{
	vector< NodeAnimation* >::iterator ppkChannel = m_vpkChannels.begin();
	vector< NodeAnimation* >::iterator ppkEnd     = m_vpkChannels.end();

	for( ; ppkChannel != ppkEnd; ++ppkChannel )
		(*ppkChannel)->Update( fDeltaTime );
}



void SkeletonAnimatorController::BlendStage( SkeletonAnimation *pkDstAnim, float fWeight, std::vector< bool > *pvbMask )
{
	for( unsigned int uiBone = 0; uiBone < m_pkSkeleton->m_uiNumBones; ++uiBone )
	{
		if( pvbMask && !(*pvbMask)[ uiBone ] )
			continue;

		NodeAnimation *pkAnim = pkDstAnim->m_vpkChannels[ uiBone ];

		//Interpolate
		NodeKeyframe *pkLastKeyframe = pkAnim->m_vpkKeyframes[ pkAnim->m_iLastKeyframe ];
		NodeKeyframe *pkNextKeyframe = pkAnim->m_vpkKeyframes[ pkAnim->m_iNextKeyframe ];

		Quaternion kQuat = pkLastKeyframe->m_kRotation;

		kQuat.Slerp( pkAnim->m_fOffset, pkNextKeyframe->m_kRotation, true );

		Quaternion kRotTarget = m_pkSkeleton->m_ppkBones[ uiBone ]->GetRotation();

		kRotTarget.Slerp( fWeight, kQuat, true );

		m_pkSkeleton->m_ppkBones[ uiBone ]->SetRotation( kRotTarget, false );


		Vector3d kTrans = pkLastKeyframe->m_kTranslation + ( ( pkNextKeyframe->m_kTranslation - pkLastKeyframe->m_kTranslation ) * pkAnim->m_fOffset );

		Vector3d kTransTarget = m_pkSkeleton->m_ppkBones[ uiBone ]->GetTranslation() + ( ( kTrans - m_pkSkeleton->m_ppkBones[ uiBone ]->GetTranslation() ) * fWeight );

		m_pkSkeleton->m_ppkBones[ uiBone ]->SetTranslation( kTransTarget, false );
	}
}


};
