/***************************************************************************
                      skybox.cpp  -  Simple skybox class
                             -------------------
    begin                : Mon Nov 19 2001
    copyright            : (C) 2001 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, skybox.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2001
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "skybox.h"
#include "scenenode.h"
#include "render.h"
#include "renderprimitive.h"
#include "vertex.h"
#include "polygon.h"

using namespace std;


namespace NeoEngine
{


SkyBox::SkyBox( float fSize, const Vector3d &rkOffset ) :
	SceneEntity()
{
	for( int i = 0; i < 6; ++i )
		m_apkVertexBuffer[i] = Core::Get()->GetRenderDevice()->CreateVertexBuffer( Buffer::STATIC, 4, &TexVertex::s_kDecl );

	//Front plate
	m_apkVertexBuffer[FRONT]->Lock( Buffer::WRITE );
	{
		TexVertex *pkVertex = (TexVertex*)m_apkVertexBuffer[FRONT]->GetVertex();

		pkVertex->m_kPosition     = Vector3d( -fSize, -fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize, -fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize,  fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 0.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize,  fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 0.0f;
	}
	m_apkVertexBuffer[FRONT]->Unlock();

	//Back plate
	m_apkVertexBuffer[BACK]->Lock( Buffer::WRITE );
	{
		TexVertex *pkVertex = (TexVertex*)m_apkVertexBuffer[BACK]->GetVertex();

		pkVertex->m_kPosition     = Vector3d(  fSize, -fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize, -fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize,  fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 0.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize,  fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 0.0f;
	}
	m_apkVertexBuffer[BACK]->Unlock();

	//Left plate
	m_apkVertexBuffer[LEFT]->Lock( Buffer::WRITE );
	{
		TexVertex *pkVertex = (TexVertex*)m_apkVertexBuffer[LEFT]->GetVertex();

		pkVertex->m_kPosition     = Vector3d( -fSize, -fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize, -fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize,  fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 0.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize,  fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 0.0f;
	}
	m_apkVertexBuffer[LEFT]->Unlock();

	//Right plate
	m_apkVertexBuffer[RIGHT]->Lock( Buffer::WRITE );
	{
		TexVertex *pkVertex = (TexVertex*)m_apkVertexBuffer[RIGHT]->GetVertex();

		pkVertex->m_kPosition     = Vector3d(  fSize, -fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize, -fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize,  fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 0.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize,  fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 0.0f;
	}
	m_apkVertexBuffer[RIGHT]->Unlock();

	//Top plate
	m_apkVertexBuffer[UP]->Lock( Buffer::WRITE );
	{
		TexVertex *pkVertex = (TexVertex*)m_apkVertexBuffer[UP]->GetVertex();

		pkVertex->m_kPosition     = Vector3d( -fSize,  fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize,  fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize,  fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 0.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize,  fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 0.0f;
	}
	m_apkVertexBuffer[UP]->Unlock();

	//Bottom plate
	m_apkVertexBuffer[DOWN]->Lock( Buffer::WRITE );
	{
		TexVertex *pkVertex = (TexVertex*)m_apkVertexBuffer[DOWN]->GetVertex();

		pkVertex->m_kPosition     = Vector3d( -fSize, -fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize, -fSize,  fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 1.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d(  fSize, -fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 1.0f;
		pkVertex->m_afTexCoord[1] = 0.0f; ++pkVertex;
		pkVertex->m_kPosition     = Vector3d( -fSize, -fSize, -fSize ) + rkOffset;
		pkVertex->m_afTexCoord[0] = 0.0f;
		pkVertex->m_afTexCoord[1] = 0.0f;
	}
	m_apkVertexBuffer[DOWN]->Unlock();


	//Polygon buffer
	m_pkPolygonBuffer = Core::Get()->GetRenderDevice()->CreatePolygonStripBuffer( Buffer::STATIC, 2 );

	m_pkPolygonBuffer->Lock( Buffer::WRITE );

	unsigned short *pusIndex = m_pkPolygonBuffer->GetIndices();

	*pusIndex++ = 0;
	*pusIndex++ = 1;
	*pusIndex++ = 3;
	*pusIndex   = 2;

	m_pkPolygonBuffer->Unlock();
}


SkyBox::~SkyBox()
{
}


void SkyBox::SetMaterial( SKYBOXPLANE ePlane, MaterialPtr &pkMaterial )
{
	m_apkMaterial[ ePlane ] = pkMaterial;

	if( m_apkMaterial[ ePlane ] )
		m_apkMaterial[ ePlane ]->m_kZBufferMode.Set( ZBufferMode::LEQUALNOWRITE );
}


bool SkyBox::Render( Frustum *pkFrustum, bool bForce )
{
	if( !SceneEntity::Render( pkFrustum, bForce ) )
		return false;

	RenderPrimitive kData;

	kData.m_ePrimitive           = RenderPrimitive::TRIANGLESTRIP;
	kData.m_uiNumPrimitives      = 2;

	for( int i = 0; i < 6; ++i )
	{
		if( !m_apkMaterial[i] )
			continue;

		kData.m_pkMaterial              = m_apkMaterial[i];
		kData.m_pkVertexBuffer          = m_apkVertexBuffer[i];
		kData.m_pkPolygonStripBuffer    = m_pkPolygonBuffer;

		if( GetSceneNode() )
			kData.m_kModelMatrix        = GetSceneNode()->GetWorldTransform();

		Core::Get()->GetRenderDevice()->Render( kData, RenderPrimitive::NOTRANSLATION );
	}

	return true;
}


}; // namespace NeoEngine
