/***************************************************************************
                       strutil.cpp  -  String utilities
                             -------------------
    begin                : Mon Oct 21 2002
    copyright            : (C) 2002 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, strutil.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "strutil.h"

using namespace std;


namespace NeoEngine
{


int Explode( const std::string &rstrSeparators, const std::string &rstrString, std::vector< std::string > *pvstrResult )
{
	if( !rstrString.length() || !pvstrResult )
		return 0;

	if( !rstrSeparators.length() )
	{
		pvstrResult->push_back( rstrString );
		return 1;
	}

	string::size_type iTokenStart = 0;
	string::size_type iTokenEnd   = 0;

	while( iTokenEnd < rstrString.length() )
	{
		iTokenStart = rstrString.find_first_not_of( rstrSeparators, iTokenEnd );
		iTokenEnd   = rstrString.find_first_of( rstrSeparators, iTokenStart );

		if( iTokenStart != string::npos )
			pvstrResult->push_back( rstrString.substr( iTokenStart, ( iTokenEnd != string::npos ) ? ( iTokenEnd - iTokenStart ) : string::npos ) );
	}

	return pvstrResult->size();
}


int Explode( const char *pszSeparators, char *pszString, std::vector<char*> *pvpszResult )
{
	if( !pszString || !pvpszResult )
		return 0;

	if( !pszSeparators )
	{
		pvpszResult->push_back( pszString );
		return 1;
	}

	int iPos        = 0;
	int iSepLoop    = 0;
	int iStrLen     = strlen( pszString );
	int iSepLen     = strlen( pszSeparators );

	while( iPos < iStrLen )
	{
		while( ( iSepLoop < iSepLen ) && ( iPos < iStrLen ) )
		{
			if( pszSeparators[iSepLoop++] == pszString[iPos] )
			{
				iPos++;
				iSepLoop = 0;
				continue;
			}
		}

		if( iPos < iStrLen )
		{
			char *pszToken = &pszString[iPos];

			while( iPos < iStrLen )
			{
				iSepLoop = 0;

				while( ( iSepLoop < iSepLen ) && ( iPos < iStrLen ) )
				{
					if( pszSeparators[iSepLoop++] == pszString[iPos] )
						break;
				}

				if( iSepLoop < iSepLen )
					break;

				++iPos;
			}

			pszString[iPos++] = 0;

			pvpszResult->push_back( pszToken );
		}
		
		iSepLoop = 0;
	}

	return pvpszResult->size();
}


string Strip( const std::string &rstrString, const std::string &rstrDelimiters )
{
	string::size_type uiStart = rstrString.find_first_not_of( rstrDelimiters );
	string::size_type uiEnd   = rstrString.find_last_not_of( rstrDelimiters ) + 1;

	if( uiStart != string::npos )
		return rstrString.substr( uiStart, ( uiEnd != string::npos ) ? ( uiEnd - uiStart ) : uiEnd );
	else
		return "";
}


};

