/***************************************************************************
                  submesh.cpp  -  Part of mesh, single material
                             -------------------
    begin                : Wed Sep 18 2002
    copyright            : (C) 2002 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, submesh.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "submesh.h"
#include "vertex.h"
#include "polygon.h"
#include "render.h"
#include "renderprimitive.h"
#include "shadow.h"


namespace NeoEngine
{


SubMesh::SubMesh() :
	m_bNeedUpdate( true ),
	m_bChanged( false )
{
}


SubMesh::SubMesh( const SubMesh &rkSubMesh ) :
	m_pkVertices(  rkSubMesh.m_pkVertices  ),
	m_pkPolygons(  rkSubMesh.m_pkPolygons  ),
	m_bNeedUpdate( rkSubMesh.m_bNeedUpdate ),
	m_pkShadowVertices( rkSubMesh.m_pkShadowVertices ),
	m_pkShadowPolygons( rkSubMesh.m_pkShadowPolygons ),
	m_pkMaterial(  rkSubMesh.m_pkMaterial  ),
	m_bChanged( true )
{
}


SubMesh::~SubMesh()
{
}



void SubMesh::Update( float fDeltaTime )
{
}


void SubMesh::UpdateData()
{
	m_bNeedUpdate = false;
}


SubMesh *SubMesh::Duplicate() const
{
	return( new SubMesh( *this ) );
}


void SubMesh::SetVertexBuffer( VertexBufferPtr pkVertices )
{
	m_pkVertices = pkVertices;
}


void SubMesh::SetPolygonBuffer( PolygonBufferPtr pkPolygons )
{
	m_pkPolygons = pkPolygons;
}


void SubMesh::SetShadowVertexBuffer( VertexBufferPtr pkVertices )
{
	m_pkShadowVertices = pkVertices;
}


void SubMesh::SetShadowPolygonBuffer( PolygonBufferPtr pkPolygons )
{
	m_pkShadowPolygons = pkPolygons;
}


void SubMesh::Render( RenderPrimitive *pkPrimitive, Frustum *pkFrustum )
{
	VertexBufferPtr  pkVertices = GetVertexBuffer();
	PolygonBufferPtr pkPolygons = GetPolygonBuffer();

	if( !pkVertices || !pkPolygons )
		return;

	if( !pkPolygons->IsStripped() )
	{
		pkPrimitive->m_ePrimitive           = RenderPrimitive::TRIANGLES;
		pkPrimitive->m_pkPolygonBuffer      = pkPolygons;
		pkPrimitive->m_uiNumPrimitives      = pkPolygons->GetNumElements();
	}
	else
	{
		pkPrimitive->m_ePrimitive           = RenderPrimitive::TRIANGLESTRIP;
		pkPrimitive->m_pkPolygonStripBuffer = pkPolygons->GetStrip();
		pkPrimitive->m_uiNumPrimitives      = pkPrimitive->m_pkPolygonStripBuffer->GetNumElements();
	}

	pkPrimitive->m_pkMaterial               = m_pkMaterial;
	pkPrimitive->m_pkVertexBuffer           = pkVertices;

	Core::Get()->GetRenderDevice()->Render( *pkPrimitive, 0 );

	if( m_pkMaterial && m_pkMaterial->m_bDynamicShadows && Core::Get()->GetRenderDevice()->m_pkShadowGenerator )
		Core::Get()->GetRenderDevice()->m_pkShadowGenerator->GenerateShadow( m_pkShadowVertices ? m_pkShadowVertices : pkVertices, m_pkShadowPolygons ? m_pkShadowPolygons : pkPolygons, (BoundingVolume*)0, pkPrimitive->m_kModelMatrix, pkPrimitive->m_kInvModelMatrix );
}


void SubMesh::CalculateTangents()
{
	VertexBufferPtr  pkVertices = GetVertexBuffer();
	PolygonBufferPtr pkPolygons = GetPolygonBuffer();

	if( !pkVertices || !pkPolygons )
		return;

	const VertexElement *pkNormalElement   = pkVertices->GetVertexFormat()->GetElement( VertexElement::FLOAT3,      VertexElement::NORMAL   );
	const VertexElement *pkTexCoordElement = pkVertices->GetVertexFormat()->GetElement( VertexElement::UNKNOWNTYPE, VertexElement::TEXCOORD );
	const VertexElement *pkTangentElement  = pkVertices->GetVertexFormat()->GetElement( VertexElement::UNKNOWNTYPE, VertexElement::TANGENT  );
	const VertexElement *pkBinormalElement = pkVertices->GetVertexFormat()->GetElement( VertexElement::UNKNOWNTYPE, VertexElement::BINORMAL );

	if( pkTangentElement )
		pkPolygons->CalculateTangents( pkVertices, pkNormalElement, pkTexCoordElement, pkTangentElement, pkBinormalElement );
}


};
