/***************************************************************************
     submeshanimator.cpp  -  Animator controller classes for submeshes
                             -------------------
    begin                : Wed Jan 29 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, submeshanimator.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "submeshanimator.h"
#include "vertex.h"
#include "color.h"
#include "nemath.h"
#include "render.h"


namespace NeoEngine
{


void SubMeshAnimatorController::InterpolateVertices( VertexBufferPtr pkBuffer )
{
	m_pkTarget = pkBuffer;

	BlendStages();

	m_pkTarget = 0;
}


void SubMeshAnimatorController::BlendStage( SubMeshAnimation *pkDstAnim, float fWeight, std::vector< bool > *pvbMask )
{
	if( fWeight < EPSILON )
		return;

	SubMeshKeyframe *pkLastKeyframe = pkDstAnim->m_vpkKeyframes[ pkDstAnim->m_iLastKeyframe ];
	SubMeshKeyframe *pkNextKeyframe = pkDstAnim->m_vpkKeyframes[ pkDstAnim->m_iNextKeyframe ];


	VertexBufferPtr pkLastSrc = pkLastKeyframe->m_pkBuffer;
	VertexBufferPtr pkNextSrc = pkNextKeyframe->m_pkBuffer;

	assert( pkDstAnim->m_fOffset >= 0.0f );
	assert( pkDstAnim->m_fOffset < 1.0f );
	assert( *pkLastSrc->GetVertexFormat() == *pkNextSrc->GetVertexFormat() );
	assert( pkLastSrc->GetNumElements()  == pkNextSrc->GetNumElements()  );

	const VertexDeclaration *pkVertexFormat = pkLastSrc->GetVertexFormat();
	unsigned int             uiNumVertices  = pkLastSrc->GetNumElements();

	float fTimeDiff   = pkDstAnim->m_fOffset;

	if( m_pkTarget->GetNumElements() != uiNumVertices )
		m_pkTarget->AllocateVertices( uiNumVertices, pkVertexFormat );

	m_pkTarget->Lock( Buffer::WRITE );
	pkLastSrc->Lock( Buffer::READ );

	if( pkLastSrc != pkNextSrc )
		pkNextSrc->Lock( Buffer::READ );

	if( fWeight > ( 1.0f - EPSILON ) )
		pkVertexFormat->InterpolateArray( (unsigned char*)pkLastSrc->GetVertex(), (unsigned char*)pkNextSrc->GetVertex(), (unsigned char*)m_pkTarget->GetVertex(), fTimeDiff, uiNumVertices, pvbMask );
	else //if( fWeight > EPSILON ) - already checked at start of method
	{
		//Interpolate in temporary buffer and blend
		if( !m_pkInterpolated )
			m_pkInterpolated = new VertexBuffer( Buffer::NORENDER, uiNumVertices, pkVertexFormat );

		if( m_pkInterpolated->GetNumElements() != uiNumVertices )
			m_pkInterpolated->AllocateVertices( uiNumVertices, pkVertexFormat );

		m_pkInterpolated->Lock( Buffer::WRITE );

		pkVertexFormat->InterpolateArray( (unsigned char*)pkLastSrc->GetVertex(), (unsigned char*)pkNextSrc->GetVertex(), (unsigned char*)m_pkInterpolated->GetVertex(), fTimeDiff, uiNumVertices, pvbMask );
		pkVertexFormat->InterpolateArray( (unsigned char*)m_pkTarget->GetVertex(), (unsigned char*)m_pkInterpolated->GetVertex(), (unsigned char*)m_pkTarget->GetVertex(), fWeight, uiNumVertices, pvbMask );

		m_pkInterpolated->Unlock();
	}

	if( pkLastSrc != pkNextSrc )
		pkNextSrc->Unlock();

	pkLastSrc->Unlock();
	m_pkTarget->Unlock();
}


};
