/***************************************************************************
                          taskmanager.cpp  -  description
                             -------------------
    begin                : Tue Feb 12 2002
    copyright            : (C) 2002 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, taskmanager.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "taskmanager.h"

using namespace std;


namespace NeoEngine
{


TaskManager::TaskManager( int iWorkerThreads ) :
	m_iMaxThreads( iWorkerThreads )
{
}


TaskManager::~TaskManager()
{
	while( m_vpkThreads.size() )
		delete m_vpkThreads[0];
}


void TaskManager::Remove( Thread *pkThread )
{
	MutexLock kLock( this );

	for( vector<Thread*>::iterator ppkThread = m_vpkThreads.begin(); ppkThread != m_vpkThreads.end(); ++ppkThread )
		if( *ppkThread == pkThread )
		{
			m_vpkThreads.erase( ppkThread );
			break;
		}
	
	while( m_vpkQueue.size() && ( m_vpkThreads.size() < m_iMaxThreads ) )
	{
		//Start thread execution
		m_vpkThreads.push_back( new Thread( m_vpkQueue[0], 0, 0, this ) );
		m_vpkQueue.erase( m_vpkQueue.begin() );
	}
}


void TaskManager::Schedule( ThreadMethod *pkTask )
{
	MutexLock kLock( this );
	
	if( m_vpkThreads.size() < m_iMaxThreads )
	{
		//Start thread execution immediately
		m_vpkThreads.push_back( new Thread( pkTask, 0, 0, this ) );
	}
	else
	{
		//Queue for execution later
		m_vpkQueue.push_back( pkTask );
	}
}


}; // namespace NeoEngine
