/***************************************************************************
                          util.cpp  -  Utility methods
                             -------------------
    begin                : Tue May 14 2002
    copyright            : (C) 2002 by David Holm
    email                : david@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, util.cpp

 The Initial Developer of the Original Code is David Holm.
 Portions created by David Holm are Copyright (C) 2002
 David Holm. All Rights Reserved.

 ***************************************************************************/

#include "util.h"

#include <string.h>


void* (*fmemcpy)( void *dest, const void *src, size_t n ) = memcpy;

#ifdef INTEL_INTRINSICS

#include <mmintrin.h>
#include <xmmintrin.h>

#if defined(__GNUC__) && defined(ARCH_X86)
#define small_memcpy(dst,src,n) \
	register unsigned long int dummy; \
	asm volatile ( \
		"rep; movsb\n\t" \
		:"=&D"(dst), "=&S"(src), "=&c"(dummy) \
        :"0" (dst), "1" (src),"2" (n) \
		: "memory")
#else
#define small_memcpy(dst,src,n) memcpy(dst,src,n)
#endif /* __GNUC__ ARCH_X86 */


#  define SSE_MMREG_SIZE 16
#  define MMX_MMREG_SIZE 8

#  define MMX1_MIN_LEN 0x800  /* 2K blocks */
#  define MIN_LEN 0x40  /* 64-byte blocks */

void *memcpy_sse( void *dst, const void *src, size_t len )
{
	void *retval = dst;
	size_t i;

	_mm_prefetch((char*) src, _MM_HINT_NTA);
	_mm_prefetch((char*) src + 64, _MM_HINT_NTA);
	_mm_prefetch((char*) src + 128, _MM_HINT_NTA);
	_mm_prefetch((char*) src + 192, _MM_HINT_NTA);
	_mm_prefetch((char*) src + 256, _MM_HINT_NTA);

	if (len >= MIN_LEN)
	{
		register unsigned long int delta;
		/* Align destinition to MMREG_SIZE -boundary */
		delta = ((unsigned long int) dst) & (SSE_MMREG_SIZE - 1);
		if (delta)
		{
			delta = SSE_MMREG_SIZE - delta;
			len -= delta;
			small_memcpy(dst, src, delta);
		}
		i = len >> 6;
		len &= 63;

		if (((unsigned long) src) & 15)
			/* if SRC is misaligned */
			for (; i > 0; i--)
			{
				__m128 xmm0, xmm1, xmm2, xmm3;
				_mm_prefetch((char*) src, _MM_HINT_NTA);
				xmm0 = _mm_loadu_ps((float*) src);
				xmm1 = _mm_loadu_ps((float*) src + 4);
				xmm2 = _mm_loadu_ps((float*) src + 8);
				xmm3 = _mm_loadu_ps((float*) src + 12);
				_mm_stream_ps((float*) dst, xmm0);
				_mm_stream_ps((float*) dst + 4, xmm1);
				_mm_stream_ps((float*) dst + 8, xmm2);
				_mm_stream_ps((float*) dst + 12, xmm3);
#ifdef __INTEL_COMPILER
				src += 64;
				dst += 64;
#else
				(char*) src += 64;
				(char*) dst += 64;
#endif
			}
		else
			/**
			 * Only if SRC is aligned on 16-byte boundary.
			 * It allows to use movaps instead of movups, which required data
			 * to be aligned or a general-protection exception (#GP) is generated.
			 */
			for (; i > 0; i--)
			{
				__m128 xmm0, xmm1, xmm2, xmm3;
				_mm_prefetch((char*) src, _MM_HINT_NTA);
				xmm0 = _mm_load_ps((float*) src);
				xmm1 = _mm_load_ps((float*) src + 4);
				xmm2 = _mm_load_ps((float*) src + 8);
				xmm3 = _mm_load_ps((float*) src + 12);
				_mm_stream_ps((float*) dst, xmm0);
				_mm_stream_ps((float*) dst + 4, xmm1);
				_mm_stream_ps((float*) dst + 8, xmm2);
				_mm_stream_ps((float*) dst + 12, xmm3);
#ifdef __INTEL_COMPILER
				src += 64;
				dst += 64;
#else
				(char*) src += 64;
				(char*) dst += 64;
#endif
			}
	}

	if (len)
		memcpy(dst, src, len);

	return retval;
}

#endif /* INTEL_INTRINSICS */

