/***************************************************************************
            vertexbuffer.cpp  -  Vertex buffer with flexible format
                             -------------------
    begin                : Tue Oct 30 2001
    copyright            : (C) 2001 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, vertexbuffer.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2001
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "vertexbuffer.h"
#include "util.h"

using namespace std;


namespace NeoEngine
{



VertexBuffer::VertexBuffer( unsigned int uiType, unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData ) :
	Buffer( uiType ),
	m_pkVertexFormat( 0 ),
	m_uiVertexFormatSize( 0 ),
	m_pucBuffer( 0 ),
	m_pucVertices( 0 )
{
	AllocateVertices( uiNumVertices, pkFormat, pData );
}


VertexBuffer::~VertexBuffer()
{
	delete m_pkVertexFormat;
	delete [] m_pucBuffer;
}


void VertexBuffer::AllocateVertices( unsigned int uiNumVertices, const VertexDeclaration *pkFormat, const void *pData )
{
	assert( !m_uiLock );

	if( uiNumVertices )
	{
		//If we reach this point, we should not care about NOREADWRITE type, but allow Lock:s below to copy data
		m_uiType &= ~Buffer::NOREADWRITE;
	}

	if( ( uiNumVertices == m_uiNumAllocated ) && pkFormat && m_pkVertexFormat && ( *pkFormat == *m_pkVertexFormat ) )
	{
		m_uiNumCurrent = uiNumVertices;

		if( !pData || !uiNumVertices )
			return;

		Lock( Buffer::WRITE | Buffer::NOUPLOAD );

		LoadVertexData( pData );

		Unlock();

		return;
	}

	delete [] m_pucBuffer;

	if( pkFormat )
	{
		delete m_pkVertexFormat;
		m_pkVertexFormat  = pkFormat->Duplicate();
	}

	m_pucBuffer           = 0;
	m_uiVertexFormatSize  = m_pkVertexFormat ? m_pkVertexFormat->GetVertexSize() : 0;
	m_uiNumAllocated      = m_pkVertexFormat ? uiNumVertices : 0;
	m_uiNumCurrent        = m_uiNumAllocated;

	if( !m_uiNumAllocated )
		return;

	m_pucBuffer = new unsigned char[ m_uiVertexFormatSize * m_uiNumAllocated ];

	if( pData )
	{
		Lock( Buffer::WRITE | Buffer::NOUPLOAD );

		LoadVertexData( pData );

		Unlock();
	}
}


};
