/***************************************************************************
          zippackage.cpp  -  File management in virtual file system
                             -------------------
    begin                : Thu Jun 03 2004
    copyright            : (C) 2002 by emedia-solutions wolf
    email                : markus@emedia-solutions-wolf.de
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, zippackage.cpp

 The Initial Developer of the Original Code is Markus Wolf.
 Portions created by Markus Wolf are Copyright (C) 2004
 emedia-soltions wolf. All Rights Reserved.

 ***************************************************************************/

#include "zippackage.h"
#include "file.h"
#include "logstream.h"

using namespace std;


namespace NeoEngine
{


ZipPackage::ZipPackage( Directory *pkParent, const string &rstrFile, FileManager *pkManager ) :
	Package( pkParent, rstrFile, pkManager )
{
	m_bIsPackage = true;

	if( !Load( pkParent->GetByName( rstrFile, false ), 0, false ) )
	{
		m_bIsPackage = false;
		m_strName    = rstrFile;

		Parse( true );
	}
}


ZipPackage::~ZipPackage()
{
}

void ZipPackage::ReadCluster( Directory *pkDirectory )
{
	char *iCursor;

	// read entire file into buffer
	int iFileSize = m_pkFile->GetSize();
	char *szFileBuffer = new char[ iFileSize ];
	m_pkFile->Seekg( 0, ios_base::beg );
	m_pkFile->Read( szFileBuffer, iFileSize );

	// search for central directory header
	iCursor = szFileBuffer;
	do
	{
		ZipCDFHeader *pCDFHeader = ( ZipCDFHeader* )iCursor;

		if ( pCDFHeader->iSignature == 0x02014b50 )
		{
			string strFileName;

			// found central directory header
			iCursor += sizeof( ZipCDFHeader );
			char *pzFileName = new char[ pCDFHeader->iFileNameLength + 1 ];
			memcpy( pzFileName, iCursor, pCDFHeader->iFileNameLength );
			pzFileName[ pCDFHeader->iFileNameLength ] = '\0';
			strFileName = string( pzFileName );
			delete pzFileName;
			iCursor += pCDFHeader->iFileNameLength + pCDFHeader->iExtraFieldLength + pCDFHeader->iFileCommentLength;

			// parse path
			vector< string > vkPath;
			string strPath = strFileName;
			while( ( strPath = File::ExtractPathName( strPath ) ) != "" )
			{
				vkPath.push_back( strPath );
			}

			// build directory tree
			Directory *pkWorkingDirectory = pkDirectory;
			Directory *pkCurrentDirectory = 0;
			while ( vkPath.size() > 0 )
			{
				strPath = vkPath.back();
				vkPath.pop_back();
				pkCurrentDirectory = pkWorkingDirectory->GetDirectoryByName( strPath );
				if ( pkCurrentDirectory == 0 )
				{
					pkCurrentDirectory = new Directory( strPath, pkWorkingDirectory );
					//pkWorkingDirectory->m_vpkDirectories.push_back( pkCurrentDirectory );
				}
				pkWorkingDirectory = pkCurrentDirectory;
			}

			// only scan real files
			strFileName = File::ExtractFileName( strFileName );
			if ( strFileName != "" )
			{
				// save file as virtual file
				VirtualFileTemplate *pkFile = new VirtualFileTemplate;

				ZipLFHeader *pZipLFHeader = ( ZipLFHeader* )( szFileBuffer + pCDFHeader->iDataOffset );
				pkFile->m_iOffset = pCDFHeader->iDataOffset 
					+ sizeof ( ZipLFHeader ) 
					+ pZipLFHeader->iFileNameLength 
					+ pZipLFHeader->iExtraFieldLength;

				pkFile->m_iSize = pCDFHeader->iCompressedSize;
				pkFile->m_iFlags = FILEFLAG_ZIP_COMPRESSED;
				pkFile->m_pkPackage = this;
				pkFile->m_aiData[0] = pCDFHeader->iUncompressedSize;
				pkWorkingDirectory->m_pkFiles->Insert( strFileName, pkFile );
			}
		}
		else {
			iCursor++;
		}

	}
	while ( iCursor < szFileBuffer + iFileSize - sizeof( ZipCDFHeader ) );


	delete szFileBuffer;
}

}; // namespace NeoEngine
