/***************************************************************************
                 base.h  -  Base defines for mip terrain
                             -------------------
    begin                : Thu Mar 18 2004
    copyright            : (C) 2004 by Cody Russell
    email                : cody `at' jhu.edu
 ***************************************************************************
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoGeoMip, base.h

 The Initial Developer of the Original Code is Cody Russell.
 Portions created by Cody Russell are Copyright (C) 2004
 Cody Russell. All Rights Reserved.
 
 ***************************************************************************/
 
#ifndef __NEOGEOMIP_BASE_H
#define __NEOGEOMIP_BASE_H
 
#if defined(HAVE_CONFIG_H) && defined(NEOENGINE_INTERNALS)
#  include "buildconfig.h"
#endif

#include <neoengine/base.h>
#include <neoengine/terrain.h>

#define NEOGEOMIPVERSION_MAJOR             NEOENGINEVERSION_MAJOR
#define NEOGEOMIPVERSION_MINOR             NEOENGINEVERSION_MINOR
#define NEOGEOMIPVERSION_REVISION          NEOENGINEVERSION_REVISION

#define NEOGEOMIPVERSIONSTRING             NEOENGINEVERSIONSTRING

                                                                                
#ifdef WIN32
                                                                                
#  ifdef _MSC_VER
#    pragma warning( disable : 4231 )
#  endif

#  ifndef BUILD_STATIC

#    ifdef NEOGEOMIP_EXPORTS
                                                                                
#      define NEOGEOMIP_API __declspec(dllexport)
#      define NEOGEOMIPEXPIMP_TEMPLATE
                                                                                
#    else
                                                                                
#      define NEOGEOMIP_API __declspec(dllimport)
#      define NEOGEOMIPEXPIMP_TEMPLATE extern
                                                                                
#    endif
                                                                                
#    ifdef __cplusplus
        // Instantiate class std::vector<classname> and std::list<classname>
        // This does not create an object. It only forces the generation of all
        // of the members of the class.
        // It exports them from the DLL and imports them into the .exe file.
#      if ( _MSC_VER >= 1300 )
#        define GEOMIPUDTVectorEXPIMP(classname) NEOGEOMIPEXPIMP_TEMPLATE template class NEOGEOMIP_API std::allocator< classname >; NEOGEOMIPEXPIMP_TEMPLATE template class NEOGEOMIP_API std::vector< classname >
#      elif !defined( __MINGW32__ )
#        define GEOMIPUDTVectorEXPIMP(classname) NEOGEOMIPEXPIMP_TEMPLATE template class NEOGEOMIP_API std::vector< classname >
#      else
#        define GEOMIPUDTVectorEXPIMP(classname)
#      endif
#    endif
#  else

#    define NEOGEOMIP_API
#    define NEOGEOMIPEXPIMP_TEMPLATE
#    define GEOMIPUDTVectorEXPIMP(classname)

#  endif

#elif defined(POSIX) || defined(__APPLE__)
                                                                                
#  define NEOGEOMIP_API
#  define NEOGEOMIP_TEMPLATE
                                                                                
#  define GEOMIPUDTVectorEXPIMP(classname)
                                                                                
#else
#  error "Platform uninplemented"
#endif

namespace NeoGeoMip
{
#ifndef BUILD_STATIC

#ifdef __cplusplus
extern "C"
{
#endif

	/**
	* Called by the engine when the plugin is loaded
	* \return                                                  Error code
	*/
	NEOGEOMIP_API int                                          Initialize();

	/**
	* Create terrain object
	* \return                                                  New terrain manager
	*/
	NEOGEOMIP_API class NeoEngine::TerrainManager             *CreateTerrainManager();

	/**
	* Called by the engine when plugin is unloaded
	* \return                                                  Error code
	*/
	NEOGEOMIP_API int                                          Shutdown();

#ifdef __cplusplus
}
#endif

#endif // BUILD_STATIC

};

#endif // __NEOGEOMIP_BASE_H

