/***************************************************************************
         dll.cpp  -  Dynamic library entry points and init/shutdown
                             -------------------
    begin                : Thu Mar 18 2004
    copyright            : (C) 2004 by Cody Russell
    email                : cody `at' jhu.edu
 ***************************************************************************
 
 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoGeoMip, dll.cpp

 The Initial Developer of the Original Code is Cody Russell.
 Portions created by Cody Russell are Copyright (C) 2004
 Cody Russell. All Rights Reserved.
 
 ***************************************************************************/

#include "base.h"
#include "link.h"
#include "terrain.h"

using namespace NeoEngine;
using namespace NeoGeoMip;

#if defined( WIN32 ) && !defined( BUILD_STATIC )
 
#ifndef WIN32_LEAN_AND_MEAN
#  define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
 
// Win32-specific init for dll loading
BOOL NEOGEOMIP_API APIENTRY DllMain( HANDLE hModule, DWORD ulReason, LPVOID lpReserved )
{
        switch( ulReason )
        {
                case DLL_PROCESS_ATTACH:
                case DLL_THREAD_ATTACH:
                        break;
                                                                                
                case DLL_THREAD_DETACH:
                case DLL_PROCESS_DETACH:
                        break;
        }
                                                                                
        return TRUE;
}
                                                                                
                                                                                
#endif

namespace NeoGeoMip
{

#ifdef BUILD_STATIC

ModuleStatic *g_pkStaticModule = 0;

#else

extern "C"
{

#endif

NEOGEOMIP_API int Initialize()
{
        return 0;
}
                                                                                
                                                                                
NEOGEOMIP_API NeoEngine::TerrainManager *CreateTerrainManager()
{
        return new NeoGeoMip::MipTerrainManager;
}

NEOGEOMIP_API void GetVersion( std::string *pstrName, int *piMajor, int *piMinor, int *piRevision )
{
        *pstrName   = "GeoMipMap Terrain Method";
        *piMajor    = NEOGEOMIPVERSION_MAJOR;
        *piMinor    = NEOGEOMIPVERSION_MINOR;
        *piRevision = NEOGEOMIPVERSION_REVISION;
}

                                                                                
NEOGEOMIP_API int Shutdown()
{
#ifdef BUILD_STATIC
	delete g_pkStaticModule;
#endif

        return 0;
}

#ifdef BUILD_STATIC

#else

}

#endif

#ifdef BUILD_STATIC

Link::Link()
{
#if defined( WIN32 ) && defined( _DEBUG )
	g_pkStaticModule = new ModuleStatic( "neogeomip_debug" );
#else
	g_pkStaticModule = new ModuleStatic( "neogeomip" );
#endif

	g_pkStaticModule->m_Symbols.Insert( "CreateTerrainManager", (void**)CreateTerrainManager );
	g_pkStaticModule->m_Symbols.Insert( "Initialize",           (void**)Initialize );
	g_pkStaticModule->m_Symbols.Insert( "Shutdown",             (void**)Shutdown );
	g_pkStaticModule->m_Symbols.Insert( "GetVersion",           (void**)GetVersion );

	NeoEngine::Core::GetModuleManager()->InsertModule( g_pkStaticModule );
}

#endif

};
