/***************************************************************************
                          neoicbmp.cpp  -  description
                             -------------------
    begin                : Wed Aug 17 2002
    copyright            : (C) 2002 by Matthew Holmes
    email                : kerion@vengeanceagenda.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoICBMP, neoicbmp.cpp

 The Initial Developer of the Original Code is Matthew Holmes.
 Portions created by Matthew Holmes are Copyright (C) 2003
 Matthew Holmes. All Rights Reserved.

 Contributors: Mattias Jansson (mattias@realityrift.com)

 ***************************************************************************/

#include "neoicbmp.h"
#include "link.h"

#include <neoengine/file.h>
#include <neoengine/util.h>
#include <neoengine/logstream.h>

#include <string.h>

#include <vector>
#include <string>

using namespace std;
using namespace NeoEngine;


namespace NeoBMP
{

#ifdef BUILD_STATIC
#  define BMPEXPORT

ModuleStatic *g_pkStaticModule = 0;

#else

extern "C"
{

#  ifdef WIN32
#    define BMPEXPORT __declspec(dllexport)
#  else
#    define BMPEXPORT
#endif

#endif

	BMPEXPORT ImageCodec *LoadImageCodec()
	{
		vector<string> vstrExtensions;

		vstrExtensions.push_back( "bmp" );
		vstrExtensions.push_back( "BMP" );

		ImageCodec *pkCodec = new NeoBMP::ImageCodec( vstrExtensions );

		return pkCodec;
	}

	BMPEXPORT int Initialize()
	{
		return 0;
	}

	BMPEXPORT int Shutdown()
	{
#ifdef BUILD_STATIC
		delete g_pkStaticModule;
#endif
		return 0;
	}

	BMPEXPORT void GetVersion( std::string *pstrName, int *piMajor, int *piMinor, int *piRevision )
	{
		*pstrName   = "NeoICBMP";
		*piMajor    = NEOENGINEVERSION_MAJOR;
		*piMinor    = NEOENGINEVERSION_MINOR;
		*piRevision = NEOENGINEVERSION_REVISION;
	}

#ifdef BUILD_STATIC

#else

}

#endif


File &operator >> ( File &rkFile, Header &rkHeader )
{
	return rkFile >> (int8_t&)rkHeader.m_acMagic[0]
		      >> (int8_t&)rkHeader.m_acMagic[1]
		      >> rkHeader.m_uiSize
		      >> rkHeader.m_usReserved1
		      >> rkHeader.m_usReserved2
		      >> rkHeader.m_uiOffBits;
}


File &operator >> ( File &rkFile, InfoUpper &rkInfo )
{
	return rkFile >> rkInfo.m_usPlanes >> rkInfo.m_usBitCount;
}



ImageCodec::ImageCodec( const vector<string> &rvstrExtensions ) :
	NeoEngine::ImageCodec( "Windows BMP Image File", rvstrExtensions )
{
	neolog << LogLevel( INFO ) << "Loading BMP image codec" << endl << "  Based on code from the SDL library, Copyright (C) 1997-2002 Sam Lantinga" << endl;
}


bool ImageCodec::IsType( File *pkFile )
{
	bool bBinary = pkFile->SetBinary( true );
	int  iOffset = pkFile->Tellg();
	
	if( iOffset != 0 )
		pkFile->Seekg( 0, ios_base::beg );
		
	Header       kHeader;
	InfoUpper    kInfo;
	unsigned int uiInfoSize;
	int          iWidth;
	int          iHeight;
	
	*pkFile >> kHeader >> uiInfoSize;
	
	if( uiInfoSize == 12 )
	{
		unsigned short usTemp0, usTemp1;
		
		*pkFile >> usTemp0 >> usTemp1 >> kInfo;

		iWidth  = (int)usTemp0;
		iHeight = (int)usTemp1;
	}
	else
		*pkFile >> iWidth >> iHeight >> kInfo;

	pkFile->Seekg( iOffset, ios_base::beg );
	pkFile->SetBinary( bBinary );

	if( strncmp( kHeader.m_acMagic, "BM", 2 ) )
		return false;

	if( !iWidth || !iHeight )
		return false;

	if( ( kInfo.m_usBitCount != 16 ) && ( kInfo.m_usBitCount != 24 ) && ( kInfo.m_usBitCount != 32 ) )
		return false;

	return true;
}


ImageData *ImageCodec::LoadImage( File *pkFile )
{
	int i;

	bool bBinary = pkFile->SetBinary( true );
	int  iOffset = pkFile->Tellg();
	
	if( iOffset != 0 )
		pkFile->Seekg( 0, ios_base::beg );
		
	Header kHeader;
	
	*pkFile >> kHeader;

	if( strncmp(kHeader.m_acMagic, "BM", 2 ) )
	{
		pkFile->Seekg( iOffset, ios_base::beg );
		pkFile->SetBinary( bBinary );
		neolog << LogLevel( ERROR ) << "*** Unable to load BMP image: Unknown BMP magic [" << kHeader.m_acMagic << "]" << endl;
		return 0;
	}

	InfoUpper kInfoUpper;
	InfoLower kInfoLower;
	unsigned int uiInfoSize;
	int iWidth, iHeight;

	*pkFile >> uiInfoSize;

	if( uiInfoSize == 12 )
	{
		// If the info size is 12, then the width/height are 2 byte entries instead of 4
		unsigned short usTemp0, usTemp1;
		
		*pkFile >> usTemp0 >> usTemp1 >> kInfoUpper;

		iWidth  = (int)usTemp0;
		iHeight = (int)usTemp1;

		kInfoLower.m_uiCompression = BI_RGB;
		kInfoLower.m_uiSizeImage = 0;
		kInfoLower.m_iXPixelsPerMeter = 0;
		kInfoLower.m_iYPixelsPerMeter = 0;
		kInfoLower.m_uiClrUsed = 0;
		kInfoLower.m_uiClrImportant = 0;
	}
	else
	{
		*pkFile >> iWidth >> iHeight >> kInfoUpper;
		pkFile->Read(&kInfoLower, sizeof(InfoLower));
	}

	ImageData *pkImage = new ImageData;

	pkImage->m_eFormat   = Texture::RGB;
	pkImage->m_iWidth    = iWidth;
	pkImage->m_iHeight   = iHeight;
	
	int iBPP             = kInfoUpper.m_usBitCount;
	int iBytesPP         = ( iBPP / 8 );
	int iImageSize       = iWidth * iHeight * iBytesPP;

	pkImage->m_iBPP      = iBPP;
	pkImage->m_iChannels = 3;
	pkImage->m_pucData   = new unsigned char[iImageSize];

	// Setup the color masks (used later to flip BGR -> RGB)
	unsigned int uiRMask = 0, uiGMask = 0, uiBMask = 0;
	switch(kInfoLower.m_uiCompression)
	{
	case BI_RGB:
		if(kHeader.m_uiOffBits == (14 + uiInfoSize))
		{
			switch(iBPP)
			{
			case 15:
			case 16:
				uiRMask = 0x7C00;
				uiGMask = 0x03E0;
				uiBMask = 0x001F;
				break;
			case 24:
/*#if SDL_BYTEORDER == SDL_BIG_ENDIAN
					Rmask = 0x000000FF;
					Gmask = 0x0000FF00;
					Bmask = 0x00FF0000;
				break;
#endif*/
			case 32:
				uiRMask = 0x00FF0000;
				uiGMask = 0x0000FF00;
				uiBMask = 0x000000FF;
				break;
			default:
				break;
			}
			break;
		}
		
	case BI_BITFIELDS:
		switch(iBPP)
		{
		case 15:
		case 16:
		case 32:
			pkFile->Read(&uiRMask, sizeof(unsigned int));
			pkFile->Read(&uiGMask, sizeof(unsigned int));
			pkFile->Read(&uiBMask, sizeof(unsigned int));
			break;
		default:
			break;
		}		
		break;
	
	default:
		neolog << LogLevel( ERROR ) << "*** Unable to load BMP image: Compressed BMP data not supported" << endl;
		FreeImage(pkImage);
		return false;
	}

	//FIXME: Support paletted bitmaps (<16 BPP)

	// Read the bitmap data (in BGR, must flip colors later in load)
	unsigned short usPitch = iWidth * iBytesPP;
	usPitch = (usPitch + 3) & ~3;
	int iPad = (usPitch % 4 ? 4 - (usPitch % 4) : 0);

	unsigned char *pucBits = pkImage->m_pucData + (iHeight * usPitch);
	while (pucBits > pkImage->m_pucData )
	{
		pucBits -= usPitch;
		if( !pkFile->Read(pucBits, usPitch) )
		{
			neolog << LogLevel( ERROR ) << "*** Unable to load BMP image: Failed to read pixel data" << endl;
			FreeImage(pkImage);
			return false;
		}

		if(iPad)
		{
			unsigned char ucPadByte;
			for (i = 0; i < iPad; ++i)
				pkFile->Read(&ucPadByte, 1);
		}
	}

	// Calculate the RGB loss factors and RGB shifts
	unsigned char ucRLoss = 8, ucGLoss = 8, ucBLoss = 8;
	unsigned char ucRShift = 0, ucGShift = 0, ucBShift = 0;
	unsigned int uiMask = 0;

	if(!uiRMask && !uiGMask && !uiBMask)
	{
		int iTempBPP = iBPP;
		if (iTempBPP > 24)
			iTempBPP = 24;

		ucRLoss = 8 - (iTempBPP / 3);
		ucGLoss = 8 - (iTempBPP / 3) - (iTempBPP % 3);
		ucBLoss = 8 - (iTempBPP / 3);

		ucRShift = ((iTempBPP / 3) + (iTempBPP % 3)) + (iTempBPP / 3);
		ucGShift = (iTempBPP / 3);
		ucBShift = 0;

		uiRMask = ((0xFF >> ucRLoss) << ucRShift);
		uiGMask = ((0xFF >> ucGLoss) << ucGShift);
		uiBMask = ((0xFF >> ucBLoss) << ucBShift);
	}
	else
	{
		if(uiRMask)
		{
			for(uiMask = uiRMask; !(uiMask & 0x01); uiMask >>= 1)
				ucRShift++;
			for(; (uiMask & 0x01); uiMask >>= 1)
				ucRLoss--;
		}
		if(uiGMask)
		{
			for(uiMask = uiGMask; !(uiMask & 0x01); uiMask >>= 1)
				ucGShift++;
			for(; (uiMask & 0x01); uiMask >>= 1)
				ucGLoss--;
		}
		if(uiBMask)
		{
			for(uiMask = uiBMask; !(uiMask & 0x01); uiMask >>= 1)
				ucBShift++;
			for(; (uiMask & 0x01); uiMask >>= 1)
				ucBLoss--;
		}
	}

	// Convert from BGR to RGB
	unsigned char *ucTempData = new unsigned char[iImageSize];
	for( i = 0; i < (iWidth * iHeight); ++i)
	{
		unsigned char r, g, b;
		unsigned int uiPixelValue = 0;

		for(int j = iBytesPP - 1; j >=0; --j)
		{
			uiPixelValue <<= 8;
			uiPixelValue = uiPixelValue | pkImage->m_pucData[(i * iBytesPP) + j];
		}

		unsigned int uiRV, uiGV, uiBV;

		uiRV = (uiPixelValue & uiRMask) >> ucRShift;
		r = (uiRV << ucRLoss) + (uiRV >> (8 - ucRLoss));

		uiGV = (uiPixelValue & uiGMask) >> ucGShift;
		g = (uiGV << ucGLoss) + (uiGV >> (8 - ucGLoss));

		uiBV = (uiPixelValue & uiBMask) >> ucBShift;
		b = (uiBV << ucBLoss) + (uiBV >> (8 - ucBLoss));

		ucTempData[(i * 3) + 0] = r;
		ucTempData[(i * 3) + 1] = g;
		ucTempData[(i * 3) + 2] = b;
	}

	memset(pkImage->m_pucData, 0, iImageSize);
	fmemcpy(pkImage->m_pucData, ucTempData, iImageSize);
	delete []ucTempData;

	pkFile->Seekg( iOffset, ios_base::beg );
	pkFile->SetBinary( bBinary );
	
	return pkImage;
}


void ImageCodec::FreeImage(ImageData *pkImageData)
{
	if(pkImageData)
		delete pkImageData;
}


void ImageCodec::WriteImage( ImageData *pkImage, File *pkFile )
{
        neolog << "WriteImage(): BMP image writing not yet implemented." << endl;
}


                                                                                                            
                                                                                                            
#ifdef BUILD_STATIC


Link::Link()
{
#if defined(WIN32) && defined(_DEBUG)
	g_pkStaticModule = new ModuleStatic( "neoicbmp_debug" );
#else
	g_pkStaticModule = new ModuleStatic( "neoicbmp" );
#endif

	g_pkStaticModule->m_Symbols.Insert( "LoadImageCodec", (void**)LoadImageCodec );
	g_pkStaticModule->m_Symbols.Insert( "GetVersion",     (void**)GetVersion     );
	g_pkStaticModule->m_Symbols.Insert( "Initialize",     (void**)Initialize     );
	g_pkStaticModule->m_Symbols.Insert( "Shutdown",       (void**)Shutdown       );

	NeoEngine::Core::GetModuleManager()->InsertModule( g_pkStaticModule );
}


#endif


}; // namespace NeoBMP
