/***************************************************************************
                        neoicjpeg.cpp  -  description
                             -------------------
    begin                : Wed Apr 18 2002
    copyright            : (C) 2002 by Mattias Jansson
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoICJPEG, neoicjpeg.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "neoicjpeg.h"
#include "link.h"


//Stupid MSVC++ LoadImageA problems...

inline NeoEngine::ImageData *LoadImageFunction( NeoEngine::File *pkFile );


namespace NeoJPEG
{


NeoEngine::ImageData *ImageCodec::LoadImage( NeoEngine::File *pkFile ) { return LoadImageFunction( pkFile ); }


};



#ifdef WIN32
#  undef ERROR
#endif

#include <neoengine/file.h>
#include <neoengine/logstream.h>

#include <stdio.h>
#include <string.h>

#ifdef WIN32
#  undef WIN32_LEAN_AND_MEAN
#  undef NOGDI
#  include <windows.h>
#endif

extern "C"
{

#include "jpeglib.h"

}

using namespace std;
using namespace NeoEngine;


namespace NeoJPEG
{

#ifdef BUILD_STATIC
#  define JPEGEXPORT

ModuleStatic *g_pkStaticModule = 0;

#else

#  ifdef WIN32
#    define JPEGEXPORT __declspec(dllexport)
#  else
#    define JPEGEXPORT
#endif

extern "C"
{

#endif

	JPEGEXPORT ImageCodec *LoadImageCodec()
	{
		vector<string> vstrExtensions;

		vstrExtensions.push_back( "jpg" );
		vstrExtensions.push_back( "JPG" );
		vstrExtensions.push_back( "jpeg" );
		vstrExtensions.push_back( "JPEG" );

		ImageCodec *pkCodec = new NeoJPEG::ImageCodec( vstrExtensions );

		return pkCodec;
	}

	JPEGEXPORT int Initialize()
	{
		return 0;
	}

	JPEGEXPORT int Shutdown()
	{
#ifdef BUILD_STATIC
		delete g_pkStaticModule;
#endif
		return 0;
	}

	JPEGEXPORT void GetVersion( std::string *pstrName, int *piMajor, int *piMinor, int *piRevision )
	{
		*pstrName   = "NeoICJPEG";
		*piMajor    = NEOENGINEVERSION_MAJOR;
		*piMinor    = NEOENGINEVERSION_MINOR;
		*piRevision = NEOENGINEVERSION_REVISION;
	}

#ifdef BUILD_STATIC

#else

}

#endif

};




typedef struct
{
  struct jpeg_source_mgr         m_kPub;
  unsigned char                 *m_pucBuffer;
  int                            m_iSize;
} JPEGBufferSrcManager;


void BufferInitSource( j_decompress_ptr cinfo )
{
	JPEGBufferSrcManager *pSrc = (JPEGBufferSrcManager*)cinfo->src;

	pSrc->m_kPub.next_input_byte = (const JOCTET*)pSrc->m_pucBuffer;
	pSrc->m_kPub.bytes_in_buffer = pSrc->m_iSize;
}


boolean BufferFillInputBuffer( j_decompress_ptr cinfo )
{
	//Something bad happened
	return false;
}


void BufferSkipInputData( j_decompress_ptr cinfo, long num_bytes )
{
	JPEGBufferSrcManager *pSrc = (JPEGBufferSrcManager*)cinfo->src;

	if( num_bytes > 0 )
	{
		while( num_bytes > (long)pSrc->m_kPub.bytes_in_buffer )
		{
			/* We do not handle num_bytes > bytes_in_buffer yet */
			pSrc->m_kPub.bytes_in_buffer = 0;
			return;
		}

		pSrc->m_kPub.next_input_byte += (size_t)num_bytes;
		pSrc->m_kPub.bytes_in_buffer -= (size_t)num_bytes;
	}
}


void BufferTermSource( j_decompress_ptr cinfo )
{
	/* no work necessary here */
}


boolean BufferResyncToRestart( j_decompress_ptr cinfo, int desired )
{
	return jpeg_resync_to_restart( cinfo, desired ); /* use default method */
}





namespace NeoJPEG
{


ImageCodec::ImageCodec( const vector<string> &rvstrExtensions ) :
	NeoEngine::ImageCodec( "JPEG Image File", rvstrExtensions )
{
	neolog << LogLevel( INFO ) << "Loading JPEG image codec" << endl << "  Based on the libjpeg library, copyright (C) 1991-1998, Thomas G. Lane" << endl << "  For license details, see the README" << endl;
}


bool ImageCodec::IsType( File *pkFile )
{
	if( strstr( pkFile->GetName().c_str(), ".jpg" ) ||
	    strstr( pkFile->GetName().c_str(), ".JPG" ) ||
	    strstr( pkFile->GetName().c_str(), ".jpeg" ) ||
	    strstr( pkFile->GetName().c_str(), ".JPEG" ) )
		return true;

	return false;
}

void ImageCodec::FreeImage( ImageData *pkImageData )
{
	delete pkImageData;
}


void ImageCodec::WriteImage( ImageData *pkImageData, File *pkFile )
{
        neolog << LogLevel( ERROR ) << "WriteImage(): JPEG image writing is not yet implemented." << endl;
}


#ifdef BUILD_STATIC


ModuleStatic g_kStaticModule( "neoicjpeg" );


Link::Link()
{
#if defined(WIN32) && defined(_DEBUG)
	g_pkStaticModule = new ModuleStatic( "neoicjpeg_debug" );
#else
	g_pkStaticModule = new ModuleStatic( "neoicjpeg" );
#endif

	g_pkStaticModule->m_Symbols.Insert( "LoadImageCodec", (void**)LoadImageCodec );
	g_pkStaticModule->m_Symbols.Insert( "GetVersion",     (void**)GetVersion     );
	g_pkStaticModule->m_Symbols.Insert( "Initialize",     (void**)Initialize     );
	g_pkStaticModule->m_Symbols.Insert( "Shutdown",       (void**)Shutdown       );

	NeoEngine::Core::GetModuleManager()->InsertModule( g_pkStaticModule );
}


#endif


}; //Namespace NeoJPEG



ImageData *LoadImageFunction( File *pkFile )
{
	int i, j;
	int iWidth, iHeight;
	ImageData *pkImage = 0;

	/* This struct contains the JPEG decompression parameters and pointers to
	* working space (which is allocated as needed by the JPEG library).
	*/
	jpeg_decompress_struct cinfo;
	jpeg_error_mgr jerr;

	JPEGBufferSrcManager *pSrcManager = 0;

	/* More stuff */
	int iFileSize = pkFile->GetSize();
	unsigned char *pSrcBuffer = new unsigned char[ iFileSize ];
	unsigned char *pDestBuffer = 0;

	int row_stride;		/* physical row width in output buffer */

	//try
	//{

	if( !pkFile->Read( pSrcBuffer, iFileSize ) )
	{
		neolog << LogLevel( ERROR ) << "*** Unable to read JPEG file: unable to read file data [" << pkFile->GetName() << "]" << endl;
		return 0;
	}


	/* Step 1: allocate and initialize JPEG decompression object */

	/* We set up the normal JPEG error routines, then override error_exit. */
	cinfo.err = jpeg_std_error(&jerr);

	/* Now we can initialize the JPEG decompression object. */
	jpeg_create_decompress( &cinfo );

	cinfo.src = (struct jpeg_source_mgr *)( pSrcManager = new JPEGBufferSrcManager );

	pSrcManager->m_pucBuffer = pSrcBuffer;
	pSrcManager->m_iSize     = iFileSize;

	pSrcManager->m_kPub.init_source        = BufferInitSource;
	pSrcManager->m_kPub.fill_input_buffer  = BufferFillInputBuffer;
	pSrcManager->m_kPub.skip_input_data    = BufferSkipInputData;
	pSrcManager->m_kPub.resync_to_restart  = BufferResyncToRestart;
	pSrcManager->m_kPub.term_source        = BufferTermSource;


	/* Step 3: read file parameters with jpeg_read_header() */

	jpeg_read_header( &cinfo, TRUE );
	/* We can ignore the return value from jpeg_read_header since
	*   (a) suspension is not possible with the stdio data source, and
	*   (b) we passed TRUE to reject a tables-only JPEG file as an error.
	* See libjpeg.doc for more info.
	*/

	/* Step 4: set parameters for decompression */

	/* In this example, we don't need to change any of the defaults set by
	* jpeg_read_header(), so we do nothing here.
	*/

	/* Step 5: Start decompressor */
	jpeg_start_decompress( &cinfo );

	/* We can ignore the return value since suspension is not possible
	* with the stdio data source.
	*/

	/* We may need to do some setup of our own at this point before reading
	* the data.  After jpeg_start_decompress() we have the correct scaled
	* output image dimensions available, as well as the output colormap
	* if we asked for color quantization.
	* In this example, we need to make an output work buffer of the right size.
	*/
	/* JSAMPLEs per row in output buffer */
	row_stride = cinfo.output_width * cinfo.output_components;

	/* Make a one-row-high sample array that will go away when done with image */
	pDestBuffer = new unsigned char[ cinfo.output_width * cinfo.output_height * cinfo.output_components ];

	/* Step 6: while (scan lines remain to be read) */
	/*           jpeg_read_scanlines(...); */

	unsigned char *pDest;
	unsigned char **ppBuffer;

	/* Here we use the library's state variable cinfo.output_scanline as the
	* loop counter, so that we don't have to keep track ourselves.
	*/
	while( cinfo.output_scanline < cinfo.output_height )
	{
		/* jpeg_read_scanlines expects an array of pointers to scanlines.
		* Here the array is only one element long, but you could ask for
		* more than one scanline at a time if that's more convenient.
		*/

		pDest = pDestBuffer + cinfo.output_scanline * row_stride;
		ppBuffer = &pDest;

		(void) jpeg_read_scanlines(&cinfo, ppBuffer, 1);
	}

	/* Step 7: Finish decompression */

	(void) jpeg_finish_decompress(&cinfo);
	/* We can ignore the return value since suspension is not possible
	* with the stdio data source.
	*/

	pkImage = new ImageData;

	iWidth  = cinfo.output_width;
	iHeight = cinfo.output_height;

	pkImage->m_eFormat   = Texture::RGB;
	pkImage->m_iWidth    = iWidth;
	pkImage->m_iHeight   = iHeight;
	pkImage->m_iChannels = 3;
	pkImage->m_iBPP      = 24;
	pkImage->m_pucData   = new unsigned char[ iWidth * iHeight * 3 ];

	for( i = 0, j = 0; i < int( cinfo.output_width * cinfo.output_height * cinfo.output_components ); )
	{
		pkImage->m_pucData[j++] = pDestBuffer[i++];
		pkImage->m_pucData[j++] = pDestBuffer[i++];
		pkImage->m_pucData[j++] = pDestBuffer[i++];

		if( cinfo.output_components > 3 )
			++i;
	}

	//}
	//catch( ... )
	//{
	//	neolog << LogLevel( ERROR ) << "*** Unable to read JPEG file: exception caught [" << pkFile->GetName() << "]" << endl;
	//}

	delete [] pDestBuffer;
	delete [] pSrcBuffer;

	/* Step 8: Release JPEG decompression object */

	/* This is an important step since it will release a good deal of memory. */
	jpeg_destroy_decompress( &cinfo );


	/* At this point you may want to check to see whether any corrupt-data
	* warnings occurred (test whether jerr.pub.num_warnings is nonzero).
	*/

	/* And we're done! */

	delete pSrcManager;

	return pkImage;
}


