/***************************************************************************
               core.cpp  -  Lua scripting core object and services
                             -------------------
    begin                : Fri Jan 16 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoLua, core.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "core.h"
#include "script.h"
#include "bindings.h"
#include "wtkmsghook.h"
#include "chunktype.h"

#ifdef HAVE_NEOCHUNKIO
#  include <neochunkio/factory.h>
#endif

using namespace std;


namespace NeoLua
{


Core *Core::s_pkSingleton = 0;


Core::Core()
{
}


Core::~Core()
{
}


bool Core::Initialize()
{
	m_vpkBinders.push_back( new DefaultBinding );

#ifdef HAVE_NEOCHUNKIO
	
#  ifdef HAVE_NEOWTK

	NeoChunkIO::ChunkFactory::RegisterChunkType( ChunkType::WTKMSGHOOK, "luawtkmsghook", LuaMsgHookChunk::Allocator );

#  endif

#endif

	return true;
}


void Core::Shutdown()
{
	vector< BindingCallback* >::iterator ppkBinder    = m_vpkBinders.begin();
	vector< BindingCallback* >::iterator ppkBinderEnd = m_vpkBinders.end();

	for( ; ppkBinder != ppkBinderEnd; ++ppkBinder )
		delete( *ppkBinder );

	s_pkSingleton = 0;

	delete this;
}


Script *Core::CreateScript()
{
	Script *pkScript = new Script;

	vector< BindingCallback* >::iterator ppkBinder    = m_vpkBinders.begin();
	vector< BindingCallback* >::iterator ppkBinderEnd = m_vpkBinders.end();

	for( ; ppkBinder != ppkBinderEnd; ++ppkBinder )
		(*ppkBinder)->Bind( pkScript );

	lua_settop( pkScript->m_pkLua, 0 );

	return pkScript;
}


void Core::AddBindingCallback( BindingCallback *pkBinder )
{
	m_vpkBinders.push_back( pkBinder );
}


};

