/***************************************************************************
      wtkmsghook.h  -  Bindings for WTK message hook objects and chunks
                             -------------------
    begin                : Fri Jan 16 2004
    copyright            : (C) 2004 by Mattias Jansson
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoLua, wtkmsghook.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEOLUA_WTKMSGHOOK_H
#define __NEOLUA_WTKMSGHOOK_H

#include "base.h"
#include "lua/lunar.h"

/**
  * \file wtkmsghook.h
  * Bindings for WTK message hook objects and chunks
  */


#ifdef HAVE_NEOWTK

#include <neowtk/msghook.h>


namespace NeoLua
{


// External classes
class Script;


/**
  * \brief Interface for message hook procedures
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOLUA_API LuaMsgHookMethod : public NeoWTK::MsgHookMethod
{
	public:

		/*! Binding class name */
		static const char                             className[];

		/*! Binding methods */
		static Lunar< LuaMsgHookMethod >::RegType     methods[];


		/*! Lua script */
		Script                                       *m_pkScript;

		/*! Method to execute */
		std::string                                   m_strMethod;


		/**
		*/
		                                              LuaMsgHookMethod( lua_State *pkLua ) {}

		/**
		*/
		virtual                                      ~LuaMsgHookMethod();

		/**
		* Execute action
		* \param pkHook                               Hook object we are invoked by
		* \param pkMsg                                Message that triggered hook
		*/
		virtual void                                  ProcessHook( NeoWTK::MsgHook *pkHook, NeoWTK::Msg *pkMsg );
};


#ifdef HAVE_NEOCHUNKIO


/**
  * \brief Chunk for message hooks
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOLUA_API LuaMsgHookChunk : public NeoWTK::MsgHookChunk
{
	public:


		/**
		* Initialize chunk
		* \param usType                               Chunk type
		* \param rstrType                             Chunk type as string
		* \param rstrID                               Chunk ID string
		*/ 
		                                              LuaMsgHookChunk( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID = "" ) : NeoWTK::MsgHookChunk( usType, rstrType, rstrID ) {}
		
		/**
		* Deallocate data and subchunks
		*/
		virtual                                      ~LuaMsgHookChunk() {}

		/**
		* Parse chunk data
		* \param uiFlags                              Parse flags
		* \param pkFileManager                        File manager
		* \return                                     <0 if error, >0 if successful (0 reserved)
		*/
		virtual int                                   ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager );

		/**
		* Allocate new chunk
		* \param usType                               Type identifier
		* \param rstrType                             Type identifier as string
		* \param rstrID                               ID string
		* \return                                     New chunk
		*/
		static NeoChunkIO::Chunk                     *Allocator( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID ) { return new LuaMsgHookChunk( usType, rstrType, rstrID ); }
};


#endif


};


#endif


#endif

