/***************************************************************************
                  core.cpp  -  NET core object and services
                             -------------------
    begin                : Mon Jun 21 2004
    copyright            : (C) 2004 by emedia-solutions wolf
    email                : markus@emedia-solutions-wolf.de
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoNET, core.cpp

 The Initial Developer of the Original Code is Markus Wolf.
 Portions created by Markus Wolf are Copyright (C) 2004
 emedia-solutions wolf. All Rights Reserved.

 ***************************************************************************/

#include "neoengine/logstream.h"
#include "core.h"

#include <neoengine/tcp.h>

#include <string>
#include <vector>
#include <algorithm>

using namespace std;

namespace NeoNET
{


Core *Core::s_pkSingleton = 0;


Core::Core()
{
}


Core::~Core()
{
}


bool Core::Initialize()
{
	neolog << LogLevel( INFO ) << "Initializing NET extension" << std::endl;
	return true;
}


void Core::Shutdown()
{
	// delete all known sockets
	std::vector< Socket* >::iterator ppkSocket = m_vpkSockets.begin();
	std::vector< Socket* >::iterator ppkEnd = m_vpkSockets.end();
	for ( ; ppkSocket != ppkEnd; ++ppkSocket )
		delete *ppkSocket;
	m_vpkSockets.clear();

	// delete all registered messages
	std::vector< Message* > vpkMessages;
	m_hashMessageTable.GetAllNodeData( &vpkMessages );
	std::vector< Message* >::iterator ppkMessages = vpkMessages.begin();
	std::vector< Message* >::iterator ppkMessagesEnd = vpkMessages.end();
	for ( ; ppkMessages != ppkMessagesEnd; ++ppkMessages )
		delete *ppkMessages;
	vpkMessages.clear();

	s_pkSingleton = 0;
	delete this;
}


void Core::RegisterMessageType( Message *pkMessage )
{
	string strHash( "MID" );
	strHash += pkMessage->GetID();
	m_hashMessageTable.Insert( strHash, pkMessage );
}

void Core::AddSocket( Socket *pkSocket )
{
	std::vector< Socket* >::iterator ppkSocket = std::find( m_vpkSockets.begin(), m_vpkSockets.end(), pkSocket );

	if ( ppkSocket != m_vpkSockets.end() )
		return;

	m_vpkSockets.push_back( pkSocket );
}

Socket *Core::RemoveSocket( Socket *pkSocket )
{
	std::vector< Socket* >::iterator ppkSocket = std::find( m_vpkSockets.begin(), m_vpkSockets.end(), pkSocket );
 
	if ( ppkSocket != m_vpkSockets.end() )
	{
		Socket *pkTempSocket = *ppkSocket;
		m_vpkSockets.erase( ppkSocket );
		return pkTempSocket;
	}
		
	return 0;
}

void Core::Receive()
{
	std::vector< Socket* >::iterator ppkSocket = m_vpkSockets.begin();
	std::vector< Socket* >::iterator ppkEnd = m_vpkSockets.end();

	for ( ; ppkSocket != ppkEnd; ++ppkSocket )
	{
		TCPSocket *pkTCPSocket = dynamic_cast< TCPSocket* >( *ppkSocket );
		if ( pkTCPSocket )
		{
			// handle TCP communication
			unsigned int uiFlags;
			if ( ( uiFlags = pkTCPSocket->Poll( 0 ) ) == Socket::INVALID )
			{
				RemoveSocket( pkTCPSocket );
				pkTCPSocket->m_pkCallback->SocketEventCallback( pkTCPSocket, Socket::HUP , 0 );
				return;
			}
			if ( uiFlags & Socket::DATAIN )
			{
				// fetch ID & length
				unsigned int uiMessageID;
				unsigned int uiMessageLength;
				
				if ( pkTCPSocket->Read( &uiMessageID, sizeof( unsigned int ) ) <= 0 )
				{
					RemoveSocket( pkTCPSocket );
					pkTCPSocket->m_pkCallback->SocketEventCallback( pkTCPSocket, Socket::HUP , 0 );
					return;
				}
				if ( pkTCPSocket->Read( &uiMessageLength, sizeof( unsigned int ) ) <= 0 )
				{
					RemoveSocket( pkTCPSocket );
					pkTCPSocket->m_pkCallback->SocketEventCallback( pkTCPSocket, Socket::HUP , 0 );
					return;
				}
				string strHash( "MID" );
				strHash += uiMessageID;
				Message *pkMessageTemplate = m_hashMessageTable.Find( strHash );
				if ( !pkMessageTemplate )
				{
					NeoEngine::neolog << NeoEngine::LogLevel( NeoEngine::ERROR )<< "Message[" << uiMessageID << "] is not registered" << endl;
					return;
				}

				// fetch message body
				char *pucMessage;
				pucMessage = ( char* )malloc( uiMessageLength + 1 );
				if ( uiMessageLength > 0 )
				{
					pkTCPSocket->Read( pucMessage, uiMessageLength );
				}
				pucMessage[ uiMessageLength ] = 0;
				Message *pkMessage = pkMessageTemplate->Deserialize( string( pucMessage, uiMessageLength ) );
				free( pucMessage );
				
				// send as socket event
				pkTCPSocket->m_pkCallback->SocketEventCallback( pkTCPSocket, Socket::DATAIN , ( Message* )pkMessage );
				delete pkMessage, pkMessage = 0;
			}
		}
		else
		{
			// TODO: handle UDP communication
		}
	}
}

void Core::Send( Message *pkMessage, Socket* pkSocket )
{
	string strMessage = pkMessage->SerializeHeader() + pkMessage->Serialize();
	TCPSocket *pkTCPSocket = dynamic_cast< TCPSocket* >( pkSocket );
	if ( pkTCPSocket )
	{
		// handle TCP communication
		pkTCPSocket->Write( strMessage.c_str(), strMessage.length() );
	}
	else
	{
		// TODO: handle UDP communication
	}
}

void Core::Send( Message *pkMessage, std::vector< Socket* > vpkSockets )
{
	string strMessage = pkMessage->SerializeHeader() + pkMessage->Serialize();
	
	vector< Socket* >::iterator ppkSocket;
	vector< Socket* >::iterator ppkEnd;
	if ( vpkSockets.size() > 0 )
	{
		ppkSocket = vpkSockets.begin();
		ppkEnd = vpkSockets.end();
	}
	else
	{
		ppkSocket = m_vpkSockets.begin();
		ppkEnd = m_vpkSockets.end();
	}

	for ( ; ppkSocket != ppkEnd; ++ppkSocket )
	{
		TCPSocket *pkTCPSocket = dynamic_cast< TCPSocket* >( *ppkSocket );
		if ( pkTCPSocket )
		{
			// handle TCP communication
			pkTCPSocket->Write( strMessage.c_str(), strMessage.length() );
		}
		else
		{
			// TODO: handle UDP communication
		}
	}
}

};

