/***************************************************************************
                   core.h  -  NET core object and services
                             -------------------
    begin                : Mon Jun 21 2004
    copyright            : (C) 2004 by emedia-solutions wolf
    email                : markus@emedia-solutions-wolf.de
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoNET, core.h

 The Initial Developer of the Original Code is Markus Wolf.
 Portions created by Markus Wolf are Copyright (C) 2004
 emedia-solutions wolf. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEONET_CORE_H
#define __NEONET_CORE_H

#include "base.h"
#include "message.h"

#include <neoengine/socket.h>
#include <neoengine/hashtable.h>

using namespace NeoEngine;

namespace NeoNET
{

#ifdef WIN32
#  ifdef _MSC_VER
#    pragma warning( disable : 4231 )
#  endif
#  ifndef __HAVE_VECTOR_NENETCORE
     NetUDTVectorEXPIMP( class Socket* );
#    define __HAVE_VECTOR_NENETCORE
#  endif
#endif

/**
  * \brief Core NET object
  * \author Markus Wolf (markus@emedia-solutions-wolf.de)
  */
class NEONET_API Core
{
	protected:

		/*! Singleton object */
		static Core                                  *s_pkSingleton;

		/*! List of all added sockets */
		std::vector< Socket* >                        m_vpkSockets;

		/*! Table with all registered message types */
		HashTable< Message >                          m_hashMessageTable;

		/**
		* Prevent new objects to be created
		*/
		                                              Core();

	public:

		/**
		*/
		                                             ~Core();


		/**
		* Initialize NET core
		* \return                                     true if successful, false if not
		*/
		bool                                          Initialize();

		/**
		* Terminate NET, delete all sockets and deallocates core object
		*/
		void                                          Shutdown();

		/**
		* Get singleton object
		* \return                                     NET core object
		*/
		inline static Core                           *Get() { if( !s_pkSingleton ) s_pkSingleton = new Core; return s_pkSingleton; }

		/**
		* Registers a new message type in the core object
		* \param rkMessage                            Message template to be registered
		*/
		void                                          RegisterMessageType( Message *pkMessage );

		/**
		* Adds a new socket for handling to the core
		* \param pkSocket                             Socket to handle
		*/
		void                                          AddSocket( Socket *pkSocket );

		/**
		* Stops handling of the given socket (but does not delete it)
		* \param pkSocket                             Socket to remove
		* \return                                     Pointer to the removed socket
		*/
		Socket                                       *RemoveSocket( Socket *pkSocket );

		/**
		* Polls the added sockets for registered messages
		*/
		void                                          Receive();
		
		/**
		*	Sends the given message to the given socket
		* \param pkMessage                            Message to send
		*	\param pkSocket                             Socket to send to
		*/
		void                                          Send( Message *pkMessage, Socket* pkSocket );

		/**
		*	Sends the given message to multiple sockets
		* \param pkMessage                            Message to send
		*	\param vpkSockets                           Sockets to send to (defaults to all known sockets if empty)
		*/
		void                                          Send( Message *pkMessage, std::vector< Socket* > vpkSockets = std::vector< Socket* >() );
};

};


#endif
