/***************************************************************************
              message.h  -  Abstract base class for all messages
                             -------------------
    begin                : Mon Jun 21 2004
    copyright            : (C) 2004 by Markus Wolf
    email                : markus@emedia-solutions-wolf.de
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoNET, message.h

 The Initial Developer of the Original Code is Markus Wolf.
 Portions created by Markus Wolf are Copyright (C) 2004
 emedia-solutions wolf. All Rights Reserved.

 ***************************************************************************/
#ifndef _NEONET_MESSAGE_H
#define _NEONET_MESSAGE_H

#include "base.h"

#include <neoengine/hashtable.h>

#include <string>

namespace NeoNET {

// HashTable DLL Export
#if defined( WIN32 ) && defined( _MSC_VER ) && !defined( BUILD_STATIC )
  class Message;
  NEONETEXPIMP_TEMPLATE template class NEONET_API NeoEngine::HashTable< Message >;
#endif

/**
  * \brief Base Message class
  * \author Markus Wolf (markus@emedia-solutions-wolf.de)
 */
class NEONET_API Message 
{
	
	public:
		/*! ID must be unique  */
		unsigned int                                  m_uiMessageID;

		/*! Messgage length in bytes without header (=> minus 2 sizeof(unsigned int) */
		unsigned int                                  m_uiMessageLength;

	public:
		/**
		* Message constructor
		*/
		                                              Message( unsigned int uiMessageID, unsigned int uiMessageLength = 0 );

		/**
		* Message destructor
		*/
		virtual                                      ~Message();

		/**
		* Returns the message ID
		* \return                                     Message ID
		*/
		unsigned int                                  GetID() { return m_uiMessageID; }

		/**
		* Returns the message length
		* \return                                     The message length
		*/
		unsigned int                                  GetMessageLength() { return m_uiMessageLength; }

		/**
		* Serializes this message to be send
		* \return                                     Serialized representation of this message
		*/
		virtual std::string                           Serialize() = 0;

		/**
		* Deserializes the message from the given string
		* \param strMessage                           message to be deserialized
		* \return                                     Resulting message object
		*/
		virtual Message                              *Deserialize( std::string strMessage ) = 0;

		/**
		* Serializes this base message as header
		* \return                                     Serialized representation of this message
		*/
		std::string                                   SerializeHeader();
};

};

#endif
