/***************************************************************************
                  attribute.h  -  Abstraction of object data
                             -------------------
    begin                : Tue Dec 17 2002
    copyright            : (C) 2002 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoWTK, attribute.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef __NEOWTKATTRIBUTE_H
#define __NEOWTKATTRIBUTE_H


#include "base.h"
#include "msg.h"

#ifdef HAVE_NEOCHUNKIO
#  include <neochunkio/complex.h>
#  include "chunktype.h"
#endif

#include <neoengine/hashstring.h>


/**
  * \file attribute.h
  * An abstraction of data for an object
  */


namespace NeoWTK
{


/**
  * \class AttributeBase
  * \brief Base class for attribute management
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOWTK_API AttributeBase
{
	public:

		/*! Attribute name */
		NeoEngine::HashString                         m_strName;


		/**
		*/
		                                              AttributeBase( const NeoEngine::HashString &rstrName ) : m_strName( rstrName ) {}

		/**
		*/
		virtual                                      ~AttributeBase() {}
};


/**
  * \class Attribute
  * \brief Abstraction of data for an object
  * \author Mattias Jansson (mattias@realityrift.com)
  */	
template <class T> class Attribute : public AttributeBase
{
	public:

		/*! Attribute data */
		T                                             m_kData;



		/**
		* \param rstrName                             Name
		* \param rkData                               Data
		*/
		                                              Attribute( const NeoEngine::HashString &rstrName, const T &rkData ) : AttributeBase( rstrName ), m_kData( rkData ) {}

		/**
		*/
		virtual                                      ~Attribute() {}
};


#ifdef HAVE_NEOCHUNKIO


/**
  * \class AttributeChunk
  * \brief Attribute data chunk
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOWTK_API AttributeChunk : public NeoChunkIO::ComplexChunk
{
	public:

		/*! Object */
		AttributeBase                                *m_pkData;


		/**
		* Initialize chunk
		* \param usType                               Chunk type
		* \param rstrType                             Chunk type as string
		* \param rstrID                               Chunk ID string
		*/ 
		                                              AttributeChunk( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID = "" ) : NeoChunkIO::ComplexChunk( usType, rstrType, rstrID ), m_pkData( 0 ) {}
		
		/**
		* Deallocate data and subchunks
		*/
		virtual                                      ~AttributeChunk();

		/**
		* Parse chunk data
		* \param uiFlags                              Parse flags
		* \param pkFileManager                        File manager
		* \return                                     <0 if error, >0 if successful (0 reserved)
		*/
		virtual int                                   ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager );

		/**
		* Allocate new chunk
		* \param usType                               Type identifier
		* \param rstrType                             Type identifier as string
		* \param rstrID                               ID string
		* \return                                     New chunk
		*/
		static NeoChunkIO::Chunk                     *Allocator( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID ) { return new AttributeChunk( usType, rstrType, rstrID ); }
};


#endif /*! HAVE_NEOCHUNKIO */


}; /*! namespace NeoWTK */


#endif
