/***************************************************************************
                   borderarea.cpp  -  Object with border areas
                             -------------------
    begin                : Sun Mar 2 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoWTK, borderarea.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2003
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "borderarea.h"
#include "area.h"

#include <string>

#ifdef HAVE_NEOCHUNKIO
#  include <neochunkio/material.h>
#  include <neochunkio/bool.h>

using namespace NeoChunkIO;

#endif

using namespace std;
using namespace NeoEngine;


namespace NeoWTK
{


BorderArea::BorderArea( Object *pkParent, Area *pkCenter, BorderArea *pkObject, bool bCreateAreas ) :
	Object( pkParent, pkObject, false )
{
	if( bCreateAreas )
	{
		//Create child objects
		if( !pkObject )
		{
			( new Area( this ) )->SetName( "TopLeftCorner" );
			( new Area( this ) )->SetName( "TopRightCorner" );
			( new Area( this ) )->SetName( "BottomLeftCorner" );
			( new Area( this ) )->SetName( "BottomRightCorner" );
			( new Area( this ) )->SetName( "TopBorder" );
			( new Area( this ) )->SetName( "LeftBorder" );
			( new Area( this ) )->SetName( "RightBorder" );
			( new Area( this ) )->SetName( "BottomBorder" );
		}
		else
		{
			assert( !m_vpkChildren.size() );

			pkObject->GetArea( BorderArea::TOP_LEFT_CORNER     )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::TOP_RIGHT_CORNER    )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::BOTTOM_LEFT_CORNER  )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::BOTTOM_RIGHT_CORNER )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::TOP_BORDER          )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::LEFT_BORDER         )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::RIGHT_BORDER        )->Duplicate( this, 0 );
			pkObject->GetArea( BorderArea::BOTTOM_BORDER       )->Duplicate( this, 0 );
		}

		if( pkCenter )
			AttachObject( pkCenter );
		else if( pkObject )
			pkObject->GetArea( BorderArea::CENTER )->Duplicate( this, 0 );
		else
			( new Area( this ) )->SetName( "Center" );
	}

	if( pkObject )
		SetMaterial( pkObject->m_pkMaterial );
}


BorderArea::~BorderArea()
{
}


void BorderArea::SetMaterial( const MaterialPtr &pkMat )
{
	m_pkMaterial = pkMat;

	if( m_vpkChildren.size() )
		for( int i = 0; i < NUMAREAS; ++i )
			static_cast<Area*>( m_vpkChildren[ i ] )->SetBackground( m_pkMaterial );
}


const Coord &BorderArea::SetSize( const Coord &rkSize )
{
	const Coord &rkRet = Object::SetSize( rkSize );

	RecalcAreas();

	return rkRet;
}


Area *BorderArea::GetArea( BORDERAREA eArea )
{
	return static_cast<Area*>( m_vpkChildren[ eArea ] );
}


bool BorderArea::Render( Frustum *pkFrustum, bool bForce )
{
	return Object::Render( pkFrustum, bForce );
}


void BorderArea::RecalcAreas()
{
	Coord kSize      = GetSize();

	//Left, right
	int aiBorderWidth[2]  = { m_vpkChildren[ TOP_LEFT_CORNER ]->GetSize().x, m_vpkChildren[ BOTTOM_RIGHT_CORNER ]->GetSize().x };

	//Top, bottom
	int aiBorderHeight[2] = { m_vpkChildren[ TOP_LEFT_CORNER ]->GetSize().y, m_vpkChildren[ BOTTOM_RIGHT_CORNER ]->GetSize().y };


	//Top left corner
	m_vpkChildren[ TOP_LEFT_CORNER ]->SetPosition( Coord( 0, 0 ) );
	m_vpkChildren[ TOP_LEFT_CORNER ]->SetSize( Coord( aiBorderWidth[0], aiBorderHeight[0] ) );


	//Top right corner
	m_vpkChildren[ TOP_RIGHT_CORNER ]->SetPosition( Coord( kSize.x - aiBorderWidth[1], 0 ) );
	m_vpkChildren[ TOP_RIGHT_CORNER ]->SetSize( Coord( aiBorderWidth[1], aiBorderHeight[0] ) );


	//Top border
	m_vpkChildren[ TOP_BORDER ]->SetPosition( Coord( aiBorderWidth[0], 0 ) );
	m_vpkChildren[ TOP_BORDER ]->SetSize( Coord( kSize.x - ( aiBorderWidth[0] + aiBorderWidth[1] ), aiBorderHeight[0] ) );


	//Bottom left corner		
	m_vpkChildren[ BOTTOM_LEFT_CORNER ]->SetPosition( Coord( 0, kSize.y - aiBorderHeight[1] ) );
	m_vpkChildren[ BOTTOM_LEFT_CORNER ]->SetSize( Coord( aiBorderWidth[0], aiBorderHeight[1] ) );


	//Bottom right corner
	m_vpkChildren[ BOTTOM_RIGHT_CORNER ]->SetPosition( Coord( kSize.x - aiBorderWidth[1], kSize.y - aiBorderHeight[1] ) );
	m_vpkChildren[ BOTTOM_RIGHT_CORNER ]->SetSize( Coord( aiBorderWidth[1], aiBorderHeight[1] ) );


	//Bottom border
	m_vpkChildren[ BOTTOM_BORDER ]->SetPosition( Coord( aiBorderWidth[0], kSize.y - aiBorderHeight[1] ) );
	m_vpkChildren[ BOTTOM_BORDER ]->SetSize( Coord( kSize.x - ( aiBorderWidth[0] + aiBorderWidth[1] ), aiBorderHeight[1] ) );


	//Left border
	m_vpkChildren[ LEFT_BORDER ]->SetPosition( Coord( 0, aiBorderHeight[0] ) );
	m_vpkChildren[ LEFT_BORDER ]->SetSize( Coord( aiBorderWidth[0], kSize.y - ( aiBorderHeight[0] + aiBorderHeight[1] ) ) );


	//Right border
	m_vpkChildren[ RIGHT_BORDER ]->SetPosition( Coord( kSize.x - aiBorderWidth[1], aiBorderHeight[0] ) );
	m_vpkChildren[ RIGHT_BORDER ]->SetSize( Coord( aiBorderWidth[1], kSize.y - ( aiBorderHeight[0] + aiBorderHeight[1] ) ) );


	//Center
	m_vpkChildren[ CENTER ]->SetPosition( Coord( aiBorderWidth[0], aiBorderHeight[0] ) );
	m_vpkChildren[ CENTER ]->SetSize( Coord( kSize.x - ( aiBorderWidth[0] + aiBorderWidth[1] ), kSize.y - ( aiBorderHeight[0] + aiBorderHeight[1] ) ) );
}


void BorderArea::SetCenterArea( Area *pkCenter )
{
	if( m_vpkChildren[ CENTER ] )
	{
		//Reset pointer in vector to make sure it's not erased by detaching call
		Object *pkOldCenter = m_vpkChildren[ CENTER ];

		m_vpkChildren[ CENTER ] = 0;

		//Pointer is no longer in vector, safe to call delete (DetachNode call will be ignored)
		delete pkOldCenter;
	}

	m_vpkChildren[ CENTER ] = pkCenter;

	//Since it is already in vector, safe to call AttachToObject
	pkCenter->AttachToObject( this );
}


Object *BorderArea::Duplicate( Object *pkParent, Object *pkObject )
{
	if( !pkObject )
		pkObject = new BorderArea( pkParent, 0, this );
	else
		dynamic_cast< BorderArea* >( pkObject )->SetMaterial( m_pkMaterial );
	
	return Object::Duplicate( pkParent, pkObject );
}


#ifdef HAVE_NEOCHUNKIO


int BorderAreaChunk::ParseData( unsigned int uiFlags, FileManager *pkFileManager )
{
	Chunk *pkChunk = 0;
	int    iRet    = 0;

	if( !m_pkObject )
		m_pkObject = new BorderArea( 0, 0, 0, false );

	BorderArea *pkArea = dynamic_cast< BorderArea* >( m_pkObject );

	if( ( iRet = ObjectChunk::ParseData( uiFlags | ObjectChunk::NOCHILDREN | ObjectChunk::NOATTRIBUTES, pkFileManager ) ) <= 0 )
		return iRet;

	{
		string astrChunkName[] = {
			"TopLeftCorner",
			"TopRightCorner",
			"BottomLeftCorner",
			"BottomRightCorner",
			"TopBorder",
			"LeftBorder",
			"RightBorder",
			"BottomBorder",
			"Center"
		};

		for( int iArea = 0; iArea < BorderArea::NUMAREAS; ++iArea )
		{
			if( ( pkChunk = FindChunk( astrChunkName[ iArea ], ChunkType::OBJECT, -1 ) ) )
			{
				ObjectChunk *pkObjChunk = dynamic_cast< AreaChunk* >( pkChunk );

				pkObjChunk->m_pkObject->SetName( astrChunkName[ iArea ] );

				pkArea->AttachObject( pkObjChunk->m_pkObject );

				pkObjChunk->m_pkObject = 0;
			}
		}
	}

	if( !( uiFlags & ObjectChunk::NOCHILDREN ) )
		AttachChildren();

	if( !( uiFlags & ObjectChunk::NOATTRIBUTES ) )
		SetAttributes();

	//Look for overriding default material
	if( ( pkChunk = FindChunk( "", NeoChunkIO::ChunkType::MATERIAL, -1 ) ) )
		pkArea->SetMaterial( dynamic_cast< MaterialChunk* >( pkChunk )->m_pkMaterial );

	if( !( pkChunk = FindChunk( "recalc", NeoChunkIO::ChunkType::BOOL, -1 ) ) || dynamic_cast< BoolChunk* >( pkChunk )->m_bData )
		pkArea->RecalcAreas();

	return 1;
}


#endif


}; /*! namespace NeoWTK */
