/***************************************************************************
                 chunktype.h  -  Builtin chunk type identifiers
                             -------------------
    begin                : Wed May 28 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoWTK, chunktype.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef _NEOWTK_CHUNKTYPE_H_
#define _NEOWTK_CHUNKTYPE_H_


/**
  * \file neowtk/chunktype.h
  * Builtin chunk type identifiers
  */

  
#include "base.h"


namespace NeoWTK
{


/**
  * \brief Chunk type identifier class with builtin chunk type identifiers
  * \author Mattias Jansson (mattias@realityrift.com)
  **/
class NEOWTK_API ChunkType
{
	public:

		/**
		* \enum CHUNKTYPE
		* \brief Builtin chunk type identifiers and string identifiers
		*/
		enum CHUNKTYPE
		{
		  /*! First reserved chunk type ID */
		  FIRSTBUILTINCHUNKTYPE                             = 0x3A00,

		  /*! Generic object, all derived wtk object chunks should use this for search type */
		  OBJECT                                            = 0x3A00,  // "wtkobject"

		  /*! Area object */
		  AREA                                              = 0x3A01,  // "wtkarea"

		  /*! Borderarea object */
		  BORDERAREA                                        = 0x3A02,  // "wtkborderarea"

		  /*! Button object */
		  BUTTON                                            = 0x3A03,  // "wtkbutton"

		  /*! Editbox object */
		  EDITBOX                                           = 0x3A04,  // "wtkeditbox"

		  /*! Textarea object */
		  TEXTAREA                                          = 0x3A05,  // "wtktextarea"

		  /*! Coordinate */
		  COORD                                             = 0x3B00,  // "wtkcoord"

		  /*! Attribute */
		  ATTRIBUTE                                         = 0x3B01,  // "wtkattribute"

		  /*! Message hook */
		  MSGHOOK                                           = 0x3B02,  // NOT SUPPORTED DIRECTLY, ONLY IDENTIFIER FOR SEARCHES

		  // Chunk ID 0x3C00 to 0x3CFF reserved for lua bindings

		  /* Widget library reference (include) */
		  LIBREF                                            = 0x3F00,  // "wtklibref"

		  /*! Last reserved chunk type ID */
		  LASTBUILTINCHUNKTYPE                              = 0x3FFF
		};
};


};


#endif

