/***************************************************************************
                  core.cpp  -  WTK core object and services
                             -------------------
    begin                : Tue Jan 20 2004
    copyright            : (C) 2004 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoWTK, core.cpp

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2004
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#include "neoengine/logstream.h"
#include "core.h"

#ifdef HAVE_NEOCHUNKIO
#  include <neochunkio/factory.h>
#  include "chunktype.h"
#  include "object.h"
#  include "area.h"
#  include "borderarea.h"
#  include "button.h"
#  include "editbox.h"
#  include "textarea.h"
#  include "attribute.h"
#  include "widgetlib.h"

using namespace NeoChunkIO;

#endif

using namespace NeoEngine;


namespace NeoWTK
{


Core *Core::s_pkSingleton = 0;


Core::Core()
{
}


Core::~Core()
{
}


bool Core::Initialize()
{
	neolog << LogLevel( INFO ) << "Initializing window toolkit extension" << std::endl;

#ifdef HAVE_NEOCHUNKIO

	neolog << "  ... registering default WTK chunk types" << std::endl;

	//Register all default types
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::OBJECT,          "wtkobject",             ObjectChunk::Allocator            );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::AREA,            "wtkarea",               AreaChunk::Allocator              );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::BORDERAREA,      "wtkborderarea",         BorderAreaChunk::Allocator        );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::BUTTON,          "wtkbutton",             ButtonChunk::Allocator            );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::EDITBOX,         "wtkeditbox",            EditBoxChunk::Allocator           );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::TEXTAREA,        "wtktextarea",           TextAreaChunk::Allocator          );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::COORD,           "wtkcoord",              CoordChunk::Allocator             );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::ATTRIBUTE,       "wtkattribute",          AttributeChunk::Allocator         );
	ChunkFactory::RegisterChunkType( NeoWTK::ChunkType::LIBREF,          "wtklibref",             LibRefChunk::Allocator            );

#endif

	return true;
}


void Core::Shutdown()
{
	s_pkSingleton = 0;

	delete this;
}


};

