/***************************************************************************
                    wdigetlib.h  -  Widget library file loader
                             -------------------
    begin                : Tue Jul 1 2003
    copyright            : (C) 2003 by Reality Rift Studios
    email                : mattias@realityrift.com
 ***************************************************************************

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at 
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is the NeoEngine, NeoWTK, widgetlib.h

 The Initial Developer of the Original Code is Mattias Jansson.
 Portions created by Mattias Jansson are Copyright (C) 2002
 Reality Rift Studios. All Rights Reserved.

 ***************************************************************************/

#ifndef _NECHUNKIO_WIDGETLIB_H_
#define _NECHUNKIO_WIDGETLIB_H_


/**
  * \file widgetlib.h
  * Widget library file loader
  */
  

#include "base.h"
#include "object.h"

#include <neoengine/loadableentity.h>

#include <vector>


#ifdef HAVE_NEOCHUNKIO


namespace NeoWTK
{


#ifdef WIN32
#  ifdef _MSC_VER
#    pragma warning( disable : 4231 )
#  endif
#  ifndef __HAVE_VECTOR_NEWTKOBJECT
     WtkUDTVectorEXPIMP( class Object* );
#    define __HAVE_VECTOR_NEWTKOBJECT
#  endif
#endif


/**
  * \class WidgetLibrary
  * \brief Widget library
  * \author Mattias Jansson (mattias@realityrift.com)
  **/
class NEOWTK_API WidgetLibrary : public NeoEngine::LoadableEntity, public Object
{
	protected:

		/**
		* Load widget library file. Called by NeLoadableEntity to load object if file was opened successfully
		* \param uiFlags                              Flags passed by caller to Load() method
		* \return                                     true if load was successful, false otherwise
		*/
		virtual bool                                  LoadNode( unsigned int uiFlags = 0 );


	public:

		/*! Used during loading to resolve references */
		static Object                                *s_pkRefRoot;


		/**
		* Initialize library for loading
		* \param pkFileManager                        File manager, will use engine core file manager if null (default)
		*/
		                                              WidgetLibrary( NeoEngine::FileManager *pkFileManager = 0 );

		/**
		*/
		virtual                                      ~WidgetLibrary();
};


/**
  * \class LibRefChunk
  * \brief Widget library reference (include) chunk
  * \author Mattias Jansson (mattias@realityrift.com)
  */
class NEOWTK_API LibRefChunk : public NeoChunkIO::ComplexChunk
{
	public:

		/**
		* Initialize chunk
		* \param usType                               Chunk type
		* \param rstrType                             Chunk type as string
		* \param rstrID                               Chunk ID string
		*/ 
		                                              LibRefChunk( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID = "" ) : NeoChunkIO::ComplexChunk( usType, rstrType, rstrID ) {}
		
		/**
		* Deallocate data and subchunks
		*/
		virtual                                      ~LibRefChunk() {}

		/**
		* Parse chunk data
		* \param uiFlags                              Parse flags
		* \param pkFileManager                        File manager
		* \return                                     <0 if error, >0 if successful (0 reserved)
		*/
		virtual int                                   ParseData( unsigned int uiFlags, NeoEngine::FileManager *pkFileManager );

		/**
		* Allocate new chunk
		* \param usType                               Type identifier
		* \param rstrType                             Type identifier as string
		* \param rstrID                               ID string
		* \return                                     New chunk
		*/
		static NeoChunkIO::Chunk                     *Allocator( unsigned short usType, const NeoEngine::HashString &rstrType, const NeoEngine::HashString &rstrID ) { return new LibRefChunk( usType, rstrType, rstrID ); }
};


};


#endif


#endif

